package com.incubator.center.listener;

import com.incubator.center.GCenter;
import com.incubator.center.base.GameController;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.core.net.handler.listener.GameServerConnectionListener;
import com.incubator.center.manager.GameManager;
import com.incubator.core.pool.WSRequestPool;
import com.incubator.core.pool.WSResponsePool;
import com.incubator.message.proto.CommonProto;
import io.netty.buffer.ByteBuf;

import java.util.HashMap;
import java.util.Map;

public class CenterInnerListener extends GameServerConnectionListener {

	@Override
	public void connectionOpened(Connection connection) {
		connection.setAttribute(HEART_CONFIG, new HeartConfig());
		//关闭心跳
		setHeartEnable(GCenter.heartEnable);
		Log.info("连接已打开: {}", connection);
	}

	@Override
	public void connectionClosed(Connection connection) {
		try {
			GameController game = GameManager.getInstance().find(connection);
			if (game != null) {
				Log.info("游戏服登出ID : {}", game.getInfo().getServerId());
				GameManager.getInstance().remove(game.getInfo().getServerId());
			}
		} catch (Exception e) {
			Log.error("游戏服登出处理时出错", e);
		} finally {
			if (connection != null) {
				connection.close();
			}
			Log.info("连接已关闭: {}", connection);
		}
	}

	@Override
	public void messageArrived(Connection conn, Object msg) {
		if (msg instanceof ByteBuf) {
			ByteBuf byteBuf = (ByteBuf) msg;

			byte[] bytes = null;
			WSRequest request = null;
			WSResponse response = null;

			try {
				// 减少对象分配，通过 ByteBuf 的内置方法获取直接内存访问
				int readableBytes = byteBuf.readableBytes();
				if (readableBytes > 0) {
					bytes = new byte[readableBytes];
					byteBuf.getBytes(byteBuf.readerIndex(), bytes); // 避免修改 ByteBuf 的读取指针
				} else {
					Log.error("收到空消息，断开连接");
					conn.close();
					return;
				}

				// 使用对象池或预先初始化的对象避免频繁创建
				request = WSRequestPool.getRequest(bytes); // 从对象池获取 WSRequest 实例

				// 心跳消息特殊处理
				if (request.getCmd() == CommonProto.Cmd.HeartBeatReq_VALUE) {
					this.processHeartbeat(conn);
					return;
				}

				// 获取处理器
				NetHandler handler = GCenter.handlers.get(request.getCmd());
				if (handler == null) {
					Log.error("消息号错误... cmd={}", request.getCmd());
					conn.close();
					return;
				}

				// 使用池化的 WSResponse 实例，避免频繁创建
				response = WSResponsePool.getResponse();

				// 处理非心跳消息
				Log.info("收到消息 : cmd={}, 请求内容 : {}", request.getCmd(), request.toJson());
				handler.onDate(conn, request, response);
				conn.writeAndFlush(response.toBytes());
				Log.info("返回消息 : cmd={}, 返回内容 : {}", request.getCmd(), response.toJson());
			} catch (Exception e) {
				Log.error("处理消息时发生错误 : ", e);
				conn.close();
			} finally {
				// 回收池化对象，避免内存泄漏
				if (request != null) WSRequestPool.recycle(request);
				if (response != null) WSResponsePool.recycle(response);
			}
		}
	}

	private void processHeartbeat(Connection conn) throws Exception {
		// 设置最后心跳时间
		conn.setAttribute("LastReadTimeMillis", System.currentTimeMillis());
		// 设置心跳
		resetHeartSeq(conn);
		// 发送心跳响应
		this.doSendHeartRsp(conn);
	}

	@Override
	public boolean doIsHeartReq(Object msg) {
		if (!(msg instanceof ByteBuf)) {
			return false;
		}

		try {
			ByteBuf byteBuf = (ByteBuf) msg;
			byte[] bytes = new byte[byteBuf.readableBytes()];
			byteBuf.getBytes(byteBuf.readerIndex(), bytes);

			WSRequest request = new WSRequest(bytes);
			return request.getCmd() == CommonProto.Cmd.HeartBeatReq_VALUE;
		} catch (Exception e) {
			Log.error("心跳请求解析失败: ", e);
			return false;
		}
	}

	@Override
	public void doSendHeartReq(Connection conn) {
		try {
			long now = System.currentTimeMillis();
			long lastTime = (long) conn.getAttribute(LAST_READ_TIME_MILLIS);
			if (now - lastTime > 10000L) {
//                Log.info("玩家网络断了：" + conn.getUid());
//                conn.close();
			} else {
//                HeartbeatPush push = new HeartbeatPush();
//                push.setTime(System.currentTimeMillis());
//                Message msg = new Message(ProtoCommon.MsgId.S2C_Heartbeat, push.toJson());
//                msg.setCode(ProtoCommon.Code.OK_VALUE);
//                conn.writeAndFlush(super.packMessage(msg));
			}
		} catch (Exception e) {
			Log.info("发送心跳请求出错：{}, {}", conn.getPlayerId(), e.getMessage());
		}
	}

	@Override
	public void doSendHeartRsp(Connection connection) {
		// 发送心跳回复
		try {
			Map<String, Object> data = new HashMap<>();
			data.put("currentTimeMillis", System.currentTimeMillis());
			connection.writeAndFlush(new WSResponse(CommonProto.Cmd.HeartBeatRes, data).toBytes());
		} catch (Exception e) {
			Log.info("发送心跳回复出错：{}, {}", connection.getPlayerId(), e.getMessage());
		}
	}
}
