package com.incubator.center.manager;

import com.incubator.center.base.ConnectionKey;
import com.incubator.center.base.GameController;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.MsgBase;
import io.netty.handler.codec.http.websocketx.TextWebSocketFrame;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.locks.ReentrantReadWriteLock;

/**
 * 服务器管理
 */
public class GameManager {

	private static GameManager instance;

	private static final Object lock0 = new Object();

	private final ReentrantReadWriteLock rwLock = new ReentrantReadWriteLock();

	private final Map<Integer, GameController> controllers = new ConcurrentHashMap<Integer, GameController>();

	private GameManager() {

	}

	public static GameManager getInstance() {
		if (instance == null) {
			synchronized (lock0) {
				instance = new GameManager();
			}
		}
		return instance;
	}

	public boolean has(int serverId) {
		rwLock.readLock().lock();
		try {
			return controllers.get(serverId) != null;
		} catch (Exception e) {
			Log.error(e.getMessage(), e);
		} finally {
			rwLock.readLock().unlock();
		}
		return false;
	}

	/**
	 * 注册游戏服务器
	 * 
	 * @param info
	 * @param conn
	 */
	public void register(ServerInfo info, Connection conn) {
		if (info == null) {
			return;
		}
		rwLock.writeLock().lock();
		try {
			conn.setAttribute(ConnectionKey.GAME_SERVER_ID, info.getServerId());
			controllers.put(info.getServerId(), new GameController(info, conn));
		} catch (Exception e) {
			Log.error(e.getMessage(), e);
		} finally {
			rwLock.writeLock().unlock();
		}
	}

	/**
	 * 移除游戏服务器
	 * 
	 * @param serverId
	 */
	public void remove(int serverId) {
		rwLock.writeLock().lock();
		try {
			controllers.remove(serverId);
		} catch (Exception e) {
			Log.error(e.getMessage(), e);
		} finally {
			rwLock.writeLock().unlock();
		}
	}

	/**
	 * 如果是游戏连接则返回游戏服务器信息
	 * 
	 * @param conn
	 * @return
	 */
	public GameController find(Connection conn) {
		rwLock.readLock().lock();
		try {
			if (conn.hasAttribute(ConnectionKey.GAME_SERVER_ID)) {
				int serverId = (int) conn.getAttribute(ConnectionKey.GAME_SERVER_ID);
				return controllers.get(serverId);
			}
		} catch (Exception e) {
			Log.error(e.getMessage(), e);
		} finally {
			rwLock.readLock().unlock();
		}
		return null;
	}

	/**
	 * 如果是游戏连接则返回游戏服务器信息
	 * 
	 * @param serverId
	 * @return
	 */
	public GameController get(int serverId) {
		rwLock.readLock().lock();
		try {
			return controllers.get(serverId);
		} catch (Exception e) {
			Log.error(e.getMessage(), e);
		} finally {
			rwLock.readLock().unlock();
		}
		return null;
	}

	public Collection<GameController> getList() {
		rwLock.readLock().lock();
		try {
			return new ArrayList<GameController>(controllers.values());
		} catch (Exception e) {
			Log.error(e.getMessage(), e);
		} finally {
			rwLock.readLock().unlock();
		}
		return null;
	}
	
	/**
	 * 广播消息特定的游戏服
	 * 
	 * @param msg
	 */
	public void broadcast(MsgBase msg, int serverid) {
		rwLock.readLock().lock();
		try {
			controllers.get(serverid).getConnection().writeAndFlush(new TextWebSocketFrame(msg.toJson()));
		} catch (Exception e) {
			Log.error(e.getMessage(), e);
		} finally {
			rwLock.readLock().unlock();
		}
	}
	
	/**
	 * 广播消息所有游戏服
	 * 
	 * @param msg
	 */
	public void broadcast(MsgBase msg) {
		rwLock.readLock().lock();
		try {
			for (GameController conn : controllers.values()) {
				conn.getConnection().writeAndFlush(new TextWebSocketFrame(msg.toJson()));
			}
		} catch (Exception e) {
			Log.error(e.getMessage(), e);
		} finally {
			rwLock.readLock().unlock();
		}
	}

	/**
	 * 广播消息
	 * 
	 * @param msg
	 */
	public void broadcast(MsgBase msg, Connection exclude) {
		rwLock.readLock().lock();
		try {
			for (GameController conn : controllers.values()) {
				if (conn.getConnection() != exclude) {
					conn.getConnection().writeAndFlush(new TextWebSocketFrame(msg.toJson()));
				}
			}
		} catch (Exception e) {
			Log.error(e.getMessage(), e);
		} finally {
			rwLock.readLock().unlock();
		}
	}

}
