package com.incubator.common.game;

import com.incubator.common.log4j.Log;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.CountDownLatch;


public abstract class AbstractService implements IService {

	private final Map<String, Object> attributes = new HashMap<>();

	private ServiceState state = ServiceState.STOPED;

	/** 非后台进程防止jvm退出 **/
	private final ThreadHolder threadHolder = new ThreadHolder();

	@Override
	public void startService() {
		switch (getState()) {
		case STOPED:
			this.setState(ServiceState.STARTING);
			try {
				this.doStart();
				this.daemonInit();
				this.setState(ServiceState.ACTIVE);
			} catch (Throwable e) {
				this.setState(ServiceState.STOPED);
				Log.error("===== 启动服务出错 : ", e);
			}
			break;
		case STARTING:
		case ACTIVE:
		case STOPING:
		default:
			break;
		}
	}

	@Override
	public void closeService() {
		switch (getState()) {
		case STOPED:
		case STOPING:
		case STARTING:
		case ACTIVE:
			setState(ServiceState.STARTING);
			try {
				doClose();
				daemonStop();
				setState(ServiceState.STOPED);
			} catch (Throwable e) {
				setState(ServiceState.ACTIVE);
				Log.error("===== 关闭服务出错 : ", e);
			}
			break;
		default:
			break;
		}
	}

	@Override
	public ServiceState getState() {
		return state;
	}

	@Override
	public boolean hasAttribute(String key) {
		return attributes.containsKey(key);
	}

	@Override
	public void setAttribute(String key, Object value) {
		attributes.put(key, value);
	}

	@Override
	public Object getAttribute(String key) {
		return attributes.get(key);
	}

	@Override
	public Object removeAttribute(String key) {
		return attributes.remove(key);
	}

	@Override
	public void clearAttributes() {
		attributes.clear();
	}

	/**
	 * 执行启动服务逻辑
	 **/
	protected abstract void doStart() throws Throwable;

	/**
	 * 执行关闭服务逻辑
	 **/
	protected abstract void doClose() throws Throwable;

	/** 设置启动状态 **/
	private void setState(ServiceState state) {
		this.state = state;
	}

	/**
	 * 增加一个关闭钩子
	 * 
	 * @author johnc
	 *
	 */
	private static class ThreadHolder implements Runnable {

		private volatile CountDownLatch cd;

		@Override
		public void run() {
			cd = new CountDownLatch(1);
			try {
				cd.await();
			} catch (Throwable e) {
				Log.error("====增加一个关闭钩子出错：", e);
			}
		}

		public void exit() {
			if (cd != null && cd.getCount() > 0) {
				cd.countDown();
			}
		}

	}

	{
		Runtime.getRuntime().addShutdownHook(new Thread(new Runnable() {
			@Override
			public void run() {
				Log.info("jvm进程关闭");
				doJvmClose();
			}
		}));
	}

	protected void daemonInit() {
		Thread t = new Thread(threadHolder);
		t.setDaemon(daemon);
		t.setName("GameService-ThreadHolder");
		t.start();
	}

	protected void daemonStop() {
		threadHolder.exit();
	}

	/** 是否后台 **/
	protected boolean daemon;

	protected boolean isDaemon() {
		return daemon;
	}

	protected void setDaemon(boolean daemon) {
		this.daemon = daemon;
	}

	/**
	 * 执行jvm关闭
	 */
	protected void doJvmClose() {
		closeService();
	}

}
