package com.incubator.common.net.netty;

import com.incubator.common.net.Connection;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.PooledByteBufAllocator;
import io.netty.channel.Channel;
import io.netty.util.AttributeKey;

import java.net.InetSocketAddress;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

/**
 * NettyConnection
 *
 * 负责管理 Netty 通道与 Connection 的绑定及数据收发。
 */
public class NettyConnection implements Connection {

	public static final AttributeKey<NettyConnection> CHANNEL_KEY = AttributeKey.newInstance("NettyConnection");

	// 使用线程安全的 ConcurrentHashMap 替代非线程安全的 HashMap
	protected final ConcurrentMap<String, Object> attributes = new ConcurrentHashMap<>();

	protected final Channel channel;

	private volatile String uid;
	private volatile String playerId;
	protected final int hashId;
	private volatile Object attachment;

	public NettyConnection(Channel channel) {
		this.channel = channel;
		this.hashId = channel.hashCode();
	}

	public Channel getChannel() {
		return channel;
	}

	@Override
	public String getUid() {
		return uid;
	}

	@Override
	public void setUid(String uid) {
		this.uid = uid;
	}

	@Override
	public String getPlayerId() {
		return playerId;
	}

	@Override
	public void setPlayerId(String playerId) {
		this.playerId = playerId;
	}

	@Override
	public int getHashCode() {
		return hashId;
	}

	@Override
	public boolean isActive() {
		return channel != null && channel.isActive() && channel.isWritable();
	}

	@Override
	public boolean hasAttribute(String key) {
		return attributes.containsKey(key);
	}

	@Override
	public void setAttribute(String key, Object value) {
		attributes.put(key, value);
	}

	@Override
	public Set<String> getAttributeNames() {
		return attributes.keySet();
	}

	@Override
	public Object getAttribute(String key) {
		return attributes.get(key);
	}

	@Override
	public Object removeAttribute(String key) {
		return attributes.remove(key);
	}

	@Override
	public void clearAttributes() {
		attributes.clear();
	}

	@SuppressWarnings("unchecked")
	@Override
	public <T> T getAttachment() {
		return (T) attachment;
	}

	@Override
	public <T> void setAttachment(T attachment) {
		this.attachment = attachment;
	}

	@Override
	public InetSocketAddress getRemoteAddress() {
		return (InetSocketAddress) channel.remoteAddress();
	}

	@Override
	public InetSocketAddress getLocalAddress() {
		return (InetSocketAddress) channel.localAddress();
	}

	@Override
	public void close() {
		if (channel != null && channel.isOpen()) {
			try {
				onClose();
				channel.close();
			} catch (Exception e) {
				// 捕获关闭异常，避免高并发时影响其他连接
				e.printStackTrace();
			}
		}
	}

	@Override
	public void write(Object obj) {
		if (isActive()) {
			channel.write(obj);
		}
	}

	@Override
	public void write(byte[] bytes) {
		if (bytes != null && bytes.length > 0 && isActive()) {
			ByteBuf buf = PooledByteBufAllocator.DEFAULT.buffer(bytes.length);
			buf.writeBytes(bytes);
			write(buf);
		}
	}

	@Override
	public void writeAndFlush(Object obj) {
		if (isActive()) {
			channel.writeAndFlush(obj);
		}
	}

	@Override
	public void writeAndFlush(byte[] bytes) {
		if (bytes != null && bytes.length > 0 && isActive()) {
			ByteBuf buf = PooledByteBufAllocator.DEFAULT.buffer(bytes.length);
			buf.writeBytes(bytes);
			writeAndFlush(buf);
		}
	}

	@Override
	public void flush() {
		if (channel != null && isActive()) {
			channel.flush();
		}
	}

	/**
	 * 关闭连接时的钩子方法，子类可重写以实现自定义逻辑
	 */
	protected void onClose() {
		// 子类可扩展关闭时的逻辑
	}
}
