package com.incubator.common.net.netty.client;

import com.incubator.common.log4j.Log;
import com.incubator.common.net.NetClient;
import io.netty.bootstrap.Bootstrap;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelFutureListener;
import io.netty.channel.EventLoopGroup;

import java.net.InetSocketAddress;
import java.util.TimerTask;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 抽象netty客户端 已实现断线重连逻辑 只关心状态,重连,地址,不关心启动器和线程
 */
public abstract class NettyClientFactory implements NetClient {

	private String name = "";

	protected volatile boolean reconnect;

	protected long timeMills;

	protected final InetSocketAddress target;

	private Channel channel;

	protected final ReconectListener reconectListener = new ReconectListener();

	/** 重连调度器 **/
	private ScheduledExecutorService reconnectExecutor;

	public final Channel getCurrentChannel() {
		return channel;
	}
	
	public NettyClientFactory(InetSocketAddress target) {
		this.target = target;
	}

	@Override
	public final String getName() {
		return name;
	}

	@Override
	public final void setName(String name) {
		if (name != null) {
			this.name = name;
		}
	}

	@Override
	public final void start() {
		if (isConnected()) {
			stop();
		}
		connect(target);
	}

	@Override
	public final void stop() {
		if (channel != null && channel.isActive()) {
			disableReconnect();
			channel.close();
		}
	}

	@Override
	public final InetSocketAddress getTarget() {
		return target;
	}

	@Override
	public boolean isConnected() {
		return channel != null && channel.isActive();
	}

	@Override
	public final void enableReconnect(ScheduledExecutorService executor, long timeMills) {
		if (executor == null || timeMills <= 0) {
			throw new IllegalArgumentException();
		}
		this.reconnect = true;
		this.reconnectExecutor = executor;
		this.timeMills = timeMills;
	}

	@Override
	public void disableReconnect() {
		this.reconnect = false;
	}

	@Override
	public final boolean isReconnect() {
		return this.reconnect;
	}

	@Override
	public final long getReconnectTimeMills() {
		return timeMills;
	}

	/**
	 * 发送数据,但不flush
	 */
	@Override
	public void sendData(byte[] data) {
		sendObject(data);
	}

	@Override
	public void sendObject(Object obj) {
		if (channel != null) {
			channel.write(obj);
		}
	}

	@Override
	public void flush() {
		if (channel != null) {
			channel.flush();
		}
	}
	
	/**
	 * 获取客户端使用的起动器
	 */
	protected abstract Bootstrap getBooter();

	/**
	 * 获取客户端使用的IO线程池
	 */
	protected abstract EventLoopGroup getIoWorkers();
	
	/**
	 * 调用启动器发起连接
	 * 
	 * @return 链接后的ChannelFuture
	 */
	protected abstract ChannelFuture doConnect(InetSocketAddress target);

	protected final ScheduledExecutorService getReconnectExecutor() {
		if (reconnectExecutor != null) {
			return reconnectExecutor;
		}
		return getIoWorkers();
	}

	private class ReconectListener implements ChannelFutureListener {

		@Override
		public void operationComplete(ChannelFuture future) throws Exception {
			Log.info("通道:" + future.channel() + "断开连接!,是否重连:" + isReconnect());
			doReconect();// 这里不能占用IO线程池
		}

	}

	/**
	 * 执行连接调用{@link ChannelFuture executeBooterConnect(InetSocketAddress target)} 执行多次会把channel顶掉
	 */
	protected final boolean connect(InetSocketAddress target) {
		boolean isConnect = false;
		try {
			Log.info("连接中(" + target + ")...");
			ChannelFuture cf = doConnect(target);
			if (cf == null) {// 如果阻塞则使用系统调度器执行
				Log.info("连接繁忙(" + target + ")!稍后重连:" + isReconnect());
				doReconect();// 这里不能占用IO线程池
			} else {
				isConnect = cf.isDone() && cf.isSuccess();
				if (isConnect) {// 连接成功
					Log.info("连接成功(" + target + ")!" + cf.channel());
					this.channel = cf.channel();
					// 给通道加上断线重连监听器
					this.channel.closeFuture().removeListener(reconectListener);
					this.channel.closeFuture().addListener(reconectListener);
				} else {// 连接不成功则10秒再执行一次连接
					Log.info("连接失败(" + target + ")!" + cf.channel());
					doReconect();// 这里不能占用IO线程池
				}
			}
		} catch (Exception e) {
			Log.error(e.getMessage(), e);
		}
		return isConnect;
	}
	
	/**
	 * 执行重连 timer.schedule(new ReConnectTask(), reconectTimeOut);
	 * 执行多次会把channel顶掉
	 */
	protected final void doReconect() {
		if (isReconnect()) {
			getReconnectExecutor().schedule(new ReConnectTask(), getReconnectTimeMills(), TimeUnit.MILLISECONDS);// 这里不能占用IO线程池
		}
	}
	
	/**
	 * 重连接任务
	 */
	private class ReConnectTask extends TimerTask {

		@Override
		public void run() {
			try {
				connect(target);
			} catch (Exception e) {
				Log.error(e.getMessage(), e);
			}
		}
	}

}