package com.incubator.common.net.netty.handler.http;

import com.incubator.common.net.ConnectionListener;
import com.incubator.common.net.netty.listener.adapter.ListenerHandlerAdapter;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelPipeline;
import io.netty.channel.socket.SocketChannel;
import io.netty.handler.codec.http.*;
import io.netty.handler.codec.http.cors.CorsConfig;
import io.netty.handler.codec.http.cors.CorsConfigBuilder;
import io.netty.handler.codec.http.cors.CorsHandler;
import io.netty.handler.ssl.SslContext;
import io.netty.handler.ssl.SslHandler;

/**
 * HTTP 服务初始化处理器
 */
public class HttpServerInitHandler extends ChannelInitializer<SocketChannel> {

	private final ConnectionListener<HttpRequest> listener;
	private final SslContext sslContext;
	private static final CorsConfig CORS_CONFIG = buildCorsConfig();

	/**
	 * 构造函数（不使用 SSL）
	 *
	 * @param listener 连接监听器
	 */
	public HttpServerInitHandler(ConnectionListener<HttpRequest> listener) {
		this(listener, null);
	}

	/**
	 * 构造函数（可选使用 SSL）
	 *
	 * @param listener 连接监听器
	 * @param sslContext SSL 上下文
	 */
	public HttpServerInitHandler(ConnectionListener<HttpRequest> listener, SslContext sslContext) {
		this.listener = listener;
		this.sslContext = sslContext;
	}

	@Override
	protected void initChannel(SocketChannel ch) throws Exception {
		ChannelPipeline pipeline = ch.pipeline();

		// 添加 SSL 支持（如果需要）
		if (sslContext != null) {
			SslHandler sslHandler = sslContext.newHandler(ch.alloc());
			pipeline.addLast(sslHandler);
		}

		// 添加 HTTP 编解码器
		pipeline.addLast(new HttpResponseEncoder());
		pipeline.addLast(new HttpRequestDecoder());
		pipeline.addLast(new HttpObjectAggregator(65536)); // 限制 contentLength，防止过大请求影响性能

		// 添加跨域支持
		pipeline.addLast(new CorsHandler(CORS_CONFIG));

		// 添加自定义监听适配器
		pipeline.addLast(new ListenerHandlerAdapter<>(listener));
	}

	/**
	 * 构建跨域配置（仅执行一次，避免重复构建提高性能）
	 *
	 * @return CorsConfig 跨域配置
	 */
	private static CorsConfig buildCorsConfig() {
		return CorsConfigBuilder.forAnyOrigin()
				.allowNullOrigin()
				.allowCredentials()
				.build();
	}
}
