package com.incubator.common.net.netty.handler.websocket;

import com.incubator.common.log4j.Log;
import io.netty.channel.*;
import io.netty.channel.ChannelHandler.Sharable;
import io.netty.handler.codec.http.HttpObjectAggregator;
import io.netty.handler.codec.http.HttpServerCodec;
import io.netty.handler.codec.http.websocketx.WebSocketServerProtocolHandler;
import io.netty.handler.codec.http.websocketx.WebSocketServerProtocolHandler.HandshakeComplete;
import io.netty.handler.stream.ChunkedWriteHandler;

/**
 * WebSocket 服务端初始化器
 *
 * 配置解码器和连接成功的监听器。
 */
@Sharable
public abstract class WebSocketServerInitializer extends ChannelInitializer<Channel> {

	private final String uri;

	public WebSocketServerInitializer(String uri) {
		this.uri = uri;
	}

	@Override
	protected final void initChannel(Channel ch) throws Exception {
		ChannelPipeline pipeline = ch.pipeline();
		// 使用 Netty 内置的解码器和处理器
		pipeline.addLast(new HttpServerCodec());
		pipeline.addLast(new ChunkedWriteHandler());
		pipeline.addLast(new HttpObjectAggregator(64 * 1024)); // 聚合 HTTP 请求
		pipeline.addLast(new WebSocketServerProtocolHandler(uri)); // WebSocket 协议支持
		pipeline.addLast(new WebSocketHandshakeHandler()); // 连接成功后的处理器
	}

	/**
	 * 用于监控 WebSocket 握手事件并调用自定义处理逻辑。
	 */
	private class WebSocketHandshakeHandler extends SimpleChannelInboundHandler<Object> {

		@Override
		public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
			// 检测握手完成事件
			if (evt instanceof HandshakeComplete) {
				HandshakeComplete handshake = (HandshakeComplete) evt;
				Log.debug("WebSocket 握手完成，URI: {}, Headers: {}", handshake.requestUri(), handshake.requestHeaders());

				// 调用自定义逻辑
				webSocketHandComplete(ctx);

				// 移除当前处理器，减少管道中的不必要负载
				ctx.pipeline().remove(this);
				Log.debug("握手完成处理器已移除，当前管道: {}", ctx.pipeline().toMap());
			} else {
				super.userEventTriggered(ctx, evt);
			}
		}

		@Override
		protected void channelRead0(ChannelHandlerContext ctx, Object msg) {
			// 不处理普通的 WebSocket 消息，仅处理握手事件
		}
	}

	/**
	 * 当握手成功后调用该抽象方法。
	 * 子类需实现自定义逻辑，并确保正确触发下游的事件。
	 *
	 * @param ctx Netty 通道上下文
	 */
	protected abstract void webSocketHandComplete(ChannelHandlerContext ctx);
}
