package com.incubator.common.net.netty.handler.websocket.binary.codec;

import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.MessageToMessageCodec;
import io.netty.handler.codec.http.websocketx.BinaryWebSocketFrame;
import io.netty.handler.codec.http.websocketx.WebSocketFrame;

import java.util.List;

/**
 * BinaryWebSocketFrame 与 ByteBuf 消息之间的转换适配器。
 *
 * 适用于高性能环境，通过优化内存管理和线程安全处理，避免业务层直接操作 ByteBuf。
 */
public class WebSocketBinaryFrameByteBufAdapter extends MessageToMessageCodec<WebSocketFrame, ByteBuf> {

	/**
	 * 将 WebSocket 消息转换为 ByteBuf 类型，以适配后续的业务逻辑。
	 *
	 * @param ctx   上下文
	 * @param frame 输入的 WebSocket 消息
	 * @param out   输出列表
	 */
	@Override
	protected void decode(ChannelHandlerContext ctx, WebSocketFrame frame, List<Object> out) {
		if (frame instanceof BinaryWebSocketFrame) {
			BinaryWebSocketFrame binaryFrame = (BinaryWebSocketFrame) frame;
			ByteBuf content = binaryFrame.content();
			out.add(content.retain()); // 引用计数增加，避免内容被释放
		} else {
			// 非二进制消息直接丢弃或日志记录
			ctx.fireExceptionCaught(new IllegalArgumentException("Unsupported WebSocketFrame type: " + frame.getClass().getName()));
		}
	}

	/**
	 * 将业务层直接发送的 ByteBuf 转换为 WebSocket 消息。
	 *
	 * @param ctx   上下文
	 * @param buf   输出的 ByteBuf 消息
	 * @param out   输出列表
	 */
	@Override
	protected void encode(ChannelHandlerContext ctx, ByteBuf buf, List<Object> out) {
		out.add(new BinaryWebSocketFrame(buf.retain())); // 包装为 BinaryWebSocketFrame，并增加引用计数
	}
}
