package com.incubator.common.net.netty.listener;

import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.common.net.ConnectionHeartbeatListener;

import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * ConnectionHeartbeatListenerFactory
 * 支持心跳机制的链路监听器
 *
 * @param <T> 消息类型
 */
public abstract class ConnectionHeartbeatListenerFactory<T> implements ConnectionHeartbeatListener<Object> {

	/** 心跳验证间隔（仅当空闲时触发） **/
	public static final long HEART_INTERVAL_MILLIS = TimeUnit.SECONDS.toMillis(5);

	/** 心跳开关 **/
	private volatile boolean heartEnable = false;

	/** 最后异常读消息时间 **/
	public static final String LAST_READ_TIME_MILLIS = "LastReadTimeMillis";

	/** 心跳配置 **/
	public static final String HEART_CONFIG = "HeartConfig";

	public boolean isHeartEnable() {
		return heartEnable;
	}

	public void setHeartEnable(boolean heartEnable) {
		this.heartEnable = heartEnable;
	}

	/** 心跳配置类 **/
	public static class HeartConfig {
		/** 最大心跳请求次数 **/
		private static final int DEFAULT_MAX_SEQ = 2;

		private final AtomicInteger seq = new AtomicInteger(0);
		private int maxSeq = DEFAULT_MAX_SEQ;

		public int getSeq() {
			return seq.get();
		}

		public void resetSeq() {
			seq.set(0);
		}

		public void incrementSeq() {
			seq.incrementAndGet();
		}

		public int getMaxSeq() {
			return maxSeq;
		}

		public void setMaxSeq(int maxSeq) {
			this.maxSeq = maxSeq;
		}
	}

	@Override
	public void messageArrived(Connection conn, Object msg) {
		try {
			conn.setAttribute(LAST_READ_TIME_MILLIS, System.currentTimeMillis());
			resetHeartSeq(conn); // 重置心跳序号

			if (isHeartReq(msg)) {
				sendHeartRsp(conn); // 回复心跳请求
				return;
			}
			doMessageArrived(conn, msg); // 处理业务消息
		} catch (Exception e) {
			Log.error("Message processing error:", e);
		}
	}

	@Override
	public long getReaderTimeMills() {
		return HEART_INTERVAL_MILLIS;
	}

	@Override
	public long getWriterTimeMills() {
		return HEART_INTERVAL_MILLIS;
	}

	@Override
	public long getAllTimeMills() {
		return getWriterTimeMills() * 4;
	}

	@Override
	public void event_AllTimeOut(Connection connection) {
		if (!isHeartEnable()) {
			return;
		}
		Log.warn("Connection all idle timeout, closing: {}", connection);
		connection.close();
	}

	@Override
	public void event_ReadTimeOut(Connection connection) {
		if (!isHeartEnable()) {
			return;
		}

		HeartConfig heartConfig = getOrCreateHeartConfig(connection);

		if (heartConfig.getSeq() >= heartConfig.getMaxSeq()) {
			Log.warn("Read timeout exceeded max heartbeats, closing connection: {}, LastReadTimeMillis: {}", connection,
					connection.getAttribute(LAST_READ_TIME_MILLIS));
			connection.close();
		} else {
			sendHeartReq(connection);
			heartConfig.incrementSeq();
			Log.debug("Read timeout, sending heartbeat request: {}, Seq: {}", connection, heartConfig.getSeq());
		}
	}

	@Override
	public void event_WriteTimeOut(Connection connection) {
		if (!isHeartEnable()) {
			return;
		}
		Log.debug("Write timeout, sending heartbeat request: {}", connection);
		sendHeartReq(connection);
	}

	/**
	 * 重置心跳发送序号
	 */
	protected void resetHeartSeq(Connection conn) {
		HeartConfig heartConfig = getOrCreateHeartConfig(conn);
		heartConfig.resetSeq();
	}

	/**
	 * 获取或创建 HeartConfig
	 */
	private HeartConfig getOrCreateHeartConfig(Connection conn) {
		HeartConfig heartConfig = (HeartConfig) conn.getAttribute(HEART_CONFIG);
		if (heartConfig == null) {
			heartConfig = new HeartConfig();
			conn.setAttribute(HEART_CONFIG, heartConfig);
		}
		return heartConfig;
	}

	/**
	 * 判断是否为心跳请求
	 */
	protected abstract boolean isHeartReq(Object msg);

	/**
	 * 发送心跳请求
	 */
	protected abstract void sendHeartReq(Connection connection);

	/**
	 * 发送心跳回复
	 */
	protected abstract void sendHeartRsp(Connection connection);

	/**
	 * 处理收到的消息
	 */
	protected abstract void doMessageArrived(Connection conn, Object msg);
}
