package com.incubator.common.net.netty.listener.adapter;

import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.common.net.ConnectionListener;
import com.incubator.common.net.netty.ChannelKeys;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.util.ReferenceCountUtil;

import java.io.IOException;

/**
 * AbstractListenerHandlerAdapter
 *
 * 负责 Channel 事件到 Listener 的适配
 */
abstract class AbstractListenerHandlerAdapter<T, L extends ConnectionListener<T>> extends ChannelInboundHandlerAdapter {

	protected final L listener;

	public AbstractListenerHandlerAdapter(L listener) {
		if (listener == null) {
			throw new UnsupportedOperationException("listener cannot be null");
		}
		this.listener = listener;
	}

	@Override
	public void channelActive(ChannelHandlerContext ctx) throws Exception {
		Channel channel = ctx.channel();
		Connection connection = buildConnection(channel);
		bindConnection(channel, connection); // 绑定链接
		try {
			listener.connectionOpened(connection);
		} catch (Exception e) {
			Log.error("Connection opened event failed: {}", e.getMessage(), e);
		}
		super.channelActive(ctx);
	}

	@Override
	public final void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
		if (msg == null) {
			return;
		}
		boolean release = false;
		try {
			T iMsg = castMessage(msg);
			if (iMsg == null) {
				ctx.fireChannelRead(msg); // 无法处理，传递到下一个 handler
				return;
			}

			Channel channel = ctx.channel();
			Connection connection = findConnection(channel);

			if (connection != null) {
				listener.messageArrived(connection, iMsg);
				release = true; // 确认消息已被处理，释放资源
			} else {
				Log.warn("Message received but no connection found for channel: {}", ctx.channel());
				ctx.fireChannelRead(msg); // 无效连接传递到下一个 handler
			}
		} catch (Exception e) {
			Log.error("Message processing error: {}", e.getMessage(), e);
			ctx.fireChannelRead(msg); // 出现异常后将消息传递到下一个 handler
		} finally {
			if (release) {
				ReferenceCountUtil.release(msg);
			}
		}
	}

	@Override
	public final void channelInactive(ChannelHandlerContext ctx) throws Exception {
		Channel channel = ctx.channel();
		Connection connection = findConnection(channel);
		if (connection != null) {
			try {
				listener.connectionClosed(connection);
			} catch (Exception e) {
				Log.error("Connection closed event failed: {}", e.getMessage(), e);
			}
		} else {
			Log.warn("Channel inactive but no connection found: {}", ctx.channel());
		}
		super.channelInactive(ctx);
	}

	@Override
	public final void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
		if (cause instanceof IOException) {
			Log.warn("IOException occurred, ignoring: {}", cause.getMessage());
		} else {
			Log.error("Exception caught in channel: {}, cause: {}", ctx.channel(), cause.getMessage(), cause);
		}
		ctx.fireExceptionCaught(cause);
	}

	/**
	 * 创建一个连接实例
	 */
	protected Connection buildConnection(Channel channel) {
		Connection connection = listener.getConnectionFactory().buildConnection(channel);
		if (connection == null) {
			Log.error("Failed to create Connection for channel: {}", channel);
			throw new UnsupportedOperationException("Failed to create Connection");
		}
		return connection;
	}

	/**
	 * 绑定连接到通道
	 */
	protected void bindConnection(Channel channel, Connection conn) {
		channel.attr(ChannelKeys.CONNECTION_KEY).set(conn);
	}

	/**
	 * 查找连接
	 */
	protected Connection findConnection(Channel channel) {
		return channel.attr(ChannelKeys.CONNECTION_KEY).get();
	}

	/**
	 * 将消息转换为目标类型
	 */
	@SuppressWarnings("unchecked")
	protected T castMessage(Object msg) {
		try {
			return (T) msg;
		} catch (ClassCastException e) {
			Log.warn("Failed to cast message: {}", e.getMessage());
			return null;
		}
	}
}
