package com.incubator.common.net.netty.listener.adapter;

import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.common.net.ConnectionHeartbeatListener;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandler.Sharable;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.timeout.IdleStateEvent;
import io.netty.handler.timeout.IdleStateHandler;

import java.util.concurrent.TimeUnit;

/**
 * ListenerHandlerHeartbeatAdapter
 *
 * 负责心跳检测并触发相应事件。必须放置在解码器之后。
 *
 * @param <M> 消息类型
 */
@Sharable
public class ListenerHandlerHeartbeatAdapter<M> extends AbstractListenerHandlerAdapter<M, ConnectionHeartbeatListener<M>> {

	private ListenerIdleHandler handler;

	public ListenerHandlerHeartbeatAdapter(ConnectionHeartbeatListener<M> listener) {
		super(listener);
	}

	@Override
	public final void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
		if (evt instanceof IdleStateEvent) {
			IdleStateEvent event = (IdleStateEvent) evt;
			Channel channel = ctx.channel();
			Connection connection = findConnection(channel);

			if (connection == null) {
				Log.warn("Heartbeat event triggered, but no connection found for channel: {}", channel);
				return;
			}

			switch (event.state()) {
				case ALL_IDLE : {
//					Log.debug("All idle timeout for connection: {}", connection);
					onIdleTimeout(connection, "ALL_IDLE");
					listener.event_AllTimeOut(connection);
				}
				case READER_IDLE : {
//					Log.debug("Read idle timeout for connection: {}", connection);
					onIdleTimeout(connection, "READER_IDLE");
					listener.event_ReadTimeOut(connection);
				}
				case WRITER_IDLE : {
//					Log.debug("Write idle timeout for connection: {}", connection);
					onIdleTimeout(connection, "WRITER_IDLE");
					listener.event_WriteTimeOut(connection);
				}
				default :
//					Log.warn("Unknown IdleStateEvent: {}", event.state());
			}
		} else {
			super.userEventTriggered(ctx, evt);
		}
	}

	@Override
	public final void handlerAdded(ChannelHandlerContext ctx) throws Exception {
		if (handler != null) {
			Log.warn("Existing ListenerIdleHandler detected: {}", handler);
			onHandlerReplaced(ctx, handler);
		}
		handler = new ListenerIdleHandler(
				listener.getReaderTimeMills(),
				listener.getWriterTimeMills(),
				listener.getAllTimeMills(),
				TimeUnit.MILLISECONDS
		);
		String handlerName = getIdleHandlerName(ctx);
		ctx.pipeline().addBefore(ctx.name(), handlerName, handler);
		Log.info("ListenerIdleHandler added to pipeline with name: {}", handlerName);

		super.handlerAdded(ctx);
	}

	/**
	 * 钩子方法：处理心跳超时事件
	 */
	protected void onIdleTimeout(Connection connection, String idleType) {
		// 子类可以重写处理心跳超时逻辑
	}

	/**
	 * 钩子方法：处理 Handler 替换事件
	 */
	protected void onHandlerReplaced(ChannelHandlerContext ctx, ListenerIdleHandler oldHandler) {
		// 子类可以重写处理 Handler 替换逻辑
		Log.warn("Replacing old ListenerIdleHandler: {}", oldHandler);
	}

	/**
	 * 和监听器相关的读写超时监测 handler，不使用全局 IdleStateHandler，避免影响其他扩展
	 */
	public static class ListenerIdleHandler extends IdleStateHandler {
		public ListenerIdleHandler(long readerIdleTime, long writerIdleTime, long allIdleTime, TimeUnit unit) {
			super(readerIdleTime, writerIdleTime, allIdleTime, unit);
		}
	}

	/**
	 * 获取心跳检测 Handler 的名称
	 */
	protected String getIdleHandlerName(ChannelHandlerContext ctx) {
		return "ListenerIdleHandler-" + ctx.channel().id().asShortText();
	}
}
