package com.incubator.common.net.netty.server;

import java.net.InetSocketAddress;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import com.incubator.common.log4j.Log;
import com.incubator.common.net.netty.ServerOptionConfiger;

import io.netty.bootstrap.ServerBootstrap;
import io.netty.buffer.ByteBufAllocator;
import io.netty.channel.*;
import io.netty.channel.epoll.Epoll;
import io.netty.channel.epoll.EpollServerSocketChannel;
import io.netty.channel.group.ChannelGroup;
import io.netty.channel.group.DefaultChannelGroup;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.util.concurrent.ImmediateEventExecutor;

/**
 * NettyServer
 */
public class NettyServer extends NettyServerFactory {

	private static final int BIND_TIMEOUT_SECONDS = 5;

	protected final ChannelGroup channelGroup = new DefaultChannelGroup(getName() + "ChannelGroup", ImmediateEventExecutor.INSTANCE);

	protected final ServerBootstrap booter = new ServerBootstrap();

	protected final NettyServerConfig config;

	protected final ChannelHandler handler;

	private final Lock lock = new ReentrantLock();

	public NettyServer(String host, int port, ChannelHandler handler) {
		this(new InetSocketAddress(host, port), handler);
	}

	public NettyServer(InetSocketAddress local, ChannelHandler handler) {
		this(new NettyServerConfig(), local, handler);
	}

	public NettyServer(NettyServerConfig config, InetSocketAddress local, ChannelHandler handler) {
		super(local);
		this.config = config;
		this.handler = handler;
		initBooter();
	}

	/** 初始化 Booter **/
	private void initBooter() {
		this.booter.group(this.config.getBoss(), this.config.getIoWorkers())
				.channel(getDefaultChannelClass())
				.option(ChannelOption.ALLOCATOR, ByteBufAllocator.DEFAULT) // 默认分配器
				.childOption(ChannelOption.ALLOCATOR, ByteBufAllocator.DEFAULT); // 子通道分配器
	}

	@Override
	public final void broadCast(Object message) {
		this.channelGroup.writeAndFlush(message);
		Log.debug("broadCast message total: {}, type: {}", this.channelGroup.size(), message);
	}

	@Override
	public int getConnectionCount() {
		return this.channelGroup.size();
	}

	@Override
	protected final ChannelFuture doBind(InetSocketAddress local) {
		this.booter.localAddress(local);
		initServerOptions(optionConfig());
		ChannelHandler fixedHandler = fixHandlerBeforeDoBooterBind(this.handler); // 修正 handler

		return doBooterBind(local, fixedHandler); // 启动端口绑定
	}

	@Override
	public ServerBootstrap getBooter() {
		return this.booter;
	}

	@Override
	public EventLoopGroup getIoWorkers() {
		return this.config.getIoWorkers();
	}

	@Override
	public EventLoopGroup getBossWorkers() {
		return this.config.getBoss();
	}

	public ChannelGroup getChannelGroup() {
		return this.channelGroup;
	}

	/**
	 * 初始化服务端选项
	 */
	protected void initServerOptions(ServerOptionConfiger configer) {
		configer.option(ChannelOption.SO_BACKLOG, 1024); // 设置连接等待队列大小
		configer.option(ChannelOption.SO_REUSEADDR, true); // 重用地址
		configer.childOption(ChannelOption.TCP_NODELAY, true); // 关闭 Nagle 算法，提升小数据传输性能
		configer.childOption(ChannelOption.SO_KEEPALIVE, true); // 保持连接
		configer.childOption(ChannelOption.SO_RCVBUF, 1024 * 64); // 接收缓冲区大小
		configer.childOption(ChannelOption.WRITE_BUFFER_WATER_MARK, new WriteBufferWaterMark(32 * 1024, 64 * 1024)); // 写缓冲区高低水位线
	}

	public ServerOptionConfiger optionConfig() {
		return new ServerOptionConfiger() {
			@Override
			public <T> ServerOptionConfiger option(ChannelOption<T> option, T value) {
				booter.option(option, value);
				return this;
			}

			@Override
			public <T> ServerOptionConfiger childOption(ChannelOption<T> option, T value) {
				booter.childOption(option, value);
				return this;
			}
		};
	}

	/**
	 * 执行启动绑定之前修正 handler
	 */
	protected ChannelHandler fixHandlerBeforeDoBooterBind(ChannelHandler handler) {
		return handler;
	}

	/**
	 * 绑定
	 */
	protected ChannelFuture doBooterBind(InetSocketAddress local, final ChannelHandler fixedHandler) {
		lock.lock();
		try {
			final CountDownLatch latch = new CountDownLatch(1);
			this.booter.childHandler(new ChannelInitializer<Channel>() {
				@Override
				protected void initChannel(Channel ch) throws Exception {
					channelGroup.add(ch);
					ch.pipeline().addLast(fixedHandler);
				}
			});

			ChannelFuture cf = this.booter.bind(local);
			cf.addListener((ChannelFutureListener) future -> {
				if (future.isSuccess()) {
					Log.info("绑定端口成功: {}", local);
				} else {
					Log.error("绑定端口失败: {}", future.cause());
				}
				latch.countDown();
			});

			if (!latch.await(BIND_TIMEOUT_SECONDS, TimeUnit.SECONDS)) {
				Log.error("绑定端口超时: {}", local);
			}
			return cf;
		} catch (InterruptedException e) {
			Thread.currentThread().interrupt();
			Log.error("绑定被中断: {}", e.getMessage(), e);
			throw new RuntimeException(e);
		} finally {
			lock.unlock();
		}
	}

	/**
	 * 获取默认的 Channel 类型
	 */
	private Class<? extends ServerChannel> getDefaultChannelClass() {
		String osName = System.getProperty("os.name").toLowerCase();
//		if (osName.contains("linux")) {
//			return EpollServerSocketChannel.class;
//		}
		return NioServerSocketChannel.class;
	}
	public static void main(String[] args) {
		System.out.println("Epoll is supported: " + Epoll.isAvailable());
		if (!Epoll.isAvailable()) {
			System.out.println("Epoll unavailable cause: " + Epoll.unavailabilityCause());
		}
	}
}
