package com.incubator.common.net.netty.server;

import com.incubator.common.log4j.Log;
import com.incubator.common.thread.NamedThreadFactory;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.ServerChannel;
import io.netty.channel.epoll.EpollServerSocketChannel;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioServerSocketChannel;

import java.util.concurrent.TimeUnit;

/**
 * NettyServerConfig
 */
public class NettyServerConfig {

	private static final int DEFAULT_BOSS_THREADS = Math.max(1, Runtime.getRuntime().availableProcessors() / 2);
	private static final int DEFAULT_IO_THREADS = Math.max(1, Runtime.getRuntime().availableProcessors() * 2);

	protected final Class<? extends ServerChannel> channelClass;

	protected final EventLoopGroup boss;

	protected final EventLoopGroup ioWorkers;

	public NettyServerConfig(Class<? extends ServerChannel> channelClass, EventLoopGroup boss, EventLoopGroup ioWorkers) {
		this.channelClass = channelClass;
		this.boss = boss;
		this.ioWorkers = ioWorkers;
	}

	public NettyServerConfig() {
		this(getDefaultChannelClass(),
				new NioEventLoopGroup(DEFAULT_BOSS_THREADS, new NamedThreadFactory("Server-boss", true)),
				new NioEventLoopGroup(DEFAULT_IO_THREADS, new NamedThreadFactory("Server-ioWorkers", true)));
	}

	public NettyServerConfig(int bossThreads, int ioThreads) {
		this(getDefaultChannelClass(),
				new NioEventLoopGroup(bossThreads, new NamedThreadFactory("Server-boss", true)),
				new NioEventLoopGroup(ioThreads, new NamedThreadFactory("Server-ioWorkers", true)));
	}

	public Class<? extends ServerChannel> getChannelClass() {
		return channelClass;
	}

	public EventLoopGroup getBoss() {
		return boss;
	}

	public EventLoopGroup getIoWorkers() {
		return ioWorkers;
	}

	/**
	 * 获取默认的 Channel 类型，根据操作系统选择。
	 */
	private static Class<? extends ServerChannel> getDefaultChannelClass() {
		if (isLinuxPlatform()) {
			return EpollServerSocketChannel.class;
		}
		return NioServerSocketChannel.class;
	}

	/**
	 * 判断是否为 Linux 平台。
	 */
	private static boolean isLinuxPlatform() {
		String osName = System.getProperty("os.name").toLowerCase();
		return osName.contains("linux");
	}

	/**
	 * 释放资源。
	 */
	public void destroy() {
		try {
			if (boss != null) {
				boss.shutdownGracefully(2, 30, TimeUnit.SECONDS).sync();
			}
		} catch (InterruptedException e) {
			Thread.currentThread().interrupt();
			Log.error("Error shutting down boss group: " + e.getMessage());
		}

		try {
			if (ioWorkers != null) {
				ioWorkers.shutdownGracefully(2, 30, TimeUnit.SECONDS).sync();
			}
		} catch (InterruptedException e) {
			Thread.currentThread().interrupt();
			Log.error("Error shutting down IO workers group: " + e.getMessage());
		}
	}
}
