package com.incubator.common.net.netty.server;

import com.incubator.common.log4j.Log;
import com.incubator.common.net.NetServer;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.socket.ServerSocketChannel;

import java.net.InetSocketAddress;

/**
 * NettyServerFactory
 */
public abstract class NettyServerFactory implements NetServer {

	protected final InetSocketAddress local;

	protected ServerSocketChannel serverChannel;

	private String name = "NettyServer";

	public NettyServerFactory(InetSocketAddress local) {
		this.local = local;
	}

	@Override
	public final boolean start() {
		long startTime = System.currentTimeMillis();
		boolean result = bind(this.local);
		long elapsedTime = System.currentTimeMillis() - startTime;

		if (result) {
			Log.info("[{}] 服务启动成功，耗时 {}ms", name, elapsedTime);
		} else {
			Log.error("[{}] 服务启动失败，耗时 {}ms", name, elapsedTime);
		}
		return result;
	}

	@Override
	public final void stop() {
		if (this.serverChannel != null) {
			try {
				Log.info("[{}] 服务正在停止...", name);
				this.serverChannel.close().sync();
				Log.info("[{}] 服务停止成功", name);
			} catch (InterruptedException e) {
				Thread.currentThread().interrupt();
				Log.error("[{}] 服务停止异常: {}", name, e.getMessage(), e);
			}
		}
	}

	@Override
	public final String getName() {
		return this.name;
	}

	@Override
	public final void setName(String name) {
		if (name != null) {
			this.name = name;
		}
	}

	@Override
	public final boolean isActive() {
		return this.serverChannel != null && this.serverChannel.isActive();
	}

	/**
	 * 启动端口绑定
	 *
	 * @param local
	 * @return 是否绑定成功
	 */
	protected final boolean bind(InetSocketAddress local) {
		boolean isBind = false;
		try {
			Log.info("[{}] 正在绑定端口: {}", name, local);
			ChannelFuture cf = doBind(local);

			// 等待绑定完成并验证通道状态
			cf.awaitUninterruptibly();

			isBind = cf.isSuccess() && cf.channel().isActive();
			if (isBind) {
				this.serverChannel = (ServerSocketChannel) cf.channel();
				Log.info("[{}] 端口绑定成功: {}", name, cf.channel().localAddress());
			} else {
				Log.error("[{}] 端口绑定失败: {}", name, cf.cause());
			}
		} catch (Exception e) {
			Log.error("[{}] 端口绑定异常: {}", name, e.getMessage(), e);
		}
		return isBind;
	}

	/**
	 * 使用启动器进行端口绑定
	 *
	 * @param local
	 * @return ChannelFuture
	 */
	protected abstract ChannelFuture doBind(InetSocketAddress local);

	/**
	 * 获取服务端使用的启动器
	 *
	 * @return ServerBootstrap
	 */
	public abstract ServerBootstrap getBooter();

	/**
	 * 获取服务端使用的 IO 线程池
	 *
	 * @return EventLoopGroup
	 */
	public abstract EventLoopGroup getIoWorkers();

	/**
	 * 获取服务端使用的 Boss 线程池
	 *
	 * @return EventLoopGroup
	 */
	public abstract EventLoopGroup getBossWorkers();

	/**
	 * 优雅关闭线程池
	 *
	 * @param eventLoopGroup 线程组
	 * @param name           线程组名称
	 */
	protected void shutdownEventLoopGroup(EventLoopGroup eventLoopGroup, String name) {
		if (eventLoopGroup != null) {
			try {
				Log.info("正在关闭线程组: {}", name);
				eventLoopGroup.shutdownGracefully().sync();
				Log.info("线程组 {} 已关闭", name);
			} catch (InterruptedException e) {
				Thread.currentThread().interrupt();
				Log.error("线程组 {} 关闭失败: {}", name, e.getMessage(), e);
			}
		}
	}
}
