package com.incubator.common.util;

import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

public final class DateUtils {

	private static ThreadLocal<SimpleDateFormat> formatHolder = new ThreadLocal<SimpleDateFormat>() {
		@Override
		protected synchronized SimpleDateFormat initialValue() {
			return new SimpleDateFormat();
		}
	};

	private static final long DAY_MILLIS = 86400000;
	private static final long HOUR_MILLIS = 3600000;
	private static final long MINUTE_MILLIS = 60000;
	public static final long HOUR_SECONDS = 3600;

	private static SimpleDateFormat dateFormat = formatHolder.get();

	/**
	 * @return 获取当前时间
	 */
	public static Date getCurrentTime() {
		return new Date();
	}

	public static Date getTime(int hour, int minute, int second) {
		Calendar cal = Calendar.getInstance();
		cal.set(Calendar.HOUR_OF_DAY, hour);
		cal.set(Calendar.MINUTE, minute);
		cal.set(Calendar.SECOND, second);
		return cal.getTime();
	}

	/**
	 * 将日期格式化为指定的格式 如yyyy-MM-dd HH:mm:ss
	 * 
	 * @param date
	 * @param pattern
	 * @return
	 */
	public static String format(Date date, String pattern) {
		if (date == null) {
			return null;
		}
		dateFormat.applyPattern(pattern);
		return dateFormat.format(date);
	}

	/**
	 * 将日期格式化为yyyy-MM
	 * 
	 * @param date
	 * @return
	 * @author TanGuiyuan
	 * @date 2013-1-17
	 */
	public static String formatMonth(Date date) {
		if (date == null) {
			return null;
		}
		dateFormat.applyPattern("yyyy-MM");
		return dateFormat.format(date);
	}

	/**
	 * 将日期格式化为yyyy-MM
	 * 
	 * @param date
	 * @return
	 * @author TanGuiyuan
	 * @date 2013-1-17
	 */
	public static String formatWeek(Date date) {
		if (date == null) {
			return null;
		}
		dateFormat.applyPattern("yyyy-ww");
		return dateFormat.format(date);
	}

	/**
	 * 将日期格式化为简单日期.(yyyy-MM-dd)
	 * 
	 * @param date
	 * @return
	 */
	public static String formatSimpleDate(Date date) {
		return format(date, "yyyy-MM-dd");
	}

	/**
	 * 将日期格式化为完整日期.(yyyy-MM-dd HH:mm:ss)
	 * 
	 * @param date
	 * @return
	 */
	public static String formatFullDate(Date date) {
		return format(date, "yyyy-MM-dd HH:mm:ss");
	}

	/**
	 * 将日期格式化为完整日期.(yyyyMMddHHmmss)
	 * 
	 * @param date
	 * @return
	 */
	public static String fmtFullDate(Date date) {
		return format(date, "yyyyMMddHHmmss");
	}

	/**
	 * 将日期格式化为完整日期.(yyyyMMdd)
	 * 
	 * @param date
	 * @return
	 */
	public static String fmtFullDateyyyyMMdd(Date date) {
		return format(date, "yyyyMMdd");
	}

	/**
	 * 将日期格式化为.(HH:mm)
	 * 
	 * @param date
	 * @return
	 */
	public static String formatHHmm() {
		return format(new Date(), "HH:mm");
	}

	/**
	 * 将日期格式化为.(HH:mm)
	 * 
	 * @param date
	 * @return
	 */
	public static String formatHHmm(Date date) {
		return format(date, "HH:mm");
	}

	/**
	 * 
	 * @param date
	 * @return
	 */
	public static int getHourOfDay(Date date) {
		if (date == null) {
			throw new RuntimeException("The date must be not null.");
		}
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		return cal.get(Calendar.HOUR_OF_DAY);
	}

	/**
	 * 获取<code>date</code>日期在处于当月的第几天
	 * 
	 * @param date
	 * @return
	 */
	public static int getDayOfMonth(Date date) {
		if (date == null) {
			throw new RuntimeException("The date must be not null.");
		}
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		return cal.get(Calendar.DAY_OF_MONTH);
	}

	/**
	 * 获取<code>date</code>日期在当月的最大天数
	 * 
	 * @param date
	 * @return
	 */
	public static int getMaxDayOfMonth(Date date) {
		if (date == null) {
			throw new RuntimeException("The date must be not null.");
		}
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		return cal.getActualMaximum(Calendar.DAY_OF_MONTH);
	}

	/***
	 * 获取Date日期处于星期的第几天(星期一是第一天)
	 * 
	 * @param date
	 * @return
	 * @author LuZhiYong
	 * @Date 2012-12-5
	 */
	public static int getDayOfWeek(Date date) {
		if (date == null) {
			throw new RuntimeException("The date must be not null.");
		}
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		int dayOfWeek = cal.get(Calendar.DAY_OF_WEEK);
		return (dayOfWeek == 1 ? 7 : dayOfWeek - 1);
	}

	/**
	 * @param date
	 * @return
	 */
	public static int getMonth(Date date) {
		if (date == null) {
			throw new RuntimeException("The date must be not null.");
		}
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		return cal.get(Calendar.MONTH);
	}

	/**
	 * 
	 * @param date
	 * @return
	 */
	public static int getYear(Date date) {
		if (date == null) {
			throw new RuntimeException("The date must be not null.");
		}
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		return cal.get(Calendar.YEAR);
	}

	/**
	 * 
	 * @param source
	 * @param pattern
	 * @return
	 */
	public static Date parseDate(String source, String pattern) {
		try {
			if (source == null || "".equals(source)) {
				return null;
			}
			dateFormat.applyPattern(pattern);
			return dateFormat.parse(source);
		} catch (ParseException e) {
			throw new RuntimeException("The source is not match pattern.");
		}
	}

	/**
	 * @param source
	 * @return
	 */
	public static Date parseSimpleDate(String source) {
		return parseDate(source, "yyyy-MM-dd");
	}

	/**
	 * @param source
	 * @return
	 */
	public static Date parseFullDate(String source) {
		return parseDate(source, "yyyy-MM-dd HH:mm:ss");
	}

	public static Date parseDateyyyyMMddHHmm(String source) {
		return parseDate(source, "yyyy-MM-dd HH:mm");
	}

	/**
	 * 判断两个日期否同一个月
	 * 
	 * @return
	 */
	public static boolean hasSameMonth(Date date1, Date date2) {

		if (date1 == null || date2 == null) {
			throw new RuntimeException("date1 and date2 must be not null.");
		}

		Calendar cal1 = Calendar.getInstance();
		cal1.setTime(date1);
		int y1 = cal1.get(Calendar.YEAR);
		int m1 = cal1.get(Calendar.MONTH);

		Calendar cal2 = Calendar.getInstance();
		cal2.setTime(date2);
		int y2 = cal2.get(Calendar.YEAR);
		int m2 = cal2.get(Calendar.MONTH);

		if (y1 == y2 && m1 == m2) {
			return true;
		}
		return false;
	}

	/**
	 * 判断两个日期是否同一天
	 * 
	 * @param date1
	 * @param date2
	 * @return
	 */
	public static boolean hasSameDay(Date date1, Date date2) {
		if (date1 == null || date2 == null) {
			throw new RuntimeException("date1 and date2 must be not null.");
		}

		Calendar cal1 = Calendar.getInstance();
		cal1.setTime(date1);
		int y1 = cal1.get(Calendar.YEAR);
		int m1 = cal1.get(Calendar.MONTH);
		int d1 = cal1.get(Calendar.DAY_OF_MONTH);

		Calendar cal2 = Calendar.getInstance();
		cal2.setTime(date2);
		int y2 = cal2.get(Calendar.YEAR);
		int m2 = cal2.get(Calendar.MONTH);
		int d2 = cal2.get(Calendar.DAY_OF_MONTH);

		if (y1 == y2 && m1 == m2 && d1 == d2) {
			return true;
		}
		return false;
	}

	/***
	 * 判断日期是否是今天
	 * 
	 * @param date
	 * @return
	 * @author LuZhiYong
	 * @Date 2013-7-5
	 */
	public static boolean isToday(Date date) {
		if (date == null) {
			throw new RuntimeException("date must be not null.");
		}

		Calendar cal1 = Calendar.getInstance();
		int y1 = cal1.get(Calendar.YEAR);
		int m1 = cal1.get(Calendar.MONTH);
		int d1 = cal1.get(Calendar.DAY_OF_MONTH);

		Calendar cal2 = Calendar.getInstance();
		cal2.setTime(date);
		int y2 = cal2.get(Calendar.YEAR);
		int m2 = cal2.get(Calendar.MONTH);
		int d2 = cal2.get(Calendar.DAY_OF_MONTH);

		if (y1 == y2 && m1 == m2 && d1 == d2) {
			return true;
		}
		return false;
	}

	/**
	 * 判断两个日期是否同一分钟
	 * 
	 * @param date1
	 * @param date2
	 * @return
	 */
	public static boolean hasSameMinute(Date date1, Date date2) {
		if (date1 == null || date2 == null) {
			throw new RuntimeException("date1 and date2 must be not null.");
		}

		Calendar cal1 = Calendar.getInstance();
		cal1.setTime(date1);
		int y1 = cal1.get(Calendar.YEAR);
		int m1 = cal1.get(Calendar.MONTH);
		int d1 = cal1.get(Calendar.DAY_OF_MONTH);
		int h1 = cal1.get(Calendar.HOUR_OF_DAY);
		int minute1 = cal1.get(Calendar.MINUTE);

		Calendar cal2 = Calendar.getInstance();
		cal2.setTime(date2);
		int y2 = cal2.get(Calendar.YEAR);
		int m2 = cal2.get(Calendar.MONTH);
		int d2 = cal2.get(Calendar.DAY_OF_MONTH);
		int h2 = cal2.get(Calendar.HOUR_OF_DAY);
		int minute2 = cal2.get(Calendar.MINUTE);
		if (y1 == y2 && m1 == m2 && d1 == d2 && minute1 == minute2 && h1 == h2) {
			return true;
		}
		return false;
	}

	/**
	 * 是否同一周 按照中国日历算 (周一为一周的开始)
	 * 
	 * @param date1
	 * @param date2
	 * @return
	 */
	public static boolean hasSameWeek(Date date1, Date date2) {
		if (date1 == null || date2 == null) {
			throw new RuntimeException("date1 and date2 must be not null.");
		}

		Calendar cal1 = Calendar.getInstance();
		cal1.setFirstDayOfWeek(Calendar.MONDAY);
		cal1.setMinimalDaysInFirstWeek(7);
		cal1.setTime(date1);
		int y1 = cal1.get(Calendar.YEAR);
		int week1 = cal1.get(Calendar.WEEK_OF_YEAR);
		Calendar cal2 = Calendar.getInstance();
		cal2.setFirstDayOfWeek(Calendar.MONDAY);
		cal2.setMinimalDaysInFirstWeek(7);
		cal2.setTime(date2);
		int y2 = cal2.get(Calendar.YEAR);
		int week2 = cal2.get(Calendar.WEEK_OF_YEAR);
		if (y1 == y2 && week1 == week2) {
			return true;
		}
		return false;
	}

	/**
	 * 获取每周的第一天的日期
	 * 
	 * @param date
	 * @return
	 */
	public static String getFirstWeekDate(Date date) {
		Calendar cal = Calendar.getInstance();
		cal.setFirstDayOfWeek(Calendar.MONDAY);
		cal.setMinimalDaysInFirstWeek(7);
		cal.setTime(date);
		cal.set(Calendar.DAY_OF_WEEK, cal.getFirstDayOfWeek());

		return formatSimpleDate(cal.getTime());
	}

	public static int getWeek(Date date) {
		Calendar cal = Calendar.getInstance();
		cal.setMinimalDaysInFirstWeek(7);
		return cal.get(Calendar.WEEK_OF_YEAR);
	}

	/**
	 * 获取每月的第一天的日期
	 * 
	 * @param date
	 * @return
	 */
	public static String getFirstMonthDate(Date date) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		cal.set(Calendar.DAY_OF_MONTH, cal.getActualMinimum(Calendar.DAY_OF_MONTH));
		return formatSimpleDate(cal.getTime());
	}

	/**
	 * <code> startDate</code>与<code>endDate</code>相差的月份
	 * 
	 * @param startDate
	 * @param endDate
	 * @return
	 */
	public static int betweenMonth(Date startDate, Date endDate) {
		if (startDate == null || endDate == null) {
			throw new RuntimeException("startDate and endDate must be not null.");
		}
		Calendar cal1 = Calendar.getInstance();
		cal1.setTime(startDate);
		int y1 = cal1.get(Calendar.YEAR);
		int m1 = cal1.get(Calendar.MONTH);

		Calendar cal2 = Calendar.getInstance();
		cal2.setTime(endDate);
		int y2 = cal2.get(Calendar.YEAR);
		int m2 = cal2.get(Calendar.MONTH);
		return 12 * (y2 - y1) + (m2 - m1);
	}

	/**
	 * startDate与endDate相差的天数
	 * 
	 * @param startDate
	 * @param endDate
	 * @return
	 * @throws ParseException
	 */
	public static int betweenDay(Date startDate, Date endDate) {
		if (startDate == null || endDate == null) {
			throw new RuntimeException("startDate and endDate must be not null.");
		}

		startDate = getFirstTimeOfDay(startDate);
		endDate = getFirstTimeOfDay(endDate);

		long time1 = startDate.getTime() / DAY_MILLIS;
		long time2 = endDate.getTime() / DAY_MILLIS;
		return (int) Math.abs(time1 - time2);
	}

	/**
	 * <code> startDate</code>与<code>endDate</code>相差的小时
	 * 
	 * @param startDate
	 * @param endDate
	 * @return
	 */
	public static long betweenHour(Date startDate, Date endDate) {

		if (startDate == null || endDate == null) {
			throw new RuntimeException("startDate and endDate must be not null.");
		}

		long startTime = startDate.getTime();
		long endTime = endDate.getTime();
		return (endTime - startTime) / HOUR_MILLIS;
	}

	// 计算两个时间相差小时，不足一小时按一小时算
	public static long betweenHour(Date startDate, Date endDate, boolean flag) {

		if (startDate == null || endDate == null) {
			throw new RuntimeException("startDate and endDate must be not null.");
		}

		long startTime = startDate.getTime();
		long endTime = endDate.getTime();

		long hourNum = (endTime - startTime) / HOUR_MILLIS;
		if (flag) {
			hourNum += ((endTime - startTime) % HOUR_MILLIS) == 0 ? 0 : 1;
		}
		return hourNum;
	}

	/****
	 * 之前某个时间和当前时间是否跨某一个时间点
	 * 
	 * @param optTime
	 *            上一次操作时间
	 * @param pointTime
	 *            固定刷新时间点
	 * @return
	 * @author LuZhiYong
	 * @Date 2015-3-16
	 */
	public static boolean isSpanNowTime(Date optTime, Date pointTime) {
		if (pointTime == null) {
			throw new RuntimeException("pointTime must be not null.");
		}

		if (optTime == null || betweenHour(optTime, pointTime) >= 24) {
			return true;
		}

		Date now = new Date();

		if (now.after(pointTime)) {
			if (hasSameDay(pointTime, optTime) && optTime.before(pointTime)) {
				return true;
			} else if (!hasSameDay(pointTime, optTime)) {
				return true;
			}
		}

		return false;
	}

	/**
	 * <code> startDate</code>与<code>endDate</code>相差的秒数
	 * 
	 * @param startDate
	 * @param endDate
	 * @return
	 */
	public static long betweenSeconds(Date startDate, Date endDate) {
		if (startDate == null || endDate == null) {
			throw new RuntimeException("startDate and endDate must be not null.");
		}

		long startTime = startDate.getTime();
		long endTime = endDate.getTime();
		return (endTime - startTime) / 1000;
	}

	/**
	 * <code> startDate</code>与<code>endDate</code>相差的周数
	 * 
	 * @param startDate
	 * @param endDate
	 * @return
	 */
	public static int betweenWeek(Date startDate, Date endDate) {
		if (startDate == null || endDate == null) {
			throw new RuntimeException("startDate and endDate must be not null.");
		}
		Calendar cal1 = Calendar.getInstance();
		cal1.setMinimalDaysInFirstWeek(7);
		cal1.setTime(startDate);

		Calendar cal2 = Calendar.getInstance();
		cal2.setTime(endDate);
		cal2.setMinimalDaysInFirstWeek(7);
		int y2 = cal2.get(Calendar.YEAR);
		int m2 = cal2.get(Calendar.MONTH);
		int w2 = cal2.get(Calendar.WEEK_OF_YEAR);
		int weeks = 0;
		// 开始日期大于结束日期时结束
		while (cal1.before(cal2)) {
			int y1 = cal1.get(Calendar.YEAR);
			int m1 = cal1.get(Calendar.MONTH);
			int w1 = cal1.get(Calendar.WEEK_OF_YEAR);
			// 同年同月同周时退出循环
			if (y1 == y2 && m1 == m2 && w1 == w2) {
				break;
			} else {
				weeks += 1;
				cal1.add(Calendar.DAY_OF_YEAR, 7);
			}
		}
		return weeks;
	}

	public static Date addMonths(Date date, int m) {
		if (date == null) {
			throw new RuntimeException("The date must be not NULL.");
		}
		if (m < 1) {
			throw new RuntimeException("The number of Month should be more than zero.");
		}
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		cal.add(Calendar.MONTH, m);
		return cal.getTime();
	}

	public static Date subMonths(Date date, int m) {
		if (date == null) {
			throw new RuntimeException("The date must be not NULL.");
		}
		if (m < 1) {
			throw new RuntimeException("The number of Month should be more than zero.");
		}
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		cal.add(Calendar.MONTH, -m);
		return cal.getTime();
	}

	public static Date addWeeks(Date date, int w) {
		if (date == null) {
			throw new RuntimeException("The date must be not NULL.");
		}
		if (w < 1) {
			throw new RuntimeException("The number of Week should be more than zero.");
		}
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		cal.add(Calendar.WEEK_OF_YEAR, w);
		return cal.getTime();
	}

	public static Date subWeeks(Date date, int w) {
		if (date == null) {
			throw new RuntimeException("The date must be not NULL.");
		}
		if (w < 1) {
			throw new RuntimeException("The number of Week should be more than zero.");
		}
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		cal.add(Calendar.WEEK_OF_YEAR, -w);
		return cal.getTime();
	}

	public static Date addDays(Date date, int d) {
		if (date == null) {
			throw new RuntimeException("The date must be not NULL.");
		}
		if (d < 1) {
			throw new RuntimeException("The number of Day should be more than zero.");
		}
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		cal.add(Calendar.DAY_OF_YEAR, d);
		return cal.getTime();
	}

	public static Date subDays(Date date, int d) {
		if (date == null) {
			throw new RuntimeException("The date must be not NULL.");
		}
		if (d < 1) {
			throw new RuntimeException("The number of Day should be more than zero.");
		}
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		cal.add(Calendar.DAY_OF_YEAR, -d);
		return cal.getTime();
	}

	public static Date addMinutes(Date date, int minute) {
		if (date == null) {
			throw new RuntimeException("The date must be not NULL.");
		}
		if (minute < 1) {
			throw new RuntimeException("The number of Minute should be more than zero.");
		}
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		cal.add(Calendar.MINUTE, minute);
		return cal.getTime();
	}

	public static Date addSeconds(Date date, int seconds) {
		if (date == null) {
			throw new RuntimeException("The date must be not NULL.");
		}
		if (seconds < 1) {
			throw new RuntimeException("The number of Second should be more than zero.");
		}
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		cal.add(Calendar.SECOND, seconds);
		return cal.getTime();
	}

	public static Date addHour(Date date, int h) {
		if (date == null) {
			throw new RuntimeException("The date must be not NULL.");
		}
		if (h < 1) {
			throw new RuntimeException("The number of Minute should be more than zero.");
		}
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		cal.add(Calendar.HOUR, h);
		return cal.getTime();
	}

	public static Date subMinutes(Date date, int minute) {
		if (date == null) {
			throw new RuntimeException("The date must be not NULL.");
		}
		if (minute < 1) {
			throw new RuntimeException("The number of Minute should be more than zero.");
		}
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		cal.add(Calendar.MINUTE, -minute);
		return cal.getTime();
	}

	public static Date subSecond(Date date, int second) {
		if (date == null) {
			throw new RuntimeException("The date must be not NULL.");
		}
		if (second < 1) {
			return date;
		}
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		cal.add(Calendar.SECOND, -second);
		return cal.getTime();
	}

	/**
	 * 计算两个时间相差多少分
	 * 
	 * @param time1
	 * @param time2
	 * @return
	 * @author TanGuiyuan
	 * @date 2013-3-6
	 */
	public static int betweenMinute(Date startDate, Date endDate) {
		if (startDate == null || endDate == null) {
			throw new RuntimeException("The date must be not NULL.");
		}
		int minute = (int) (endDate.getTime() - startDate.getTime()) / 60000;
		return minute;
	}

	/**
	 * 计算两个时间相差多少秒
	 * 
	 * @param time1
	 * @param time2
	 * @return
	 * @author LuZhiYong
	 * @Date 2012-8-16
	 */
	public static int betweenSecond(Date time1, Date time2) {
		if (time1 == null || time2 == null) {
			throw new RuntimeException("The date must be not NULL.");
		}

		long second = (time1.getTime() - time2.getTime()) / 1000;
		return (int) Math.abs(second);
	}

	/**
	 * 计算两个时间相差多少秒
	 * 
	 * @param time1
	 * @param time2
	 * @return 如果出现错误则返回 -1
	 * @author LuZhiYong
	 * @Date 2012-8-16
	 */
	public static int betweenSecond(Date time1, long time2) {
		if (time1 == null) {
			throw new RuntimeException("The date must be not NULL.");
		}

		long second = (time1.getTime() - time2) / 1000;
		return (int) Math.abs(second);
	}

	/**
	 * 获取今天的开始时间
	 * 
	 * @param time
	 * @return
	 * @author LuZhiYong
	 * @Date 2012-8-16
	 */
	public static Date getFirstTimeOfDay() {
		Calendar cal = Calendar.getInstance();
		cal.set(Calendar.HOUR_OF_DAY, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		return cal.getTime();
	}

	/**
	 * 获取今天的结束时间
	 * 
	 * @param time
	 * @return
	 * @author LuZhiYong
	 * @Date 2012-8-16
	 */
	public static Date getEndTimeOfDay() {
		Calendar cal = Calendar.getInstance();
		cal.set(Calendar.HOUR_OF_DAY, 23);
		cal.set(Calendar.MINUTE, 59);
		cal.set(Calendar.SECOND, 59);
		return cal.getTime();
	}

	/**
	 * 获取某一天的开始时间
	 * 
	 * @param time
	 * @return
	 * @author LuZhiYong
	 * @Date 2012-8-16
	 */
	public static Date getFirstTimeOfDay(Date time) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(time);
		cal.set(Calendar.HOUR_OF_DAY, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		return cal.getTime();
	}

	/**
	 * 获取某一天的结束时间
	 * 
	 * @param time
	 * @return
	 * @author LuZhiYong
	 * @Date 2012-8-16
	 */
	public static Date getEndTimeOfDay(Date time) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(time);
		cal.set(Calendar.HOUR_OF_DAY, 23);
		cal.set(Calendar.MINUTE, 59);
		cal.set(Calendar.SECOND, 59);
		return cal.getTime();
	}

	/**
	 * 获取某周指定的周几的时间 <BR/>
	 * Demo: 上一周 周五=[week = -1, weekDay = 5 (周一=1,周日=7)]
	 * 
	 * @param week
	 * @param weekDay
	 *            1~7
	 * @return
	 * @author LuZhiYong
	 * @Date 2012-9-10
	 */
	public static Date getOneDayTimeOfWeek(int week, int weekDay) {
		if (weekDay < 1 || weekDay > 7) {
			throw new RuntimeException("The weekDay must be 1~7.");
		}
		Calendar c = Calendar.getInstance();
		c.setFirstDayOfWeek(Calendar.MONDAY);
		c.add(Calendar.WEEK_OF_YEAR, week);

		int preNowWeekDay = c.get(Calendar.DAY_OF_WEEK);
		// 如果今天是周日则设置成上周
		if (preNowWeekDay == 1) {
			c.add(Calendar.DAY_OF_YEAR, -1);
			preNowWeekDay = c.get(Calendar.DAY_OF_WEEK);
		}

		int preSubDays = weekDay - preNowWeekDay + 1;
		c.add(Calendar.DAY_OF_WEEK, preSubDays);
		return c.getTime();
	}

	public static String formatTime(long time) {
		long min_m = time % HOUR_MILLIS;
		long sec_m = time % MINUTE_MILLIS;

		long hour = time / HOUR_MILLIS;
		long min = min_m / MINUTE_MILLIS;
		long sec = sec_m / 1000;

		String str = "";

		if (hour > 0) {
			str += hour + "H:";
		}
		if (min > 0) {
			str += min + "m:";
		}

		return str + sec + "s";
	}

	public static String formatTime_c(long time) {
		long min_m = time % HOUR_MILLIS;
		long sec_m = time % MINUTE_MILLIS;

		long hour = time / HOUR_MILLIS;
		long min = min_m / MINUTE_MILLIS;
		long sec = sec_m / 1000;

		String str = "";

		if (hour > 0) {
			str += hour + "时";
		}
		if (min > 0) {
			str += min + "分";
		}

		return str + sec + "秒";
	}

	public static String formatStr(Date date) {
		long time = System.currentTimeMillis() - date.getTime();
		long week = time / (7 * DAY_MILLIS);
		if (week > 0) {
			return week + "周前";
		}
		long day = time / DAY_MILLIS;
		if (day > 0) {
			return day + "天前";
		}
		long hour = time / HOUR_MILLIS;
		if (hour > 0) {
			return hour + "时前";
		}

		long minute = time / MINUTE_MILLIS;
		if (minute > 0) {
			return minute + "分前";
		}
		long sec = time / 1000;
		if (sec <= 0) {
			sec = 1;
		}
		return sec + "秒前";
	}

	public static long minute2expire(int min) {
		return System.currentTimeMillis() + min * 60000;
	}

	/** HH:mm:ss */
	public static int getSecond(String s) {
		String[] arr = s.split(":");
		return 3600 * Integer.parseInt(arr[0]) + 60 * Integer.parseInt(arr[1]) + Integer.parseInt(arr[2]);
	}

	/**
	 * 获取当前时间的前一天
	 * 
	 * @author HuangFu
	 * @param date
	 * @return
	 */
	public static Date getFrontHour(Date date) {
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(date);
		calendar.add(Calendar.HOUR, -1);
		date = calendar.getTime();
		return date;
	}

	/**
	 * 获取当前时间
	 * 
	 * @return
	 */
	public static String getNowTime() {
		long now = System.currentTimeMillis();
		Timestamp ts = new Timestamp(now);
		DateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		String data = sdf.format(ts);
		return data;
	}

	/**
	 * 获取当年的第几个星期
	 * 
	 * @param time
	 * @return
	 */
	public static int getWeekOfYear(long time) {
		Calendar instance = Calendar.getInstance();
		instance.setTimeInMillis(time);
		int day = instance.get(Calendar.DATE);
		instance.set(Calendar.DATE, day - 1);
		return instance.get(Calendar.WEEK_OF_YEAR);
	}

	/**
	 * 将字符串日期时间转换成Date对象
	 * 
	 * @param s
	 * @param pattern
	 * @param dv
	 * @return
	 */
	public static Date parseDateTime(String s, String pattern, Date dv) {
		try {
			SimpleDateFormat df = new SimpleDateFormat();
			if ((pattern != null) && (pattern.length() > 0))
				df.applyPattern(pattern);
			return df.parse(s);
		} catch (Exception e) {
		}
		return dv;
	}

	/**
	 * 获取当前系统前一天日期
	 * @param date
	 * @return
	 */
    public static Date getNextDay(Date date) {  
        Calendar calendar = Calendar.getInstance();  
        calendar.setTime(date);  
        calendar.add(Calendar.DAY_OF_MONTH, -1);  
        date = calendar.getTime();  
        return date;  
    } 
    
	public static Date parseDateTime(String s, String pattern) {
		return parseDateTime(s, pattern, null);
	}

	public static Date parseDateTime(String s, Date dv) {
		return parseDateTime(s, null, dv);
	}

	public static Date parseDateTime(String s) {
		return parseDateTime(s, null, null);
	}

	public static long minuteDiff(long d1, long d2) {
		return (d1 - d2) / (1000 * 60);
	}

	public static long minuteDiff(Date d1, Date d2) {
		return minuteDiff(d1.getTime(), d2.getTime());
	}

	public static long minuteDiff(String d1, String d2) {
		return minuteDiff(parseDateTime(d1), parseDateTime(d2));
	}

	public static long secondDiff(long d1, long d2) {
		return (d1 - d2) / 1000L;
	}

	public static long secondDiff(Date d1, Date d2) {
		return secondDiff(d1.getTime(), d2.getTime());
	}

	public static long secondDiff(String d1, String d2) {
		return secondDiff(parseDateTime(d1), parseDateTime(d2));
	}

	public static void main(String[] args) {
		System.out.println(getWeekOfYear(new Date().getTime()));
	}
}
