package com.incubator.core.db;

import com.incubator.common.log4j.Log;
import com.incubator.core.db.handler.ParamMapper;
import com.incubator.core.db.pool.DBFactory;
import com.incubator.core.db.pool.DBSource;

import java.math.BigDecimal;
import java.sql.*;
import java.util.Arrays;
import java.util.Date;

public abstract class DBBase {

	protected ISQLExceptionHandler __exception__ = null;
	
	// 默认数据源   
	protected static final String __DS_NAME__ = "master";
	// 主数据库
	protected static final String __DS_MASTER__ = __DS_NAME__;
	// 从数据库
	protected static final String __DS_SLAVE__ = "slave";
	
	public void run() {
		
	}
	
	protected DBBase() {
	}
	

	/** 默认数据源 */
	protected Connection getConn() {
		return getConnection(__DS_NAME__);
	}

	/** 主库数据源（用于写数据），未指定等同默认数据源 */
	protected Connection getMaster() {
		return getConnection(__DS_MASTER__);
	}

	/** 从库数据源（用于读数据），未指定等同默认数据源 */
	protected Connection getSlave() {
		return getConnection(__DS_SLAVE__);
	}
	
	
	/** 获取指定数据源 */
	protected Connection getConnection(String dsName) {
		Connection con = null;
		DBSource source = null;
		try {
			source = DBFactory.lookup(dsName);
			if (null == source) {
				Log.error("getConnection source = null, name = {}", dsName);
			}
			con = source.getConnection();
		} catch (SQLException e) {
			Log.error("GDao getConnection", e);
		}
		return con;
	}

	public void Init(ISQLExceptionHandler exceptionHandler) {
		__exception__ = exceptionHandler;
	}
	
	protected void exceptionSQL(String logName, Exception e, Object... args){
		if (__exception__ != null) {
			__exception__.exceptionSQL(logName, e);
		} else {
			StringBuilder argsData = new StringBuilder("args:");
			try {
				if (args != null) {
					for (int i = 0; i < args.length; i++) {
						argsData.append(Arrays.toString(args)).append(", ");
					}
				}
				
			} catch (Exception e1) {
				Log.warn("exceptionSQL get args catch: " + Arrays.toString(args) + ", " + e1);
			}

			Log.error(logName, e, argsData.toString());
		}
	}
	
	protected void exceptionSQL(SQLException e) {
		if (__exception__ != null) {
			__exception__.exceptionSQL(e);
		} else {
			e.printStackTrace();
		}
		Log.error("exceptionSQL - ", e);
	}
	
	/** 释放SQL语句执行 */
	protected static void free(ResultSet rs, PreparedStatement pstmt, Connection conn) {
		close(rs);
		close(pstmt);
		close(conn);
	}
	/** 释放存储过程执行 */
	protected static void free(ResultSet rs, CallableStatement cs, Connection conn) {
		close(rs);
		close(cs);
		close(conn);
	}
	/** 关闭 记录集 */
	protected static void close(ResultSet rs) {
		if (rs != null) {
			try {
				rs.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	/** 关闭 SQL */
	protected static void close(PreparedStatement pstmt) {
		if (pstmt != null) {
			try {
				pstmt.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	/** 关闭存储过程~  */
	protected static void close(CallableStatement cs) {
		if (cs != null) {
			try {
				cs.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	/** 关闭连接  */
	protected static void close(Connection conn) {
		if (conn != null) {
			try {
				conn.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}	
	
	protected static void setParams(PreparedStatement pstmt, Object[] args) throws SQLException {
		if (args == null) {
			return;
		}
		for (int i = 0; i < args.length; i++) {
			if (args[i] instanceof Integer) {
				pstmt.setInt(i + 1, (Integer) args[i]);
			} else if (args[i] instanceof String) {
				pstmt.setString(i + 1, (String) args[i]);
			} else if (args[i] instanceof Byte) {
				pstmt.setByte(i + 1, (Byte) args[i]);
			}  else if (args[i] instanceof Boolean) {
				pstmt.setByte(i + 1, (byte) (((Boolean) args[i]) ? 1 : 0));
			} else if (args[i] instanceof Short) {
				pstmt.setShort(i + 1, (Short) args[i]);
			} else if (args[i] instanceof Timestamp) {
				pstmt.setTimestamp(i + 1, (Timestamp) args[i]);
			} else if (args[i] instanceof Date) {
				pstmt.setDate(i + 1, new java.sql.Date(((Date) args[i]).getTime()));
			} else if (args[i] instanceof java.sql.Date) {
				pstmt.setDate(i + 1, new java.sql.Date(((Date) args[i]).getTime()));
			} else if (args[i] instanceof Double) {
				pstmt.setDouble(i + 1, (Double) args[i]);
			} else if (args[i] instanceof byte[]) {
				pstmt.setBytes(i + 1, (byte[]) args[i]);
			} else if (args[i] instanceof Long) {
				pstmt.setLong(i + 1, (Long) args[i]);
			} else if (args[i] instanceof BigDecimal) {
				String value = args[i].toString();
				pstmt.setBigDecimal(i + 1, new BigDecimal(value));
			} else {
				if (args[i] != null) {
					Log.warn("暂未支持的持久化数据类型，请确认: {}", args[i].getClass());
				}
				pstmt.setObject(i + 1, args[i]);
			}
		}
	}	
	
	/** 是否支持批量更新 */
	protected static boolean supportsBatchUpdates(Connection conn) throws SQLException {
		DatabaseMetaData dbmd = conn.getMetaData();
		if (dbmd != null) {
			return dbmd.supportsBatchUpdates();
		}
		return false;
	}

	/**
	 * 保存并生产主键
	 */
	protected int generated(String sql, Object... args) {
		Connection conn = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			conn = getMaster();
			pstmt = conn.prepareStatement(sql, Statement.RETURN_GENERATED_KEYS);
			setParams(pstmt, args);
			if (pstmt.executeUpdate() > 0) {
				rs = pstmt.getGeneratedKeys();
				if (rs.next()) {
					return rs.getInt(1);
				}
			}
		} catch (SQLException e) {
			exceptionSQL(e);
		} finally {
			free(rs, pstmt, conn);
		}
		return -1;
	}
	
	/**
	 * 保存并生产主键
	 */
	protected int generated(String sql, ParamMapper param) {
		Connection conn = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			conn = getMaster();
			pstmt = conn.prepareStatement(sql, Statement.RETURN_GENERATED_KEYS);
			param.setParams(pstmt, 0);
			if (pstmt.executeUpdate() > 0) {
				rs = pstmt.getGeneratedKeys();
				if (rs.next()) {
					return rs.getInt(1);
				}
			}
		} catch (SQLException e) {
			exceptionSQL(e);
		} finally {
			free(rs, pstmt, conn);
		}
		return -1;
	}
}
