package com.incubator.core.net.handler.listener;

import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.common.net.ConnectionFactory;
import com.incubator.common.net.netty.NettyConnectionFactory;
import com.incubator.common.net.netty.listener.ConnectionHeartbeatListenerFactory;

/**
 * 服务端连接监听
 *
 * 负责管理连接和心跳机制。
 *
 * @author john
 */
public abstract class GameServerConnectionListener extends ConnectionHeartbeatListenerFactory<Object> {

	@Override
	protected boolean isHeartReq(Object msg) {
		if (msg == null) {
			return false;
		}
		try {
			return doIsHeartReq(msg);
		} catch (Exception e) {
			Log.warn("Failed to parse heartbeat request: {}", e.getMessage());
			return false;
		}
	}

	@Override
	protected void sendHeartReq(Connection connection) {
		if (connection != null && connection.isActive()) {
			try {
				doSendHeartReq(connection);
				Log.debug("Heartbeat request sent to connection: {}", connection);
			} catch (Exception e) {
				Log.warn("Failed to send heartbeat request: {}", e.getMessage(), e);
			}
		}
	}

	@Override
	protected void sendHeartRsp(Connection connection) {
		if (connection != null && connection.isActive()) {
			try {
				doSendHeartRsp(connection);
				Log.debug("Heartbeat response sent to connection: {}", connection);
			} catch (Exception e) {
				Log.warn("Failed to send heartbeat response: {}", e.getMessage(), e);
			}
		}
	}

	@Override
	protected void doMessageArrived(Connection conn, Object msg) {
		if (conn == null || msg == null) {
			Log.warn("Received null connection or message.");
			return;
		}
		try {
			messageArrived(conn, msg);
		} catch (Exception e) {
			Log.error("Failed to process message: {}", e.getMessage(), e);
		}
	}

	@Override
	public final ConnectionFactory getConnectionFactory() {
		return NettyConnectionFactory.getInstance();
	}

	/**
	 * 请求到达
	 *
	 * @param conn 连接
	 * @param msg  消息
	 */
	public abstract void messageArrived(Connection conn, Object msg);

	/**
	 * 是否是心跳请求
	 *
	 * @param msg 消息
	 * @return 是否是心跳请求
	 */
	public abstract boolean doIsHeartReq(Object msg);

	/**
	 * 发送心跳请求
	 *
	 * @param connection 连接
	 */
	public abstract void doSendHeartReq(Connection connection);

	/**
	 * 发送心跳回复
	 *
	 * @param connection 连接
	 */
	public abstract void doSendHeartRsp(Connection connection);
}
