package com.incubator.core.net.ws;

import com.alibaba.fastjson2.JSON;
import org.msgpack.core.MessageBufferPacker;
import org.msgpack.core.MessagePack;
import org.msgpack.core.MessageUnpacker;
import org.msgpack.value.ValueType;

import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * ws消息传输类
 */
public abstract class MsgBase {

    /** 消息协议号 **/
    protected int cmd;
    /** 消息码 200:OK **/
    protected int code;
    /** 消息提示 **/
    protected String message = "";
    /** 消息内容 **/
    protected Map<String, Object> data = new ConcurrentHashMap<>();

    public MsgBase() {}

    public int getCmd() {
        return cmd;
    }

    public void setCmd(int msgId) {
        this.cmd = msgId;
    }

    public int getCode() {
        return code;
    }

    public void setCode(int code) {
        this.code = code;
    }

    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public Map<String, Object> getData() {
        return data;
    }

    public void setData(Map<String, Object> data) {
        this.data = data;
    }

    public String toJson() {
        return JSON.toJSONString(this);
    }

    public void reset() {
        this.cmd = 0;
        this.code = 0;
        this.message = "";
        this.data = new ConcurrentHashMap<>();
    }

    /**
     * 从字节数组反序列化为 WSRequest 对象
     */
    public void fromBytes(byte[] bytes) throws Exception {
        MessageUnpacker unpacker = MessagePack.newDefaultUnpacker(new ByteArrayInputStream(bytes));

        // 解包 Map Header，期望有 2 个键值对（"cmd" 和 "data"）
        int mapSize = unpacker.unpackMapHeader();
        for (int i = 0; i < mapSize; i++) {
            // 解包键（假设键是字符串）
            String key = unpacker.unpackString();
            switch (key) {
                case "cmd":
                    // 解包 "cmd" 的值（假设是整数）
                    this.setCmd(unpacker.unpackInt());
                    break;
                case "data":
                    // 检查是否有下一个值，如果是 `null`，跳过并设置 `data` 为 `null`
                    if (unpacker.tryUnpackNil()) {
                        this.setData(null); // 设置 `data` 为 `null`
                    } else {
                        // 解包 "data" 的值（假设是一个 Map）
                        int dataSize = unpacker.unpackMapHeader();
                        Map<String, Object> dataMap = new HashMap<>();
                        for (int j = 0; j < dataSize; j++) {
                            String dataKey = unpacker.unpackString();
                            Object dataValue = unpackNextValue(unpacker);
                            dataMap.put(dataKey, dataValue);
                        }
                        this.setData(dataMap);
                    }
                    break;
                default:
                    // 如果有未处理的键，跳过它的值
                    unpacker.skipValue();
            }
        }

        unpacker.close();
    }

    /**
     * 辅助方法：根据类型解包值
     */
    private Object unpackNextValue(MessageUnpacker unpacker) throws Exception {
        switch (unpacker.getNextFormat().getValueType()) {
            case STRING:
                return unpacker.unpackString();
            case INTEGER:
                // 根据值范围判断是否为 int 或 long
                long longValue = unpacker.unpackLong();
                if (longValue >= Integer.MIN_VALUE && longValue <= Integer.MAX_VALUE) {
                    return (int) longValue; // 转换为 int
                } else {
                    return longValue; // 返回 long
                }
            case FLOAT:
                return unpacker.unpackFloat();
            case BOOLEAN:
                return unpacker.unpackBoolean();
            case MAP:
                // 递归解包 Map
                int mapSize = unpacker.unpackMapHeader();
                Map<String, Object> map = new HashMap<>();
                for (int i = 0; i < mapSize; i++) {
                    String key = unpacker.unpackString();
                    Object value = unpackNextValue(unpacker);
                    map.put(key, value);
                }
                return map;
            case ARRAY:
                // 处理数组（如果有这种情况）
                int arraySize = unpacker.unpackArrayHeader();
                if (arraySize > 0 && unpacker.getNextFormat().getValueType() == ValueType.INTEGER) {
                    // 如果数组是整型数组
                    int[] intArray = new int[arraySize];
                    for (int i = 0; i < arraySize; i++) {
                        intArray[i] = unpacker.unpackInt();
                    }
                    return intArray;
                } else {
                    // 处理其他类型的数组
                    Object[] array = new Object[arraySize];
                    for (int i = 0; i < arraySize; i++) {
                        array[i] = unpackNextValue(unpacker);
                    }
                    return array;
                }
            default:
                // 跳过未知类型
                unpacker.skipValue();
                return null;
        }
    }

    /**
     * 将 WSResponse 序列化为字节数组
     */
    public byte[] toBytes() throws Exception {
        MessageBufferPacker packer = MessagePack.newDefaultBufferPacker();

        // 序列化成一个 Map，包含 "cmd" 和 "data"
        packer.packMapHeader(4); // Map 大小为 2
        // 序列化 "cmd"
        packer.packString("cmd");
        packer.packInt(this.cmd);
        // 序列化 "code"
        packer.packString("code");
        packer.packInt(this.code);
        // 序列化 "message"
        packer.packString("message");
        packer.packString(this.message);
        // 序列化 "data"
        packer.packString("data");
        packer.packMapHeader(this.data.size()); // 序列化 dataMap 的大小

        // 序列化 dataMap 的每个键值对
        for (Map.Entry<String, Object> entry : this.data.entrySet()) {
            // 序列化 key
            packObject(packer, entry.getKey());
            // 序列化 value
            packObject(packer, entry.getValue());
        }

        packer.close();
        return packer.toMessageBuffer().toByteArray();
    }

    /**
     * 辅助方法：根据对象类型序列化
     */
    private void packObject(MessageBufferPacker packer, Object value) throws Exception {
        if (value instanceof String) {
            packer.packString((String) value);
        } else if (value instanceof Integer) {
            packer.packInt((Integer) value);
        } else if (value instanceof Long) {
            packer.packLong((Long) value);
        } else if (value instanceof Float) {
            packer.packFloat((Float) value);
        } else if (value instanceof Boolean) {
            packer.packBoolean((Boolean) value);
        } else if (value instanceof int[]) {
            int[] intArray = (int[]) value;
            packer.packArrayHeader(intArray.length);
            for (int item : intArray) {
                packer.packInt(item);
            }
        } else if (value instanceof Integer[]) {
            Integer[] intArray = (Integer[]) value;
            packer.packArrayHeader(intArray.length);
            for (int item : intArray) {
                packer.packInt(item);
            }
        } else if (value instanceof long[]) {
            long[] intArray = (long[]) value;
            packer.packArrayHeader(intArray.length);
            for (long item : intArray) {
                packer.packLong(item);
            }
        } else if (value instanceof Long[]) {
            Long[] intArray = (Long[]) value;
            packer.packArrayHeader(intArray.length);
            for (Long item : intArray) {
                packer.packLong(item);
            }
        } else if (value instanceof String[]) {
            String[] intArray = (String[]) value;
            packer.packArrayHeader(intArray.length);
            for (String item : intArray) {
                packer.packString(item);
            }
        } else if (value instanceof ArrayList) {
            ArrayList<Object> list = (ArrayList<Object>) value;
            packer.packArrayHeader(list.size());
            for (Object item : list) {
                // 递归处理 ArrayList 的元素
                packObject(packer, item);
            }
        } else if (value instanceof Map) {
            // 递归处理嵌套的 Map
            packNestedMap(packer, (Map<String, Object>) value);
        } else {
            packer.packNil(); // 不支持的类型序列化为 Nil
        }
    }

    /**
     * 辅助方法：递归序列化嵌套的 Map
     */
    private void packNestedMap(MessageBufferPacker packer, Map<String, Object> map) throws Exception {
        packer.packMapHeader(map.size()); // Map 的大小
        for (Map.Entry<String, Object> entry : map.entrySet()) {
            // 序列化 key
            Object key = entry.getKey();
            if (key == null) {
                throw new IllegalArgumentException("Map key cannot be null");
            }
            packObject(packer, key);

            // 序列化 value
            Object value = entry.getValue();
            packObject(packer, value);
        }
    }
}
