package com.incubator.game;

import com.incubator.common.MessageHandler;
import com.incubator.common.env.Environment;
import com.incubator.common.filter.SensitiveDictionary;
import com.incubator.common.game.AbstractService;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.NetClient;
import com.incubator.common.net.NetServer;
import com.incubator.common.net.netty.client.NettyClient;
import com.incubator.common.net.netty.handler.websocket.binary.WebSocketServerBinaryAdapterHandler;
import com.incubator.common.net.netty.server.NettyServer;
import com.incubator.common.thread.NamedThreadFactory;
import com.incubator.common.util.PropertiesUtil;
import com.incubator.common.util.RandomUtil;
import com.incubator.core.db.PoolFactory;
import com.incubator.core.net.http.HttpHandler;
import com.incubator.core.quartz.QuartzServer;
import com.incubator.core.quartz.QuartzTask;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.constant.WkServerStatus;
import com.incubator.game.contest.ContestService;
import com.incubator.game.contest.ContestThreadedScheduler;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.listener.CenterClientListener;
import com.incubator.game.listener.HttpInnerListener;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.handler.InitalizerHttpServerHandler;
import com.incubator.core.net.handler.InitalizerProtocolHandler;
import com.incubator.game.data.po.GlobalInfoPO;
import com.incubator.game.data.jdbc.LSDBHandler;
import com.incubator.game.listener.PublicListener;
import com.incubator.game.player.GPlayer;
import com.incubator.game.player.PlayerPool;
import com.incubator.game.room.RoomService;
import com.incubator.game.timer.FlushDB;
import com.incubator.game.timer.RefreshNewDay;
import com.incubator.game.util.GameDataUtil;
import io.netty.util.CharsetUtil;
import org.apache.commons.lang3.StringUtils;
import org.reflections.Reflections;

import java.io.FileInputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.*;
import java.util.concurrent.*;

public class GGame extends AbstractService {

    public static ScheduledThreadPoolExecutor scheduleExec;
    /** 异步业务执行器 */
    public static Executor asyncExec;
    /** 运行环境 **/
    public static final Environment env = Environment.getInstance();
    private static final LSDBHandler lsDBHandler = new LSDBHandler();
    /** 服务器设置目录 **/
    public static PropertiesUtil SERVER_SETTINGS;
    public static PropertiesUtil DB_SETTINGS;
    public static String CONFIG_FILE = "config.properties";
    public static String DB_FILE = "db.properties";
    public static String WORDS_FILE = "filgerWords.txt";
    public static SensitiveDictionary sd;

    /** 服务器id **/
    public static int SERVER_ID = 1001;
    /** 服务器绑定IP **/
    public static String BIND_HOST;
    /** 服务器绑定端口 **/
    public static int BIND_PORT;
    /** 绑定的http ip **/
    public static String BIND_HTTP_HOST;
    /** 绑定的http 端口 **/
    public static int BIND_HTTP_PORT;
    /** 中控服绑定ip **/
    public static String CENTER_HOST;
    /** 中控服绑定端口 **/
    public static int CENTER_PORT;
    /** 策划JSON数据目录 **/
    public static String DIR_GAME_DATA = "";

    /** 玩家心跳开关 **/
    public static boolean heartEnable = false;
    /** 最大空闲玩家数量 **/
    public static int maxIdlePlayers = 100;

    /** 游戏世界的时间
     *  此两个时间最小精度为秒,不能用于超过精度一秒的时间计算
     *  (APP_TIME每隔一秒执行一下,所以最小精度一秒),方法为JobFactory.addScheduleJob
     * */
    public static volatile long APP_TIME = System.currentTimeMillis();

    public static volatile int APP_TIME_SEC = (int) (APP_TIME / 1000);


    /** 是否注册中控 **/
    public static boolean isRegCenter;

    /** ws消息事件 **/
    public static Map<Integer, NetHandler> handlers = new ConcurrentHashMap<>();

    /** http消息事件 **/
    public static Map<String, HttpHandler> httpHandlers = new ConcurrentHashMap<>();

    /** 玩家对象池 **/
    public static PlayerPool playerPool = new PlayerPool();

    /** 本服所有在线玩家 */
    public static Map<String, GPlayer> onlinePlayers;

    /** 已生成的用户名集合，确保唯一性 */
    public static Set<String> generatedUsernames = new HashSet<>();

    public static GlobalInfoPO globalInfoPO = new GlobalInfoPO();

    private static volatile GGame instance;

    public static GGame getInstance() {
        if (instance == null) {
            synchronized (GGame.class) {
                if (instance == null) {
                    instance = new GGame();
                }
            }
        }
        return instance;
    }

    protected GGame() {
        scheduleExec = new ScheduledThreadPoolExecutor(2, new NamedThreadFactory("ServerExecutor", true));
        asyncExec = new ThreadPoolExecutor(1, 5, 30, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>(), new PoolFactory("Async_Game"));

        onlinePlayers = new ConcurrentHashMap<>(2000);
    }

    /** 异步业务执行器 **/
    public void asyncExec(final Runnable command) {
        if (command == null) {
            Log.error("asyncExec command is null!");
            return;
        }
        asyncExec.execute(new Runnable() {
            @Override
            public void run() {
                try {
                    long lStartTime = System.currentTimeMillis();
                    command.run();

                    long lTimeUse = System.currentTimeMillis() - lStartTime;
                    if (lTimeUse > 1000) {
                        Log.info("asyncExec时间过长：" + lTimeUse + ", " + command);
                    }
                } catch (Exception e) {
                    Log.error("GGame asyncExec catch - ", e);
                }
            }
        });
    }

    public static LSDBHandler getDBHandler() {
        return lsDBHandler;
    }

    @Override
    protected void doStart() throws Throwable {
        // 1.初始化Properties配置
        this.initProperties();
        // 2.初始化数据库连接池
//        DBFactory.init();
        // 3.初始化游戏数据
        this.initGameData();
        // 4.初始化定时任务
//        this.initQuartzTask();
        // 5.初始化消息事件
        this.initHandler();
        // 6.初始化网络服务
        this.initNetService();
        // 7.初始化网络ip地址
        initServiceIp();
        //8 开启赛事定时任务
//        ContestThreadedScheduler.startScheduler();
    }


    private void initServiceIp() {
        //RedisUtil.set(RedisKeyConstant.WK_NUMBER+RedisKeyConstant.WK_NUMBER_VALUE, "0");
        HashMap<String, WkServerStatus> wkServe = RedisUtil.getWkToRedis(RedisKeyConstant.WK_SERVE);
        String name = RedisKeyConstant.WK_SERVE_KEY;
        String type = RedisKeyConstant.WK_SERVE_TYPE;
        WkServerStatus wkServerStatus = new WkServerStatus();
        wkServerStatus.setIntranetIp(RedisKeyConstant.WK_SERVE_IP);
        wkServerStatus.setName(name);
        wkServerStatus.setNum(0);
        wkServerStatus.setType(type);
        RedisUtil.saveWkToRedis(RedisKeyConstant.WK_SERVE, wkServerStatus);
    }

    @Override
    protected void doClose() throws Throwable {
        Log.info("服务器维护中, 已关闭服务器!");
    }

    /**
     * 1. 初始化 Properties 配置
     *
     * @throws Exception 如果配置加载失败
     */
    public void initProperties() throws Exception {
        Log.info("开始加载 Properties 配置...");

        try {
            // 加载服务器配置
            SERVER_SETTINGS = loadProperties(CONFIG_FILE);
            DB_SETTINGS = loadProperties(DB_FILE);
            sd = new SensitiveDictionary(Files.newInputStream(Paths.get(env.getConfDir() + "/" + WORDS_FILE)), CharsetUtil.UTF_8);
            // 读取服务器配置
            SERVER_ID = getInt(SERVER_SETTINGS, "SERVER_ID", 1001);
            BIND_HOST = getString(SERVER_SETTINGS, "BIND_HOST", "0.0.0.0");
            BIND_PORT = getInt(SERVER_SETTINGS, "BIND_PORT", 9000);
            BIND_HTTP_HOST = getString(SERVER_SETTINGS, "BIND_HTTP_HOST", "0.0.0.0");
            BIND_HTTP_PORT = getInt(SERVER_SETTINGS, "BIND_HTTP_PORT", 9001);
            CENTER_HOST = getString(SERVER_SETTINGS, "CENTER_HOST", "0.0.0.0");
            CENTER_PORT = getInt(SERVER_SETTINGS, "CENTER_PORT", 4000);
            DIR_GAME_DATA = getString(SERVER_SETTINGS, "DIR_GAME_DATA", "");

            Log.info("Properties 配置加载完成");
        } catch (Exception e) {
            Log.error("加载 Properties 配置失败: {}", e.getMessage(), e);
            throw new Exception("Properties 配置加载失败", e);
        }
    }

    /**
     * 加载 Properties 文件
     *
     * @param fileName 文件名
     * @return Properties 对象
     * @throws Exception 如果加载失败
     */
    private PropertiesUtil loadProperties(String fileName) throws Exception {
        String filePath = env.getConfDir() + "/" + fileName;
        Log.info("加载配置文件: {}", filePath);
        return new PropertiesUtil(filePath);
    }

    /**
     * 获取 String 类型属性值
     *
     * @param properties Properties 对象
     * @param key 属性键
     * @param defaultValue 默认值
     * @return 属性值（如果属性不存在，则返回默认值）
     */
    private String getString(PropertiesUtil properties, String key, String defaultValue) {
        return properties.getProperty(key, defaultValue);
    }

    /**
     * 获取 int 类型属性值
     *
     * @param properties Properties 对象
     * @param key 属性键
     * @param defaultValue 默认值
     * @return 属性值（如果属性不存在或格式错误，则返回默认值）
     */
    private int getInt(PropertiesUtil properties, String key, int defaultValue) {
        try {
            return Integer.parseInt(Objects.requireNonNull(properties.getProperty(key, String.valueOf(defaultValue))));
        } catch (NumberFormatException e) {
            Log.warn("属性 {} 格式错误，使用默认值 {}", key, defaultValue);
            return defaultValue;
        }
    }

    /**
     * 3.初始化游戏数据
     */
    public void initGameData() {
        //加载策划配置表
        GameDataUtil.initData();
        //加载数据库数据
//        this.initAllBlobGameData();
        //初始化 房间服务
        RoomService.getInstance().init(5);
        //初始化 官方比赛服务
        ContestService.getInstance().init(5);
    }

//    private ProtoData.GlobalAllBlobData.Builder initAllBlobGameData() {
//        ProtoData.GlobalAllBlobData.Builder allBlobData = ProtoData.GlobalAllBlobData.newBuilder();
//        try {
//            globalInfoPO = getDBHandler().getGlobalInfoByServer(SERVER_ID);
//            if (globalInfoPO == null) {
//                globalInfoPO = new GlobalInfoPO();
//                globalInfoPO.server_id = SERVER_ID;
//            }
//            if (globalInfoPO.allBlobData == null) {
//                globalInfoPO.allBlobData = ProtoData.GlobalAllBlobData.newBuilder().build().toByteArray();
//                getDBHandler().insertGlobal(globalInfoPO);
//            }
//            allBlobData = ProtoData.GlobalAllBlobData.parseFrom(globalInfoPO.allBlobData).toBuilder();
//        } catch (InvalidProtocolBufferException e) {
//            Log.error("初始化服务器数据出错...{}", e.getMessage());
//        }
//        return allBlobData;
//    }

    /**
     * 4. 初始化定时器任务
     */
    private void initQuartzTask() {
        Log.info("开始初始化定时任务...");
        try {
            QuartzServer quartzServer = new QuartzServer();

            // 定期刷新数据库任务
            addScheduledTask(quartzServer, new FlushDB(), "刷新数据库任务");

            // 每日刷新任务
            addScheduledTask(quartzServer, new RefreshNewDay(), "每日刷新任务");

            // 如果需要添加其他任务，可以在此处扩展
            // addScheduledTask(quartzServer, new OtherTask(), "其他任务");

            Log.info("定时任务初始化完成");
        } catch (Exception e) {
            Log.error("初始化定时任务时发生错误: {}", e.getMessage(), e);
        }
    }

    /**
     * 添加单个定时任务
     * @param quartzServer Quartz 任务管理器
     * @param job 任务对象
     * @param taskDescription 任务描述（用于日志输出）
     */
    private void addScheduledTask(QuartzServer quartzServer, QuartzTask job, String taskDescription) {
        try {
            quartzServer.addJob(job);
            Log.info("{} 已成功注册", taskDescription);
        } catch (Exception e) {
            Log.error("{} 注册失败: {}", taskDescription, e.getMessage(), e);
        }
    }

    /**
     * 5. 初始化消息事件
     */
    private void initHandler() {
        Log.info("初始化消息事件（基于注解）...");

        try {
            // 使用 Reflections 库扫描指定包
            Reflections reflections = new Reflections("com.incubator.game.handler");

            // 扫描 WebSocket 事件
            Set<Class<?>> annotatedClasses = reflections.getTypesAnnotatedWith(MessageHandler.class);
            for (Class<?> clazz : annotatedClasses) {
                MessageHandler annotation = clazz.getAnnotation(MessageHandler.class);
                if (annotation == null) continue;

                // 注册 WebSocket 事件
                if (annotation.id() > 0 && NetHandler.class.isAssignableFrom(clazz)) {
                    NetHandler handler = (NetHandler) clazz.getDeclaredConstructor().newInstance();
                    registerWebSocketHandler(annotation.id(), handler);
                }

                // 注册 HTTP 事件
                if (!annotation.httpPath().isEmpty() && HttpHandler.class.isAssignableFrom(clazz)) {
                    HttpHandler handler = (HttpHandler) clazz.getDeclaredConstructor().newInstance();
                    registerHttpHandler(annotation.httpPath(), handler);
                }
            }

            Log.info("消息事件初始化完成");
        } catch (Exception e) {
            Log.error("初始化消息事件时发生异常: {}", e.getMessage(), e);
        }
    }

    /**
     * 注册 WebSocket 消息事件
     */
    private void registerWebSocketHandler(int id, NetHandler handler) {
        if (handlers.containsKey(id)) {
            Log.warn("WebSocket 消息事件已存在: id={}, handler={}", id, handler.getClass().getSimpleName());
            return;
        }
        handlers.put(id, handler);
        Log.info("已注册 WebSocket 消息事件: id={}, handlerClass={}", id, handler.getClass().getSimpleName());
    }

    /**
     * 注册 HTTP 消息事件
     */
    private void registerHttpHandler(String path, HttpHandler handler) {
        if (httpHandlers.containsKey(path)) {
            Log.warn("HTTP 消息事件已存在: path={}, handler={}", path, handler.getClass().getSimpleName());
            return;
        }
        httpHandlers.put(path, handler);
        Log.info("已注册 HTTP 消息事件: path={}, handlerClass={}", path, handler.getClass().getSimpleName());
    }

    /**
     * 6. 初始化网络服务
     */
    private void initNetService() {
        Log.info("开始初始化网络服务...");

        try {
            // 初始化 WebSocket 服务
            initializeWebSocketServer();
            // 初始化 HTTP 服务
            initializeHttpServer();
            // 初始化中控连接服务（可选）
//            initializeCenterService();
            Log.info("网络服务初始化完成");
        } catch (Exception e) {
            Log.error("初始化网络服务时发生错误: {}", e.getMessage(), e);
        }
    }

    /**
     * 初始化 WebSocket 服务
     */
    private void initializeWebSocketServer() throws Exception {
        Log.info("初始化 WebSocket 服务...");
        NetServer publicServer = new NettyServer(
                BIND_HOST,
                BIND_PORT,
                new WebSocketServerBinaryAdapterHandler(
                        "/ws",
                        new InitalizerProtocolHandler(new PublicListener())
                )
        );
        publicServer.start();
        Log.info("WebSocket 服务启动成功，地址: {}:{}", BIND_HOST, BIND_PORT);
    }

    /**
     * 初始化 HTTP 服务
     */
    private void initializeHttpServer() throws Exception {
        Log.info("初始化 HTTP 服务...");
        NetServer httpServer = new NettyServer(
                BIND_HTTP_HOST,
                BIND_HTTP_PORT,
                new InitalizerHttpServerHandler(new HttpInnerListener())
        );
        httpServer.start();
        Log.info("HTTP 服务启动成功，地址: {}:{}", BIND_HTTP_HOST, BIND_HTTP_PORT);
    }

    /**
     * 初始化中控服务（可选功能）
     */
    private void initializeCenterService() {
        if (StringUtils.isEmpty(CENTER_HOST) || CENTER_PORT <= 0) {
            Log.warn("中控服务地址未配置，跳过初始化...");
            return;
        }

        Log.info("初始化中控服务...");
        try {
            NetClient centerClient = new NettyClient(
                    CENTER_HOST,
                    CENTER_PORT,
                    new WebSocketServerBinaryAdapterHandler(
                            "/ws",
                            new InitalizerProtocolHandler(new CenterClientListener())
                    )
            );
            centerClient.enableReconnect(GGame.scheduleExec, TimeUnit.SECONDS.toMillis(5));
            centerClient.setName("CenterClient");
            centerClient.start();
            Log.info("中控服务连接成功，地址: {}:{}", CENTER_HOST, CENTER_PORT);
        } catch (Exception e) {
            Log.error("中控服务连接失败: {}", e.getMessage(), e);
        }
    }

    /**
     * 生成唯一的用户名，格式为 user_123456
     *
     * @return 唯一的用户名
     */
    public static synchronized String generateUniqueUsername() {
        String username;
        do {
            username = RandomUtil.generateUsername();
        } while (generatedUsernames.contains(username));

        // 将新生成的用户名加入集合
        generatedUsernames.add(username);
        return username;
    }
}
