package com.incubator.game.contest;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONException;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson2.JSONObject;
import com.incubator.common.log4j.Log;
import com.incubator.common.util.DateUtils;
import com.incubator.common.util.StringUtils;
import com.incubator.game.GGame;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.constant.RoomStatus;
import com.incubator.game.constant.WkServerStatus;
import com.incubator.game.data.data.GameData;
import com.incubator.game.data.data.PromotionLvCO;
import com.incubator.game.data.data.ext.ContestExt;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.mq.RabbitMQUtil;
import com.incubator.game.data.po.ContestPO;
import com.incubator.game.handler.contest.GetContestListHandler;
import com.incubator.game.handler.login.SubRoomHandler;
import com.incubator.game.player.GPlayer;
import com.incubator.game.player.Player;
import com.incubator.game.room.GDRoom;
import com.incubator.game.room.Room;
import com.incubator.game.room.RoomService;
import com.incubator.game.room.ZDRoom;
import com.incubator.game.util.ContestUtil;
import com.incubator.game.util.PlayerUtil;
import com.incubator.game.util.ProtoUtil;
import com.incubator.game.util.Struct;
import com.incubator.message.proto.CommonProto;

import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * 官方比赛实体类
 */
public class Contest {

    /**
     * 比赛数据
     **/
    public ContestPO data;
    /**
     * 比赛策划表配置
     **/
    public ContestExt prop;
    /**
     * 当前晋级配置
     **/
    public PromotionLvCO propPromotionLv;

    /**
     * 激活标记
     **/
    private boolean isActive;
    /**
     * 检测标记
     **/
    public boolean flag;
    /**
     * 时间秒数
     **/
    public int time;
    /**
     * 比赛状态(0等待/报名中 1比赛开始 2进行中 3已结束)
     **/
    public int state;

    /**
     * 比赛当前状态0等待/报名中 1比赛开始 2进行中 3当局结束
     */
    public int contestState;

    /**
     * 每个房间玩家容量
     **/
    private final int roomCapacity = 4;

    /**
     * 比赛报名角色 [key:玩家id, value:玩家对象]
     **/
    public Map<String, Player> actors = new ConcurrentHashMap<>();

    /**
     * 报名玩家集合
     */
    public List<Player> contestPlayers = new ArrayList<>();

    /**
     * 比赛房间列表
     **/
    public Map<Long, Room> rooms = new ConcurrentHashMap<>();
    /**
     * 等待队列
     **/
    private final ConcurrentLinkedQueue<String> waitingPlayers = new ConcurrentLinkedQueue<>();
    /**
     * 淘汰队列
     */
    private final ConcurrentLinkedQueue<String> DieOutPlayers = new ConcurrentLinkedQueue<>();
    /**
     * 替补队列
     */
    private final ConcurrentLinkedQueue<String> SubstitutePlayers = new ConcurrentLinkedQueue<>();

    /**
     * 是否进行下一轮
     **/
    public boolean isNext = true;

    /**
     * 比赛任务主线程
     **/
    private ScheduledExecutorService scheduler;

    public Contest() {
        this.scheduler = Executors.newSingleThreadScheduledExecutor();
    }

    /**
     * 初始化
     *
     * @param roomType  比赛玩法类型
     * @param contestId 比赛id
     * @param prop      比赛策划表配置
     */
    public void init(int roomType, long contestId, ContestExt prop) {
        this.prop = prop;
        this.prop.competeType = prop.competeType;
        this.propPromotionLv = new PromotionLvCO();
        this.propPromotionLv.iD = contestId;
        this.propPromotionLv.max = prop.getMax();
        this.propPromotionLv.min = prop.getMin();
        // 初始化比赛
        this.data = new ContestPO();
        this.data.contestId = contestId;
        this.data.startTime = LocalDateTime.parse(this.prop.openTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")).atZone(ZoneId.of("Asia/Shanghai")).toInstant().toEpochMilli();
        this.data.level = this.prop.level;
        if (this.prop.getStates() == 2){
            this.state = 0;
        }else if (this.prop.getStates() == 3){
            this.state = 2;
        }else {
            this.state = 3;
        }
    }

    public void LobbyInit(long contestId, ContestExt prop) {
        this.prop = prop;
        this.prop.competeType = prop.competeType;
        this.propPromotionLv = new PromotionLvCO();
        this.propPromotionLv.iD = contestId;
        this.propPromotionLv.max = prop.getMax();
        this.propPromotionLv.min = prop.getMin();
        // 初始化比赛
        this.data = new ContestPO();
        this.data.contestId = contestId;
        this.data.startTime = LocalDateTime.parse(this.prop.openTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")).atZone(ZoneId.of("Asia/Shanghai")).toInstant().toEpochMilli();
        this.data.level = this.prop.level;
        List<Player> actors = this.prop.actors;
        if(actors!=null&&!actors.isEmpty()){
            this.actors = actors.stream()
                    .collect(Collectors.toMap(Player::getId, player -> player));
        }
        if (this.prop.getStates() == 2){
            this.state = 0;
        }else if (this.prop.getStates() == 3){
            this.state = 2;
        }else {
            this.state = 3;
        }
    }



    /**
     * 根据当前人数刷新晋级配置
     *
     * @param num 实际报名人数
     */
    public void refreshPromotionLv(int num) {
        if (num >= this.propPromotionLv.min && num <= this.propPromotionLv.max) {
            this.propPromotionLv.num = num / 2;
        }
    }

    /**
     * 玩家报名加入比赛
     *
     * @param player
     */
    public synchronized void joinContest(Player player) {
        // 加入比赛角色缓存
        if (!this.actors.containsKey(player.getId())) {
            this.actors.put(player.getId(), player);
        }
        if (!this.data.playerList.contains(player.getId())) {
            this.data.playerList.add(player.getId());
        }
        // 加入等待队列
        if (!this.waitingPlayers.contains(player.getId())) {
            this.waitingPlayers.add(player.getId());
        }
        boolean isPlayerInList = contestPlayers.stream()
                .anyMatch(currentPlayer -> currentPlayer.equals(player));
        if (!isPlayerInList) {
            this.contestPlayers.add(player);
        }
        // 根据报名人数,刷新晋级配置
        refreshPromotionLv(this.actors.size());
        if (this.propPromotionLv == null) {
            Log.error("根据报名人数刷新晋级配置出错...比赛 : {}, {}, 人数 : {}",
                    this.data.contestId, this.prop.desc, this.actors.size());
        }
        //给其他人推送当前
        GGame.onlinePlayers.forEach((id, p) -> {
            Map<String, Object> map = ProtoUtil.contestInfoToMessage(this);
            p.receive(CommonProto.Cmd.GetSignUpNum_VALUE, map);
        });
        updateRedis();
    }

    /**
     * 玩家退出报名
     *
     * @param player
     */
    public synchronized void quitContest(Player player) {
        this.actors.remove(player.getId());
        this.data.playerList.remove(player.getId());
        this.waitingPlayers.remove(player.getId());
        this.contestPlayers.remove(player);
        // 根据报名人数,刷新晋级配置
        refreshPromotionLv(this.actors.size());
        if (this.propPromotionLv == null) {
            Log.error("根据报名人数刷新晋级配置出错...比赛 : {}, {}, 人数 : {}",
                    this.data.contestId, this.prop.desc, this.actors.size());
        }
        //给其他人推送当前
        this.actors.forEach((id, p) -> {
            Map<String, Object> map = ProtoUtil.contestInfoToMessage(this);
            p.receive(CommonProto.Cmd.GetSignUpNum_VALUE, map);
        });
        updateRedis();
    }

    /**
     * 更新redis
     */
    public void updateRedis() {
        String competeListStr = RedisUtil.get(RedisKeyConstant.COMPETE_LIST);
        JSONArray jsonArray = null;
        try {
            jsonArray = new JSONArray(competeListStr);
            Log.info("赛事信息获取成功...");
        } catch (JSONException e) {
            e.printStackTrace();
            Log.error("JSON信息转换异常: {}", e.getMessage(), e);
            return;
        }
        for (Object o : jsonArray) {
            cn.hutool.json.JSONObject jsonObject = (cn.hutool.json.JSONObject) o;
            int iD = (Integer)jsonObject.get("iD") ;
            if (this.data.contestId!=iD)continue;
            jsonObject.put("actors",this.actors);
        }
    }


    /**
     * 加入候补
     */
    public synchronized void candidateContest(Player player) {
        // 加入候补队列
        if (!this.SubstitutePlayers.contains(player.getId())) {
            this.SubstitutePlayers.add(player.getId());
        }
    }

    /**
     * 开启官方比赛主线程
     */
    public void startContestTask() {
        this.isActive = this.prop.getRunStatus().equals("Y")&&(this.prop.states==2||this.prop.states==3);//运行状态为运行中和赛事状态为报名中则开启线程
        this.scheduler.scheduleAtFixedRate(() -> {
            try {
                if (this.isActive) {
                    this.doUpdate();
                }
            } catch (Exception e) {
                this.isActive = false;
                this.state = 3; //已结束
                e.printStackTrace();
                Log.error("官方比赛执行出错...{}", e);
                //移除所有玩家
                this.actors.forEach((playerId, player1) -> {
                    player1.data.contestId = 0;
                    player1.data.roomId = 0;
                    Map<String, Object> data = ProtoUtil.contestInfoToMessage(this);
                    data.put("message", "当前比赛异常结束,参加的费用将会原路返回");
                    player1.receive(CommonProto.Cmd.EndOfRace_VALUE, data);
                });
            }
        }, 0, 1, TimeUnit.SECONDS);
    }

    /**
     * 比赛主线程逻辑
     */
    public void doUpdate() {
        switch (this.contestState) {
            case 0:
                // 等待开始
//                Date date = new Date();
//                if (date.getTime() >= this.data.startTime - (30 * 1000)){
//                    for (Player tmPlayer : this.actors.values()) {
//                        if (tmPlayer != null) {
//                            Map<String, Object> data = new HashMap<>();
//                            HashMap<String, WkServerStatus> wkServes = RedisUtil.getWkToRedis(RedisKeyConstant.WK_SERVE);
//                            AtomicReference<String> wkIp = new AtomicReference<>("");
//                            wkServes.forEach((k,v) ->{
//                                if (v.getType().equals("1")){
//                                    wkIp.set(v.getName());
//                                }
//                            });
//                            data.put("wkIp", wkIp.toString());
//                            Log.debug("比赛准备开始 : 比赛 : {},切换ip: {}", this.data.contestId, wkIp);
//                            tmPlayer.receive(CommonProto.Cmd.ToggleIPRes_VALUE, data);
//                        }
//                    }
//                    List<String> keyList = new ArrayList<>(actors.keySet());
//                    //将所有的玩家加入到赛事
//                    RedisUtil.set(RedisKeyConstant.COMPETE_PLAYERS+this.data.contestId, keyList);
//                }
                //从redis中获取玩家
//                String ids = RedisUtil.get(RedisKeyConstant.COMPETE_PLAYERS+this.data.contestId);
//                if (ids!=null){
//                    JSONArray jsonArray = new JSONArray(ids);
//                    for (Object o : jsonArray) {
//                        String playerId = (String) o;
//                        Player player = (Player)PlayerUtil.getOnlinePlayer(playerId);
//                        if (player != null) {
//                            this.actors.put(playerId, player);
//                        }
//                    }
//                }
                if (this.checkReadyStart()) {
                    this.state = 1;
                    this.contestState = 1;
                    this.flag = false;
                    this.time = 0;
                }
                break;
            case 1:
                // 比赛开始
                if (!this.flag) {
                    //开始
                    if (this.time >= 3) {
                        Log.debug("检测: 比赛id:{}, 比赛人数:{}, 状态:比赛开始...", this.data.contestId, this.waitingPlayers.size());
                        this.state = 2;
                        this.contestState = 2;
                        this.doStart();
                        this.flag = false;
                        this.time = 0;
                    }
                }
                break;
            case 2:
                // 进行中
                if (!this.flag) {
                    // 检测当轮是否结束
                    if (this.checkRoomsAllOver()) {
                        break;
                    }
                }
                break;
            case 3:
                // 下一轮
                if (this.isNext && this.time >= 27) {
//                    this.state = 1;
                    this.contestState = 1;
                    this.flag = false;
                    this.time = 0;
                }
                break;
        }

        this.time += 1;
    }

    /**
     * 检测是否可以开始 默认3秒后比赛开始
     *
     * @return
     */
    public boolean checkReadyStart() {
        Date now = new Date();
//        if (!this.isStart(now) || this.state != 0) {
        if (!this.isStart(now) || this.contestState != 0) {
            return false;
        }

        // 广播3秒倒计时
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null) {
                Map<String, Object> data = ProtoUtil.contestInfoToMessage(this);
                data.put("contestState", 1);
                data.put("contestTime", this.data.startTime);
                Log.debug("比赛准备开始 : 比赛 : {}, 开始时间 : {}, 人数 : {}", this.data.contestId, DateUtils.formatFullDate(new Date(this.data.startTime)), this.actors.size());
                tmPlayer.receive(CommonProto.Cmd.MatchPreparationStart_VALUE, data);
            }
        }

        return true;
    }

    /**
     * 比赛是否可开始
     *
     * @return
     */
    public boolean isStart(Date date) {
        // 时间是否到了
        return date.getTime() >= this.data.startTime - (3 * 1000);
    }

    /**
     * 根据玩家实时人数，动态分配比赛桌子
     */
    public synchronized void allocateRooms() {
        // 房间最大局数 分为预赛和决赛
        if (this.waitingPlayers.size() <= 4) {//决赛
            this.prop.maxRound = this.prop.finalNum;
        } else {
            this.prop.maxRound = this.prop.readyNum;
        }
        while (this.waitingPlayers.size() >= roomCapacity) {
            // 玩家匹配,分配房间
            JSONObject jsonData = new JSONObject();
            cn.hutool.json.JSONObject rule = this.prop.rule;
            jsonData.put("upgradeType", rule.get("upgradeType"));
            jsonData.put("flippingType", rule.get("flippingType"));
            jsonData.put("advanceType", rule.get("advanceType"));
            jsonData.put("timeoutType", rule.get("timeoutType"));
            jsonData.put("voiceType", rule.get("voiceType"));
            jsonData.put("recordType", rule.get("recordType"));
            jsonData.put("watchType", rule.get("watchType"));
            jsonData.put("tc", rule.get("levelType")==null?0:rule.get("levelType"));
            jsonData.put("pv", rule.get("pointsType")==null?0:rule.get("pointsType"));
            Integer doublingType =(Integer) (rule.get("doublingType") == null ? 0 : rule.get("doublingType"));
            doublingType = setDoublingType(doublingType);
            jsonData.put("dv", doublingType);
//            jsonData.put("team", rule.get("teamType")==null?0:rule.get("teamType"));
            jsonData.put("team",0);
            // 房间id
            jsonData.put("roomId", RoomService.getInstance().randomRoomIdByContest());
//            HashMap<String, RoomStatus> wkToRedis = RedisUtil.getRoomToRedis(RedisKeyConstant.WK_ROOM_MAP);
//            jsonData.put("roomId", SubRoomHandler.randomRoomId7(wkToRedis));
//
//            //保存房间信息
//            RoomStatus roomStatus = new RoomStatus();
//            roomStatus.setRoomId(jsonData.getLong("roomId"));
//            roomStatus.setRoomType(1);
//            //查询服务器信息
//            HashMap<String, WkServerStatus> wkServes = RedisUtil.getWkToRedis(RedisKeyConstant.WK_SERVE);
//            ArrayList<WkServerStatus> wkServerStatuses = new ArrayList<>();
//            for (Map.Entry<String, WkServerStatus> entry : wkServes.entrySet()) {
//                String type1 = entry.getValue().getType();
//                if (type1.equals(String.valueOf(1))){
//                    wkServerStatuses.add(entry.getValue());
//                }
//            }
//            String name = SubRoomHandler.selectLeastPopulatedServer(wkServerStatuses).get().getName();
//            roomStatus.setName(name);
//            // 保存房间信息
//            RedisUtil.saveRoomToRedis(RedisKeyConstant.WK_ROOM_MAP, roomStatus);


            // 人数模式 2:2人 4:4人
            jsonData.put("playerNum", 4);
            Integer playingTypeOne = (Integer)rule.get("playingTypeOne");
            Integer playingTypeTwo = (Integer)rule.get("playingTypeTwo");
            playingTypeOne = setTypOne(playingTypeTwo);
            // 玩法1 1:连续打，2:按局数，3:按时间
            jsonData.put("mode", playingTypeOne);
            if (playingTypeOne==2){ //按局数
                jsonData.put("modev", this.prop.maxRound);
            }else {//按时间
                //	 * 4:4局 6:6局 8:8局 10:10局
                //	 * 20:20分钟 30:30分钟 40:40分钟 60:60分钟
                jsonData.put("modev", playingTypeTwo);
            }
            // 房间类型 0:好友房 1:官方比赛房
            jsonData.put("type", 1);
            jsonData.put("contestId", this.data.contestId);
            // 房间玩法类型 1:掼蛋
//            jsonData.put("gameType", 1);
            jsonData.put("gameType", rule.get("gameType"));

            jsonData.put("isTribute", false);

            Room room = null;
            try {
                room = RoomService.getInstance().creatRoom(null, 2, jsonData);
            } catch (Exception e) {
                Log.error("比赛开始...创建房间出错...", e);
            }
            if (room == null) {
                Log.error("比赛开始...创建房间出错...比赛 : {}, name : {}", this.data.contestId, this.prop.desc);
                return;
            }

            // 开启房间主线程
//            room.startRoomTask();

            // 添加房间缓存
            this.rooms.put(room.data.roomId, room);

            Log.info("比赛开始...比赛 : {}, name : {}, Room created: id={}, data={}",
                    this.data.contestId, this.prop.desc, room.data.roomId, room.jsonData);

            // 推送广播,拉玩家进房间
            for (int i = 0; i < this.roomCapacity; i++) {
                String playerId = this.waitingPlayers.poll();
                if (!StringUtils.isEmpty(playerId)) {
                    Player tmPlayer = (Player) PlayerUtil.getOnlinePlayer(playerId);
                    if (tmPlayer == null) {
                        //判断替补队列是否有替补玩家
                        ArrayList<String> objects = new ArrayList<>(this.SubstitutePlayers);
                        if (objects.size() > 0) {
                            for (String object : objects) {
                                tmPlayer = (Player) PlayerUtil.getOnlinePlayer(object);
                                if (tmPlayer != null) {
                                    break;
                                }
                            }
                        }
                        // todo 此处首轮需要替补机器人
                        continue;
                    }
                    Map<String, Object> data = ProtoUtil.contestInfoToMessage(this);
                    data.put("roomId", room.data.roomId);
                    data.put("roomType", room.data.type);
                    data.put("type", 1);
//                    data.put("wkIp", roomStatus.getName());
                    Log.debug("比赛开始 : 比赛 : {}, 玩家 : {}", this.data.contestId, this.actors);
                    tmPlayer.receive(CommonProto.Cmd.MatchOfficialStart_VALUE, data);
                }
            }
        }

        // 比赛房间缓存列表
//        Map<Long, List<String>> tmpRooms = this.data.rooms.getOrDefault(this.data.curRound, new HashMap<>());
//        this.data.rooms.put(this.data.curRound, tmpRooms);
    }

    private Integer setTypOne(Integer playingTypeOne){
        switch (playingTypeOne){
            case 1: return 2;
            case 2: return 3;
            default: return 2;
        }
    }

    private Integer setDoublingType(Integer playingTypeOne){
        switch (playingTypeOne){
            case 1: return 0;
            case 2: return 1;
            case 3: return 2;
            default: return 0;
        }
    }

    /**
     * 比赛开始
     */
    public void doStart() {
        cn.hutool.json.JSONObject object = new cn.hutool.json.JSONObject();
        object.put("type", "updateCompete");
        // 设置比赛数据
        this.data.curRound += 1;
        // 检测比赛人数是否满足最小人数配置
        if (this.data.curRound == 1 && this.actors.size() < this.propPromotionLv.min) {
            // 流局结束
            for (Player tmPlayer : this.actors.values()) {
                if (tmPlayer != null) {
                    Map<String, Object> data = ProtoUtil.contestInfoToMessage(this);
                    data.put("contestState", 3);
                    data.put("contestTime", new Date());
                    Log.debug("比赛结束 : 比赛 : {}, 结束时间 : {}, 人数 : {}", this.data.contestId, DateUtils.formatFullDate(new Date(this.data.startTime)), this.actors.size());
                    tmPlayer.receive(CommonProto.Cmd.FlowOffice_VALUE, data);
                    tmPlayer.data.contestId = 0;
                    tmPlayer.data.matchPoints = 0;
                }
            }
            this.state = 3;
            this.isActive = false;
            object.put("status", "5");
            object.put("competeId", this.data.contestId);
            try {
                RabbitMQUtil.sendMessage(object.toString());
            } catch (Exception e) {
                e.printStackTrace();
            }
            return;
        }
        object.put("status", "3");
        object.put("num", this.actors.size());
        object.put("competeId", this.data.contestId);
        try {
            RabbitMQUtil.sendMessage(object.toString());
        } catch (Exception e) {
            e.printStackTrace();
        }
        int i = this.waitingPlayers.size() % 4;
        if (i != 0) {
            List<String> list = new ArrayList<>(this.waitingPlayers);
            // 打乱列表顺序
            Collections.shuffle(list);
            // 选择前 i 个元素
            List<String> poppedPlayers = list.subList(0, i);
            // 从原队列中移除这些元素
            for (String player : poppedPlayers) {
                this.waitingPlayers.remove(player);
                this.DieOutPlayers.add(player);
                Player tmPlayer = (Player) PlayerUtil.getOnlinePlayer(player);
                Map<String, Object> data = ProtoUtil.contestInfoToMessage(this);
                data.put("contestState", 3);
                data.put("contestTime", new Date());
                data.put("message", "您当前已被候补");
                tmPlayer.receive(CommonProto.Cmd.DieOutPlayer_VALUE, data);
            }
        }
        // 动态分配比赛房间
        this.allocateRooms();
        Log.info("当前轮次 :{}", this.data.curRound);
    }

    /**
     * 检测桌子是否全部结束
     */
    public boolean checkRoomsAllOver() {
        int cnt = 0;
        for (Room room : this.rooms.values()) {
            if (room == null) {
                continue;
            }
//            switch (room.data.gameType) {
//                case 1:
            if (room.data.curRound >= this.prop.maxRound && room.state == 3) {
                cnt += 1;
            }
//                    break;
//            }
        }

        if (cnt == this.rooms.size()) {
            this.flag = true;
            //统计分数 晋级
            this.promotionLv();
            this.propPromotionLv.num = this.rooms.size() / 2;
            if (this.rooms.size() > 1) {
                // 结束
                this.gameOver();
                //不结算 ,下一轮
                this.isNext = true;
                //清理所有房间数据
                this.rooms.clear();
            } else {
                this.contestState = 3;
                this.state = 3;
                //结束比赛
                this.isActive = false;
                this.actors.forEach((playerId, player1) -> {
                    player1.data.contestId = 0;
                    player1.data.roomId = 0;
                    player1.data.matchPoints = 0;
                    player1.data.accumulatedPoints = 10000;
                });
                Log.info("比赛结束 : 比赛 : {}, 结束时间 : {}", this.data.contestId, DateUtils.formatFullDate(new Date()));
                //给在线玩家发送比赛列表
                Map<String, GPlayer> onlinePlayers = GGame.onlinePlayers;
                for (GPlayer player : onlinePlayers.values()) {
                    Map<String, Object> map = new HashMap<>();
                    // 官方比赛信息列表
                    map.put("contestInfo", GetContestListHandler.contestInfoList((Player) player));
                    player.receive(CommonProto.Cmd.GetContestListRes_VALUE, map);
                }
                //发送比赛结束消息
                cn.hutool.json.JSONObject object = new cn.hutool.json.JSONObject();
                object.put("type", "updateCompete");
                object.put("status", "4");
                object.put("competeId", this.data.contestId);
                try {
                    RabbitMQUtil.sendMessage(object.toString());
                } catch (Exception e) {
                    e.printStackTrace();
                }
                rooms.values().forEach(Room::resetNextRoom);

                //组装赛况数据
//                List<Map<String, Object>> mapList = assembleContest(this);
//                RedisUtil.set(RedisKeyConstant.COMPETE_RESULT+this.data.contestId, mapList);
            }
            return true;
        }
        return false;
    }

    private List<Map<String, Object>> assembleContest(Contest contest){
        List<Map<String, Object>> objects = new ArrayList<>();
        Map<Integer, Map<Long, List<Struct.PromotionData>>> promotionMap = contest.data.promotionMap;
        if (promotionMap != null && !promotionMap.isEmpty()){
            promotionMap.forEach((curRound, promotionDataMap) -> {
                HashMap<String, Object> curRoundMap = new HashMap<>();
                long total = promotionDataMap.values().stream().mapToLong(List::size).sum();
                curRoundMap.put("particiNum", total);// 参赛人数
                curRoundMap.put("ADVNum",total==4?0:total/2);// 晋级人数
                long count = contest.rooms.values().stream()
                        .filter(room -> room.state == 3)
                        .count();//未完成的桌数
                curRoundMap.put("ITNum", count);
                curRoundMap.put("pAdv", false);
                ArrayList<Map<String, Object>> roomList = new ArrayList<>();
                promotionDataMap.forEach((roomId, promotionDataList) -> {
                    HashMap<String, Object> roomMap = new HashMap<>();
                    Room room = RoomService.getInstance().getRoom(roomId);
                    roomMap.put("tStatus",room.state);
                    roomMap.put("tTime",room.time);
                    ArrayList<Map<String,Object>> playerList = new ArrayList<>();
                    for (Struct.PromotionData promotionData : promotionDataList) {
                        HashMap<String, Object> playerMap = new HashMap<>();
                        playerMap.put("id",promotionData.playerId);
                        playerMap.put("avatar",promotionData.avatar);
                        playerMap.put("name",promotionData.name);
                        playerMap.put("source",promotionData.score);
                        playerMap.put("isAdv",promotionData.isPromotion);
                        playerMap.put("pRank",promotionData.rank);
                        playerList.add(playerMap);
//                        if (promotionData.playerId.equals(session.getPlayerId())){
//                            curRoundMap.put("pAdv", promotionData.isPromotion);
//                            curRoundMap.put("pRank", promotionData.rank);
//                        }
                    }
                    roomMap.put("pList",playerList);
                    roomList.add(roomMap);
                });
                curRoundMap.put("TList",roomList);

                objects.add(curRoundMap);
            });
        }
        return objects;
    }


    /**
     * 统计分数 晋级
     */
    private void promotionLv() {
        Map<Long, List<Struct.PromotionData>> roomMap = this.data.promotionMap.getOrDefault(this.data.curRound, new HashMap<>());

        List<Player> objects = new ArrayList<>();
        for (Room room : this.rooms.values()) {
            if (room == null) {
                continue;
            }
            List<Struct.PromotionData> list = roomMap.getOrDefault(room.data.roomId, new ArrayList<>());
            if (room.data.result.size() >= room.data.maxNum) {
                for (int i = 1; i <= room.data.result.size(); i++) {
                    Struct.SettlementData settlementData = room.data.result.get(i);
                    if (settlementData != null) {
                        Player tmPlayer = this.actors.getOrDefault(settlementData.playerId, null);
                        if (tmPlayer != null) {
                            list.add(new Struct.PromotionData(tmPlayer.getId(), tmPlayer.data.avatarUrl,tmPlayer.data.name , tmPlayer.data.accumulatedPoints-10000,i,this.rooms.size() == 1));
                            tmPlayer.data.matchPoints += tmPlayer.data.accumulatedPoints-10000;
                            tmPlayer.data.accumulatedPoints = 10000;
                            objects.add(tmPlayer);
                        }
                    }
                }
            }
            roomMap.put(room.data.roomId, list);
        }
        //排序
        objects.sort((o1, o2) -> Long.compare(o2.data.matchPoints, o1.data.matchPoints));
        //结算
        for (int i = 1; i <= objects.size(); i++) {
            Player player = objects.get(i - 1);
            player.data.roomId = 0;
            Map<String, Object> messageMap = new HashMap<>();
            String message;
            int cmd;
            //如果只有一张桌子则直接结束
            if (this.rooms.size() <= 1) {
                HashMap<String, Object> map = new HashMap<>();
                JSONArray rewards = this.prop.rewards;
                switch (i) {
                    case 1:
                        processRank(1, player, rewards, map);
                        break;
                    case 2:
                        processRank(2, player, rewards, map);
                        break;
                    case 3:
                        processRank(3, player, rewards, map);
                        break;
                    case 4:
                        processRank(4, player, rewards, map);
                        break;
                }
                cmd = CommonProto.Cmd.FirstPlayer_VALUE;
                map.put("QR", "");
                // 创建一个Date对象，表示当前时间
                Date now = new Date();
                // 定义日期时间格式
                SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                // 格式化Date对象为字符串
                String formattedDateTime = formatter.format(now);
                map.put("date", formattedDateTime);
                map.put("contestName", this.prop.desc);
                messageMap.put("message", map);
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }

                player.receive(cmd, messageMap);

                cn.hutool.json.JSONObject object = new cn.hutool.json.JSONObject();
                object.put("type", "GetCompeteReward");
                object.put("userId", player.data.playerId);
                object.put("competeId", this.data.contestId);
                object.put("rank", i);
                try {
                    RabbitMQUtil.sendMessage(object.toString());
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else {
                if (i <= this.propPromotionLv.num) {
                    roomMap.forEach((key, value) -> {
                        Optional<Struct.PromotionData> first = value.stream()
                                .filter(promotionData -> promotionData.getPlayerId().equals(player.getId()))
                                .findFirst();
                        if (first.isPresent()) {
                            Struct.PromotionData promotionData = first.get();
                            promotionData.setPromotion(true);
                        }
                    });
                    message = "恭喜你晋级下一轮比赛";
                    this.waitingPlayers.add(player.getId());
                    cmd = CommonProto.Cmd.PromotePlayer_VALUE;
                    messageMap.put("message", message);
                    // 往后延迟30秒开始下一轮比赛
                    long nextStartTime = LocalDateTime.now().plusSeconds(30).atZone(ZoneId.systemDefault()).toInstant().toEpochMilli();
                    messageMap.put("nextStartTime", nextStartTime);
                    try {
                        Thread.sleep(1000);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                    player.receive(cmd, messageMap);
                } else {
                    roomMap.forEach((key, value) -> {
                        Optional<Struct.PromotionData> first = value.stream()
                                .filter(promotionData -> promotionData.getPlayerId().equals(player.getId()))
                                .findFirst();
                        if (first.isPresent()) {
                            Struct.PromotionData promotionData = first.get();
                            promotionData.setPromotion(false);
                        }
                    });
                    message = "很抱歉,您已被淘汰";
                    this.waitingPlayers.remove(player.getId());
                    this.DieOutPlayers.add(player.getId());
                    cmd = CommonProto.Cmd.DieOutPlayer_VALUE;
                    //判断当前比赛能否替补
                    if (this.prop.substituteRule==1){
                        message = message+",本次赛事可候补,是否花费" + this.prop.substituteCost + "钻石替补";
                    }
                    messageMap.put("message", message);
                    messageMap.put("substituteRule", this.prop.substituteRule);
                    try {
                        Thread.sleep(1000);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                    player.receive(cmd, messageMap);
                }
            }
        }
        this.data.promotionMap.put(this.data.curRound, roomMap);
    }


    private void processRank(int rank, Player player, JSONArray rewards, Map<String, Object> map) {
        if (rewards != null) {
            StringBuilder msg = new StringBuilder();
            for (Object o : rewards) {
                cn.hutool.json.JSONObject reward = (cn.hutool.json.JSONObject) o;
                String rank1 = (String) reward.get("rank");
                String[] split = rank1.split("-");
                if (split.length > 1) {
                    int min = Integer.parseInt(split[0]);
                    int max = Integer.parseInt(split[1]);
                    if (rank >= min && rank <= max) {
                        getReward(reward, msg);//领奖品处理
                    }
                } else {
                    if (Integer.parseInt(split[0]) == rank) {
                        getReward(reward, msg);//领奖品处理
                    }
                }
            }
            map.put("award", msg.toString());
        }
        map.put("rank", rank);
        map.put("name", player.data.name);
    }

    private static void getReward(cn.hutool.json.JSONObject reward, StringBuilder msg) {
        if (reward == null || !reward.containsKey("reward")) {
            return;
        }
        JSONArray objects = new JSONArray(reward.get("reward"));
        if (objects == null || objects.isEmpty()) {
            return;
        }
        List<String> rewards = objects.stream()
                .map(obj -> {
                    cn.hutool.json.JSONObject o = (cn.hutool.json.JSONObject) obj;
                    return o.getStr("goodsName") + "x" + o.getInt("goodsNum");
                })
                .collect(Collectors.toList());
        msg.append(String.join(",", rewards));
    }

    /**
     * 比赛结束
     */
    public void gameOver() {
        //清空房间玩家
//        this.rooms.forEach((id, room) -> {
//            room.actors.clear();
//        });
//        this.state = 3;
        this.contestState = 3;
        this.time = 0;
        Log.debug("比赛本轮结束...比赛 : {}, {}, 人数 : {}, 当前轮次 : {}",
                this.data.contestId, this.prop.desc, this.actors.size(), this.data.curRound);
    }

    /**
     * 比赛重置逻辑
     */
    public void resetContest() {

    }

    /**
     * 重置比赛，用于对象池回收前清理状态
     */
    public void reset() {
        this.isActive = false;
    }

    /**
     * 销毁比赛，释放资源
     */
    public void destroy() {
        this.reset();
        this.scheduler.shutdownNow();
        this.scheduler = null;
        // 从缓存中移除
        ContestService.getInstance().contestMap.remove(this.data.contestId);
        // 重置数据
        this.resetContest();
        // 归还对象
        ContestService.getInstance().contestPool.releaseContest(this);
        Log.debug("Contest {} has been destroyed.", this.data.contestId);
    }
}
