package com.incubator.game.contest;

import java.util.Queue;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 比赛对象池
 */
public final class ContestPool {

    /** 官方比赛对象池 **/
    private final Queue<Contest> contestPool;
    /** 最大空闲比赛数量 **/
    private final int maxIdleContest;
    /** 清理线程 **/
    private final ScheduledExecutorService cleaner;

    /**
     * 初始化
     *
     * @param maxIdleContest 最大空闲数量
     */
    public ContestPool(int maxIdleContest) {
        this.contestPool = new ConcurrentLinkedQueue<>();
        this.maxIdleContest = maxIdleContest;
        this.cleaner = Executors.newSingleThreadScheduledExecutor();
        // 定期清理空闲比赛
        this.cleaner.scheduleAtFixedRate(this::cleanIdleContest, 10, 30, TimeUnit.SECONDS);
    }

    /**
     * 获取比赛对象
     *
     * @param roomType 房间类型
     * @return
     */
    public Contest acquireContest(int roomType) {
        Contest contest = this.contestPool.poll();
        if (contest == null) {
            // 创建指定类型的比赛
            contest = new Contest();
        }
        return contest;
    }

    /**
     * 归还比赛对象
     */
    public void releaseContest(Contest contest) {
        if (contest != null) {
            this.contestPool.offer(contest); // 放回池中
        }
    }

    /**
     * 定期清理空闲比赛对象
     */
    private void cleanIdleContest() {
        while (this.contestPool.size() > maxIdleContest) {
            Contest contest = this.contestPool.poll();
            if (contest != null) {
                // 销毁比赛，释放资源
                contest.destroy();
            }
        }
    }

    /**
     * 关闭对象池
     */
    public void shutdown() {
        this.cleaner.shutdownNow();
        this.contestPool.forEach(Contest::destroy);
        this.contestPool.clear();
    }

    public int getAvailableContest() {
        return this.contestPool.size();
    }

}
