package com.incubator.game.data.jedis;

import com.incubator.common.log4j.Log;
import com.incubator.game.GGame;
import com.incubator.common.util.PropertiesUtil;
import org.apache.commons.lang3.StringUtils;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;
import redis.clients.jedis.JedisPoolConfig;

import java.time.Duration;
import java.util.Objects;

public final class JedisPoolUtil {

	private static JedisPool jedisPool;

	private static String ADDR_ARRAY;

	private static int PORT;

	private static int TIMEOUT;

	private static int DEFAULT_DB = 0; // 默认 Redis 数据库为 0

	static {
		if (jedisPool == null) {
			try {
				initPool();
			} catch (Exception e) {
				Log.error("初始化Jedis连接池失败...", e);
			}
		}
	}

	/**
	 * 初始化连接池
	 */
	public synchronized static void initPool() {
		JedisPoolConfig config = new JedisPoolConfig();
		PropertiesUtil dbRedis = GGame.DB_SETTINGS;
		if (dbRedis.getProperty("redis.maxTotal") != null) {
			int maxTotal = Integer.parseInt(Objects.requireNonNull(dbRedis.getProperty("redis.maxTotal")));
			config.setMaxTotal(maxTotal);
		}
		if (dbRedis.getProperty("redis.maxIdle") != null) {
			int maxIdle = Integer.parseInt(Objects.requireNonNull(dbRedis.getProperty("redis.maxIdle")));
			config.setMaxIdle(maxIdle);
		}
		if (dbRedis.getProperty("redis.maxWaitMillis") != null) {
			int maxWaitMillis = Integer.parseInt(Objects.requireNonNull(dbRedis.getProperty("redis.maxWaitMillis")));
			config.setMaxWait(Duration.ofMillis(maxWaitMillis));
		}
		if (dbRedis.getProperty("redis.testOnBorrow") != null) {
			boolean testOnBorrow = Boolean.parseBoolean(dbRedis.getProperty("redis.testOnBorrow"));
			config.setTestOnBorrow(testOnBorrow);
		}
		ADDR_ARRAY = dbRedis.getProperty("redis.host");
		PORT = Integer.parseInt(Objects.requireNonNull(dbRedis.getProperty("redis.port")));
		TIMEOUT = Integer.parseInt(Objects.requireNonNull(dbRedis.getProperty("redis.timeout")));
		if (dbRedis.getProperty("redis.db") != null) {
			DEFAULT_DB = Integer.parseInt(Objects.requireNonNull(dbRedis.getProperty("redis.db")));
		}
		if (dbRedis.getProperty("redis.auth") != null) {
			String AUTH = dbRedis.getProperty("redis.auth");
			jedisPool = !StringUtils.isEmpty(AUTH)? new JedisPool(config, ADDR_ARRAY, PORT, TIMEOUT, AUTH)
					: new JedisPool(config, ADDR_ARRAY, PORT, TIMEOUT);
		} else {
			jedisPool = new JedisPool(config, ADDR_ARRAY, PORT, TIMEOUT);
		}
	}

	/**
	 * 获取 Jedis 实例并指定数据库
	 */
	public synchronized static Jedis getJedis() {
		Jedis jedis = jedisPool.getResource();
		try {
			if (DEFAULT_DB != 0) {
				jedis.select(DEFAULT_DB); // 切换到指定的 Redis 数据库
			}
		} catch (Exception e) {
            Log.error("Failed to select Redis database: {}", DEFAULT_DB, e);
		}
		return jedis;
	}

	/**
	 * 关闭 Jedis 连接
	 */
	public static void closeJedis(Jedis jedis) {
		if (jedis != null) {
			jedis.close();
		}
	}
}
