package com.incubator.game.handler.contest;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.contest.Contest;
import com.incubator.game.contest.ContestService;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.mq.RabbitMQUtil;
import com.incubator.game.data.po.ZjClubMemberPo;
import com.incubator.game.player.Player;
import com.incubator.game.util.PlayerUtil;
import com.incubator.game.util.ProtoUtil;
import com.incubator.message.proto.CommonProto;

import java.math.BigDecimal;
import java.util.List;
import java.util.Optional;

/**
 * 加入比赛/报名 请求
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.MatchSignUpReq_VALUE)
public class JoinContestHandler extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.MatchSignUpRes_VALUE);

        Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
        if (player == null) {
            Log.info("创建房间失败，找不到玩家");
            response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
            response.setMessage("操作失败...");
            return;
        }

        // 判断参数
        long contestId = request.getDataValue("contestId", Long.class, 0L);
        if (contestId <= 0L) {
            Log.info("参数错误");
            response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
            response.setMessage("参数错误...");
            return;
        }

        // 比赛是否存在
        Contest contest = ContestService.getInstance().contestMap.getOrDefault(contestId, null);
        if (contest == null) {
            Log.info("比赛不存在或已结束");
            response.setCode(CommonProto.Code.CONTEST_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，比赛不存在或已结束...");
            return;
        }

        //判断赛事人数限制
        if (contest.actors.size()>=contest.propPromotionLv.max){
            Log.info("比赛人数已满");
            response.setCode(CommonProto.Code.CONTEST_NUMBER_IS_MAX_VALUE);
            response.setMessage("操作失败，比赛人数已满...");
            return;
        }

        // 是否已加入其他比赛
        if (player.data.contestId > 0 && player.data.contestId != contestId) {
            Log.info("不可重复加入其他比赛");
            response.setCode(CommonProto.Code.CONTEST_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，不可重复加入其他比赛...");
            return;
        }

        //俱乐部赛判断
        if (contest.prop.getCompeteType()==2){
            String clubId = contest.prop.getClubId();
            String memberListStr = RedisUtil.get(RedisKeyConstant.CLUB_MEMBER + clubId);
            JSONObject memberObject = JSONUtil.parseObj(memberListStr);
            JSONArray clubMemberArray = memberObject.getJSONArray("clubMember");
            List<ZjClubMemberPo> clubMember = clubMemberArray.toList(ZjClubMemberPo.class);
            Optional<ZjClubMemberPo> first = clubMember.stream()
                    .filter(c -> c.getUserId().equals(player.data.playerId)&&c.getMemberStatus().equals("1"))
                    .findFirst();
            if (!first.isPresent()) {
//                Log.info("俱乐部赛，玩家未加入该俱乐部");
                response.setCode(CommonProto.Code.CONTEST_NOT_EXIST_ERR_VALUE);
                response.setMessage("操作失败，玩家未加入该俱乐部...");
                return;
            }
        }

        // 扣门票或者报名费
        String currencyType = contest.prop.getCurrencyType();
        BigDecimal cost = contest.prop.getCost();
        String errorMessage = "";

        switch (currencyType) {
            case "1":
                if (player.data.gold < cost.longValue()) {
                    errorMessage = "金蛋不足";
                    break;
                }
                player.data.gold -= cost.longValue();
                break;
            case "2":
                if (player.data.diamond < cost.longValue()) {
                    errorMessage = "钻石不足";
                    break;
                }
                player.data.diamond -= cost.longValue();
                break;
            case "3":
                if (player.data.foca < cost.longValue()) {
                    errorMessage = "福卡不足";
                    break;
                }
                player.data.foca -= cost.longValue();
                break;
        }

        if (!errorMessage.isEmpty()) {
            Log.info(errorMessage);
            response.setCode(CommonProto.Code.BALANCE_NOT_ENOUGH_VALUE);
            response.setMessage("报名失败，您的" + errorMessage);
            return;
        }
        //报名成功 发送扣费消息
        JSONObject object = new JSONObject();
        object.put("type","JoinContest");
        object.put("userId",player.data.playerId);
        object.put("competeId",contest.data.contestId);
        object.put("goodsId",currencyType);
        object.put("goodsNum",cost);
        try {
            RabbitMQUtil.sendMessage(object.toString());
        } catch (Exception e) {
            e.printStackTrace();
        }


        // 报名加入比赛
        contest = ContestService.getInstance().joinContest(player, contest.data.contestId);

        // 正常返回
        response.setData(ProtoUtil.contestInfoToMessage(contest));
    }


}
