package com.incubator.game.handler.room;

import com.incubator.common.MessageHandler;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.player.Player;
import com.incubator.game.room.Room;
import com.incubator.game.room.RoomService;
import com.incubator.game.util.PlayerUtil;
import com.incubator.message.proto.CommonProto;
import org.apache.commons.lang3.StringUtils;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * 准备阶段更换位置 同意请求
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.ChangePositionAgreeReq_VALUE)
public class ChangePositionAgreeHandler extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.ChangePositionAgreeRes_VALUE);

        Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
        if (player == null) {
            Log.info("创建房间失败，找不到玩家");
            response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
            response.setMessage("操作失败...");
            return;
        }

        // 房间是否存在
        Room room = RoomService.getInstance().getRoom(player.data.roomId);
        if (room == null) {
            Log.info("房间已解散");
            response.setCode(CommonProto.Code.ROOM_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，房间已解散...");
            return;
        }

        // 房间类型
        if (room.data.type != 0) {
            Log.info("房间类型不可换位置");
            response.setCode(CommonProto.Code.ROOM_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，房间类型不可换位置...");
            return;
        }

        // 房间是否是准备状态
        if (room.state != 0) {
            Log.info("房间正在进行中");
            response.setCode(CommonProto.Code.ROOM_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，房间正在进行中...");
            return;
        }

        // 0:拒绝 1:同意
        int op = request.getDataValue("op", Integer.class, 0);
        if (op != 0 && op != 1) {
            Log.info("参数错误");
            response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
            response.setMessage("参数错误...");
            return;
        }

        String fromId = room.changePosMap.getOrDefault(player.getId(), "");
        if (StringUtils.isEmpty(fromId)) {
            Log.info("找不到发起人");
            response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
            response.setMessage("操作失败，找不到发起人...");
            return;
        }

        Player fromPlayer = room.actors.getOrDefault(fromId, null);
        if (fromPlayer == null) {
            Log.info("找不到发起人");
            response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
            response.setMessage("操作失败，找不到发起人...");
            return;
        }

        // 清理缓存
        room.changePosMap.remove(player.getId());

        // 拒绝
        if (op == 0) {
            // 推送
            Map<String, Object> map = new HashMap<>();
            map.put("roomId", room.data.roomId);
            // 0请求交换 1拒绝
            map.put("state", 1);
            map.put("formPlayer", player.getId());
            fromPlayer.receive(CommonProto.Cmd.ChangePositionDialog_VALUE, map);
        }

        // 同意
        if (op == 1) {
            // 设置换位标识
            player.data.isChangePos = true;
            // 换座位
            room.switchSeat(player, fromPlayer.data.pos);
            // 推送房间其他3个玩家
            for (Player tmPlayer : room.actors.values()) {
                if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                    Map<String, Object> data = room.roomToMessage(tmPlayer, null);
                    tmPlayer.receive(CommonProto.Cmd.ChangePositionBroadcast_VALUE, data);
                }
            }
        }

        // 正常返回
        response.setData(room.roomToMessage(player, null));
    }
}
