package com.incubator.game.handler.room;

import com.incubator.common.MessageHandler;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.player.Player;
import com.incubator.game.room.Room;
import com.incubator.game.room.RoomService;
import com.incubator.game.util.PlayerUtil;
import com.incubator.message.proto.CommonProto;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * 准备阶段更换位置 请求
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.ChangePositionReq_VALUE)
public class ChangePositionHandler extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.ChangePositionRes_VALUE);

        Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
        if (player == null) {
            Log.info("创建房间失败，找不到玩家");
            response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
            response.setMessage("操作失败...");
            return;
        }

        // 判断参数
        int pos = request.getDataValue("pos", Integer.class, 0);
        if (pos < 0) {
            Log.info("参数错误");
            response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
            response.setMessage("参数错误...");
            return;
        }

        // 判断玩家绑定房间
        if (player.data.roomId <= 0) {
            Log.info("玩家未加入房间");
            response.setCode(CommonProto.Code.JOIN_ROOM_REPEATEDLY_ERR_VALUE);
            response.setMessage("操作失败，玩家未加入房间...");
            return;
        }

        // 房间是否存在
        Room room = RoomService.getInstance().getRoom(player.data.roomId);
        if (room == null) {
            Log.info("房间已解散");
            response.setCode(CommonProto.Code.ROOM_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，房间已解散...");
            return;
        }

        // 房间类型
        if (room.data.type != 0) {
            Log.info("房间类型不可换位置");
            response.setCode(CommonProto.Code.ROOM_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，房间类型不可换位置...");
            return;
        }

        // 房间是否是准备状态
        if (room.state != 0) {
            Log.info("房间正在进行中");
            response.setCode(CommonProto.Code.ROOM_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，房间正在进行中...");
            return;
        }

        // 是否自己与自己换
        if (pos == player.data.pos) {
            Log.info("不可与自己交换位置");
            response.setCode(CommonProto.Code.ROOM_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，不可与自己交换位置...");
            return;
        }

        Player oldPlayer = room.data.playerMap[pos];

        // 坑位没人
        if (oldPlayer == null) {
            // 换座位
            room.switchSeat(player, pos);
            // 推送房间其他3个玩家
            for (Player tmPlayer : room.actors.values()) {
                if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                    Map<String, Object> data = room.roomToMessage(tmPlayer, null);
                    tmPlayer.receive(CommonProto.Cmd.ChangePositionBroadcast_VALUE, data);
                }
            }
        }

        // 坑位有人
        if (oldPlayer != null) {
            // 设置换位标识
            player.data.isChangePos = true;
            // 设置数据
            room.changePosMap.put(oldPlayer.getId(), player.getId());
            // 推送
            Map<String, Object> map = new HashMap<>();
            map.put("roomId", room.data.roomId);
            // 0请求交换 1拒绝
            map.put("state", 0);
            map.put("formPlayer", player.getId());
            oldPlayer.receive(CommonProto.Cmd.ChangePositionDialog_VALUE, map);
        }

        // 正常返回
        response.setData(room.roomToMessage(player, null));
    }

}
