package com.incubator.game.handler.room;

import com.incubator.common.MessageHandler;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.player.Player;
import com.incubator.game.room.FSGDRoom;
import com.incubator.game.room.FSGDTable;
import com.incubator.game.room.Room;
import com.incubator.game.room.RoomService;
import com.incubator.game.util.CardUtils;
import com.incubator.game.util.GDUtils;
import com.incubator.game.util.PlayerUtil;
import com.incubator.message.proto.CommonProto;

import java.util.Map;

/**
 * 出牌请求
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.PlayDisCardsReq_VALUE)
public class DisCardHandler extends NetHandler {

    @Override
    public synchronized void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.PlayDisCardsRes_VALUE);

        Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
        if (player == null) {
            Log.info("创建房间失败，找不到玩家");
            response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
            response.setMessage("操作失败...");
            return;
        }

        // 判断玩家绑定房间
        if (player.data.roomId <= 0) {
            Log.info("准备失败，玩家不在房间");
            response.setCode(CommonProto.Code.PLAYER_ROOM_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，玩家不在房间...");
            return;
        }

        // 玩家房间是否存在
        Room room = RoomService.getInstance().getRoom(player.data.roomId);
        if (room == null) {
            Log.info("玩家房间已解散");
            response.setCode(CommonProto.Code.ROOM_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，玩家房间已解散...");
            return;
        }

        // 当前状态是否可操作
        if (room.state != 2 || player.data.state != 2) {
            Log.info("房间或玩家不在游戏中状态...");
            response.setCode(7);
            response.setMessage("操作失败，房间或玩家不在游戏中状态...");
            return;
        }
        // 当前状态是否可操作
        if (room.data.tributeState != 0) {
            Log.info("当前不是自己出牌回合...");
            response.setCode(8);
            response.setMessage("操作失败，当前不是自己出牌回合...");
            return;
        }

        switch (room.data.gameType) {
            case 1:
            case 2:
                if (!room.data.currentPlayer.getId().equals(player.getId())) {
                    Log.info("当前不是自己出牌回合...");
                    response.setCode(8);
                    response.setMessage("操作失败，当前不是自己出牌回合...");
                    return;
                }
                break;
            case 3:
                FSGDTable table = ((FSGDRoom) room).tableMap.getOrDefault(player.data.tableId, null);
                if (table != null) {
                    if (!table.data.currentPlayer.getId().equals(player.getId())) {
                        Log.info("当前不是自己出牌回合...");
                        response.setCode(8);
                        response.setMessage("操作失败，当前不是自己出牌回合...");
                        return;
                    }
                }
                break;
             default:
                 break;
        }

        // 判断参数
        int[] disCardList = request.getDataValue("disCardList", int[].class, new int[0]);
        if (disCardList == null || disCardList.length == 0) {
            Log.info("参数错误");
            response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
            response.setMessage("参数错误...");
            return;
        }

        // 判断手牌
        if (!CardUtils.contains(player.data.remainCards, disCardList)) {
            Log.info("参数错误，玩家出的牌不在手上...");
            response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
            response.setMessage("参数错误，玩家出的牌不在手上...");
            return;
        }

        // 判断玩家出牌牌型
        GDUtils.CardType disCardType = GDUtils.determineCardType(disCardList, room.data.wildCard,room.data.curDisCardType);
        if (disCardType == GDUtils.CardType.INVALID) {
            Log.info("牌型错误");
            response.setCode(9);
            response.setMessage("牌型错误...");
            return;
        }

        // 是否大过桌上牌
        if (!GDUtils.canDisCard(room.data.curDisCardList, room.data.curDisCardType,
                disCardList, disCardType, room.data.curLevelPoint, room.data.wildCard)) {
            Log.info("出牌错误，无法大过当前牌...");
            response.setCode(10);
            response.setMessage("出牌错误，无法大过当前牌...");
            return;
        }

        // 设置出牌队堆
        player.data.discardList = disCardList;
        player.data.disCardType = disCardType;

        // 出牌
        room.playerDisCard(player, disCardList, disCardType);

        // 正常返回
        Map<String, Object> data = room.roomToMessage(player, room.data.currentPlayer);
        data.put("time", 20);
        response.setData(data);
    }
}
