package com.incubator.game.handler.room;

import com.incubator.common.MessageHandler;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.player.Player;
import com.incubator.game.room.FSGDRoom;
import com.incubator.game.room.FSGDTable;
import com.incubator.game.room.Room;
import com.incubator.game.room.RoomService;
import com.incubator.game.util.PlayerUtil;
import com.incubator.message.proto.CommonProto;

import java.util.HashMap;
import java.util.Map;

/**
 * 过请求
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.PlayNoCardsReq_VALUE)
public class PassHandler extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.PlayNoCardsRes_VALUE);

        Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
        if (player == null) {
            Log.info("创建房间失败，找不到玩家");
            response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
            response.setMessage("操作失败...");
            return;
        }

        // 判断玩家绑定房间
        if (player.data.roomId <= 0) {
            Log.info("准备失败，玩家不在房间");
            response.setCode(CommonProto.Code.PLAYER_ROOM_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，玩家不在房间...");
            return;
        }

        // 玩家房间是否存在
        Room room = RoomService.getInstance().getRoom(player.data.roomId);
        if (room == null) {
            Log.info("玩家房间已解散");
            response.setCode(CommonProto.Code.ROOM_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，玩家房间已解散...");
            return;
        }

        // 当前状态是否可操作
        if (room.state != 2 || player.data.state != 2) {
            Log.info("房间或玩家不在游戏中状态...");
            response.setCode(7);
            response.setMessage("操作失败，房间或玩家不在游戏中状态...");
            return;
        }

        Map<String, Object> data = new HashMap<>();

        switch (room.data.gameType) {
            case 1:
            case 2:
                if (!room.data.currentPlayer.getId().equals(player.getId())) {
                    Log.info("当前不是自己出牌回合...");
                    response.setCode(8);
                    response.setMessage("操作失败，当前不是自己出牌回合...");
                    return;
                }
                // 该自己出牌 不可不出
                if (room.data.disCardPlayer == null && player.getId().equals(room.data.currentPlayer.getId())) {
                    Log.info("当前是自己出牌回合...");
                    response.setCode(9);
                    response.setMessage("操作失败，当前是自己出牌回合...");
                    return;
                }
                // 接风玩家 不可不出
                if (room.data.maxNum == 4) {
                    if (room.data.disCardPlayer != null && !player.getId().equals(room.data.disCardPlayer.getId())
                            && room.data.jieFengPlayer != null && room.data.passList.isEmpty()
                            && player.getId().equals(room.data.jieFengPlayer.getId())) {
                        Log.info("当前是自己出牌回合...");
                        response.setCode(9);
                        response.setMessage("操作失败，当前是自己出牌回合...");
                        return;
                    }
                }
                // 3家要不起 不可不出
                if (room.data.disCardPlayer != null && player.getId().equals(room.data.disCardPlayer.getId())) {
                    Log.info("当前是自己出牌回合...");
                    response.setCode(9);
                    response.setMessage("操作失败，当前是自己出牌回合...");
                    return;
                }

                // 玩家操作过
                room.playerPass(player, false);

                // 正常返回
                data = room.roomToMessage(player, room.data.currentPlayer);
                // 接风玩家
                data.put("jieFengPlayer", room.data.otherOpPlayerList.isEmpty()
                        && room.data.jieFengPlayer != null? room.data.jieFengPlayer.getId() : "");
                break;
            case 3:
                FSGDTable table = ((FSGDRoom) room).tableMap.getOrDefault(player.data.tableId, null);
                if (table != null) {
                    if (!table.data.currentPlayer.getId().equals(player.getId())) {
                        Log.info("当前不是自己出牌回合...");
                        response.setCode(8);
                        response.setMessage("操作失败，当前不是自己出牌回合...");
                        return;
                    }
                    // 该自己出牌 不可不出
                    if (table.data.disCardPlayer == null && player.getId().equals(table.data.currentPlayer.getId())) {
                        Log.info("当前是自己出牌回合...");
                        response.setCode(9);
                        response.setMessage("操作失败，当前是自己出牌回合...");
                        return;
                    }
                    // 接风玩家 不可不出
                    if (table.data.maxNum == 4) {
                        if (table.data.disCardPlayer != null && !player.getId().equals(table.data.disCardPlayer.getId())
                                && table.data.jieFengPlayer != null && table.data.passList.isEmpty()
                                && player.getId().equals(table.data.jieFengPlayer.getId())) {
                            Log.info("当前是自己出牌回合...");
                            response.setCode(9);
                            response.setMessage("操作失败，当前是自己出牌回合...");
                            return;
                        }
                    }
                    // 3家要不起 不可不出
                    if (table.data.disCardPlayer != null && player.getId().equals(table.data.disCardPlayer.getId())) {
                        Log.info("当前是自己出牌回合...");
                        response.setCode(9);
                        response.setMessage("操作失败，当前是自己出牌回合...");
                        return;
                    }

                    // 玩家操作过
                    table.playerPass(player, false);

                    // 正常返回
                    data = table.roomToMessage(player, table.data.currentPlayer);
                    // 接风玩家
                    data.put("jieFengPlayer", table.data.otherOpPlayerList.isEmpty()
                            && table.data.jieFengPlayer != null? table.data.jieFengPlayer.getId() : "");
                }
                break;
        }

        data.put("time", 20);
        response.setData(data);
    }
}
