package com.incubator.game.handler.room;

import com.incubator.common.MessageHandler;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.player.Player;
import com.incubator.game.room.Room;
import com.incubator.game.room.RoomService;
import com.incubator.game.util.CardUtils;
import com.incubator.game.util.GDUtils;
import com.incubator.game.util.PlayerUtil;
import com.incubator.message.proto.CommonProto;
import org.apache.commons.lang3.StringUtils;

import java.util.*;

/**
 * 进还贡请求
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.EnterBackCardsReq_VALUE)
public class TributeHandler extends NetHandler {

	@Override
	public void onDate(Connection session, WSRequest request, WSResponse response) {
		response.setCmd(CommonProto.Cmd.EnterBackCardsRes_VALUE);

		Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
		if (player == null) {
			Log.info("操作失败，找不到玩家");
			response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
			response.setMessage("操作失败...");
			return;
		}

		// 判断玩家绑定房间
		if (player.data.roomId <= 0) {
			Log.info("准备失败，玩家不在房间");
			response.setCode(CommonProto.Code.PLAYER_ROOM_NOT_EXIST_ERR_VALUE);
			response.setMessage("操作失败，玩家不在房间...");
			return;
		}

		// 玩家房间是否存在
		Room room = RoomService.getInstance().getRoom(player.data.roomId);
		if (room == null) {
			Log.info("玩家房间已解散");
			response.setCode(CommonProto.Code.ROOM_NOT_EXIST_ERR_VALUE);
			response.setMessage("操作失败，玩家房间已解散...");
			return;
		}

		// 是否是进还贡状态
		if (room.data.tributeList.isEmpty() || room.data.tributeState == 3) {
			Log.info("没有进还贡或已抗贡，进还贡失败");
			response.setCode(CommonProto.Code.ROOM_NOT_EXIST_ERR_VALUE);
			response.setMessage("没有进还贡或已抗贡，进还贡失败...");
			return;
		}

		// 判断参数
		int card = request.getDataValue("card", Integer.class, 0);
		if (!Arrays.asList(GDUtils.values).contains(card)) {
			Log.info("参数错误");
			response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
			response.setMessage("参数错误...");
			return;
		}

		// 判断手牌
		if (!CardUtils.contains(player.data.remainCards, card)) {
			Log.info("参数错误，选的牌不在手上...");
			response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
			response.setMessage("参数错误，选的牌不在手上...");
			return;
		}

		List<String> tributes = new ArrayList<>();
		List<String> backs = new ArrayList<>();

		for (String[] playerIds : room.data.tributeList) {
			if (!StringUtils.isEmpty(playerIds[0]) && !StringUtils.isEmpty(playerIds[1])) {
				tributes.add(playerIds[0]);
				backs.add(playerIds[1]);
			}
		}

		// 进贡者判断 是否是玩家手牌中最大的牌
		if (tributes.contains(player.getId())) {
//			int maxCard = GDUtils.getTributeMaxCard(player.data.remainCards, room.data.curLevelPoint);
//			if (card != maxCard) {
//				Log.info("参数错误，进贡者不是最大牌...");
//				response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
//				response.setMessage("参数错误，进贡者不是最大牌...");
//				return;
//			}
		}

		// 还贡者判断 是否是10以内的牌
		if (backs.contains(player.getId())) {
			if (GDUtils.getCardPoint(card) > 10) {
				Log.info("参数错误，还贡者不是小于10的牌...");
				response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
				response.setMessage("参数错误，还贡者不是小于10的牌...");
				return;
			}
		}

		// 移除手牌
		player.data.remainCards = CardUtils.remove(player.data.remainCards, card);

		// 设置进还贡牌
		if (!room.data.tributeCardMap.containsKey(player.getId())) {
			room.data.tributeCardMap.put(player.getId(), card);
		}
		player.data.tributeCard = card;

		// 进还贡数据
		Map<String, Object> tributeInfo = new HashMap<>();

		tributeInfo.put("tributeList", room.data.tributeList);
		tributeInfo.put("twoKingInfo", room.data.towKingList);
		tributeInfo.put("tributeState", room.data.tributeState);
		// 进还贡的选牌map
		tributeInfo.put("tributeCardMap", room.data.tributeCardMap);

		// 还贡是否广播
		boolean isBroadcast = tributes.contains(player.getId());

		if (backs.contains(player.getId()) && room.checkBackTributeStart()) {
			isBroadcast = true;
		}

		// 广播其他3人
		if (isBroadcast) {
			for (Player tmPlayer : room.actors.values()) {
				if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
					Map<String, Object> data = room.roomToMessage(tmPlayer, null);

					Map<String, Integer> tributeCardMap = new HashMap<>();
					for (Map.Entry<String, Integer> entry : room.data.tributeCardMap.entrySet()) {
						tributeCardMap.put(entry.getKey(), entry.getKey().equals(tmPlayer.getId())? entry.getValue() : 0);
					}
					tributeInfo.put("tributeCardMap", tributeCardMap);
					data.put("tributeInfo", tributeInfo);
					tmPlayer.receive(CommonProto.Cmd.EnterBackCardsBroadcast_VALUE, data);
				}
			}
		}

		// 正常返回
		Map<String, Object> data = room.roomToMessage(player, null);
		Map<String, Integer> tributeCardMap = new HashMap<>();
		for (Map.Entry<String, Integer> entry : room.data.tributeCardMap.entrySet()) {
			tributeCardMap.put(entry.getKey(), entry.getKey().equals(player.getId())? entry.getValue() : 0);
		}
		tributeInfo.put("tributeCardMap", tributeCardMap);
		data.put("tributeInfo", tributeInfo);
		response.setData(data);
	}
}
