package com.incubator.game.jdgd;

import com.alibaba.fastjson2.JSONObject;
import com.incubator.common.log4j.Log;
import com.incubator.common.util.StringUtils;
import com.incubator.game.player.Player;
import com.incubator.game.room.Room;
import com.incubator.game.room.RoomService;
import com.incubator.game.util.PlayerUtil;
import com.incubator.message.proto.CommonProto;

import java.util.Map;
import java.util.Queue;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;

/**
 * Author:  xhb
 * Date:  2024-12-31 16:48
 * Description: 经典掼蛋游戏玩法实体类
 */
public class JDGDGameplay {
    // 定义房间列表和对应的玩家队列
    private static final Map<Long, Room> roomMap = new ConcurrentHashMap<>();
    private static final Map<String, Queue<String>> playerQueueMap = new ConcurrentHashMap<>();

    static {
        playerQueueMap.put("novice", new ConcurrentLinkedQueue<>());
        playerQueueMap.put("middle", new ConcurrentLinkedQueue<>());
        playerQueueMap.put("advanced", new ConcurrentLinkedQueue<>());
        playerQueueMap.put("master", new ConcurrentLinkedQueue<>());
    }

    private static final int ROOM_CAPACITY = 4;

    public static void joinMatchQueue(String lvType, Player player) {
        String queueKey = getQueueKey(lvType);
        if (queueKey == null) {
            Log.warn("未知的等级类型: {}", lvType);
            return;
        }

        Queue<String> queue = playerQueueMap.get(queueKey);
        queue.add(player.data.playerId);
        Log.info("玩家 {} 加入 {} 匹配队列, 当前人数: {}", player.data.playerId, lvType, queue.size());

        // 检查是否达到房间容量
        if (queue.size() >= ROOM_CAPACITY) {
            createRoomAndNotifyPlayers(queueKey);
        }
    }

    /**
     * 根据等级类型获取对应的队列键
     *
     * @param lvType 等级类型
     * @return 队列键或 null
     */
    private static String getQueueKey(String lvType) {
        switch (lvType) {
            case "1": return "novice";
            case "2": return "middle";
            case "3": return "advanced";
            case "4": return "master";
            default: return null;
        }
    }

    /**
     * 创建房间并通知玩家
     *
     * @param queueKey 队列键
     */
    private static void createRoomAndNotifyPlayers(String queueKey) {
        Queue<String> queue = playerQueueMap.get(queueKey);
        Room room = allocateRooms();
        if (room == null) {
            Log.error("创建房间失败");
            return;
        }

        // 添加房间到对应列表
        roomMap.put(room.data.roomId, room);

        // 通知玩家进入房间
        for (int i = 0; i < ROOM_CAPACITY; i++) {
            String playerId = queue.poll();
            if (StringUtils.isEmpty(playerId)) {
                break; // 队列中玩家不足，退出循环
            }
            Player tmPlayer = (Player) PlayerUtil.getOnlinePlayer(playerId);
            if (tmPlayer == null) {
                // TODO: 首轮替补机器人
                continue;
            }
            Map<String, Object> map = room.roomToMessage(tmPlayer, null);
            tmPlayer.receive(CommonProto.Cmd.MatchOfficialStart_VALUE, map);
        }
    }

    public static synchronized Room allocateRooms() {
        // 玩家匹配,分配房间
        JSONObject jsonData = new JSONObject();
        jsonData.put("playingTypeOne", 2);
        jsonData.put("playingTypeTwo", 1);
        jsonData.put("upgradeType", 1);
        jsonData.put("flippingType", 2);
        jsonData.put("advanceType", 2);
        jsonData.put("timeoutType", 1);
        jsonData.put("voiceType", 2);
        jsonData.put("recordType", 2);
        jsonData.put("watchType", 4);
        // 房间id
        jsonData.put("roomId", RoomService.getInstance().randomRoomIdByContest());
        // 人数模式 2:2人 4:4人
        jsonData.put("playerNum", 4);
        // 房间最大局数
//            jsonData.put("maxRound", this.prop.maxRound);
        jsonData.put("maxRound", 1);
        // 房间类型 0:好友房 1:官方比赛房
        jsonData.put("type", 1);
        // 房间玩法类型 1:掼蛋
        jsonData.put("gameType", 1);

        jsonData.put("isTribute", false);

        Room room = null;
        try {
            room = RoomService.getInstance().creatRoom(null, 1, jsonData);
        } catch (Exception e) {
            Log.error("比赛开始...创建房间出错...", e);
        }

        if (room == null) {
            Log.error("比赛开始...创建房间出错...");
            return null;
        }
        return room;
    }
}
