package com.incubator.game.listener;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.handler.listener.GameHttpServerConnectionListener;
import com.incubator.core.net.http.HttpHandler;
import com.incubator.game.GGame;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.handler.codec.http.*;
import io.netty.util.ReferenceCountUtil;

import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;

public class HttpInnerListener extends GameHttpServerConnectionListener {

	@Override
	public void messageArrived(Connection conn, HttpRequest msg) {
		// 因为此方法执行完后就会回收msg,所以这里保留引用，防止 msg 被回收
		ReferenceCountUtil.retain(msg);

		// 解析路径和参数
		QueryStringDecoder decoder = new QueryStringDecoder(msg.uri());
		String path = decoder.path();

		// 忽略特殊路径
		if ("/favicon.ico".equals(path)) {
			ReferenceCountUtil.release(msg);
			return;
		}

		//获取参数
		String body = "";
		Map<String, String> params = new HashMap<>();

		try {
			// 如果是 POST 请求且为 FullHttpRequest 类型，则读取请求体
			if (msg.method() == HttpMethod.POST && msg instanceof FullHttpRequest) {
				FullHttpRequest request = (FullHttpRequest) msg;
				body = this.getBody(request);

				// 解析 body 参数到 Map
				if (request.headers().get(HttpHeaderNames.CONTENT_TYPE).contains("application/x-www-form-urlencoded")) {
					params.putAll(parseFormParams(body));
				} else if (request.headers().get(HttpHeaderNames.CONTENT_TYPE).contains("application/json")) {
					params.putAll(parseJsonParams(body));
				}
			} else if (msg.method() == HttpMethod.GET) {
				// 解析 GET 请求参数
				decoder.parameters().forEach((key, values) -> params.put(key, values.get(0)));
			}

			// 根据路径获取对应的处理器
			HttpHandler handler = GGame.httpHandlers.get(path);
			if (handler == null) {
				Log.warn("未找到路径 {} 对应的处理器", path);
				sendErrorResponse(conn, HttpResponseStatus.NOT_FOUND, "Path not found: " + path);
				return;
			}

			Log.info("收到 HTTP 消息: path={}, params={}", path, body);

			// 创建自定义 HttpRequest 对象并绑定会话
			com.incubator.core.net.http.HttpRequest request = new com.incubator.core.net.http.HttpRequest();

			// 调用处理器逻辑
			com.incubator.core.net.http.HttpResponse response = handler.onDate(request, params);
			if (response != null) {
				sendSuccessResponse(conn, path, response);
			} else {
				Log.warn("处理器 {} 返回空响应", path);
				sendErrorResponse(conn, HttpResponseStatus.INTERNAL_SERVER_ERROR, "Empty response from handler.");
			}
		} catch (Exception e) {
            Log.error("====http调用{}出错：", path, e);
		} finally {
			ReferenceCountUtil.release(msg);
		}
	}

	@Override
	public void connectionOpened(Connection connection) {
		
	}

	@Override
	public void connectionClosed(Connection connection) {
		
	}

	/**
	 * 获取body参数
	 * @param request
	 * @return
	 */
	private String getBody(FullHttpRequest request) {
		ByteBuf content = request.content();
		return content.isReadable() ? content.toString(StandardCharsets.UTF_8) : "";
	}

	private Map<String, String> parseFormParams(String body) {
		Map<String, String> params = new HashMap<>();
		QueryStringDecoder decoder = new QueryStringDecoder("?" + body, false);
		decoder.parameters().forEach((key, values) -> params.put(key, values.get(0)));
		return params;
	}

	private Map<String, String> parseJsonParams(String body) {
		// 使用 FastJSON 解析 JSON 字符串
		JSONObject jsonObject = JSON.parseObject(body);
		Map<String, String> params = new HashMap<>();
		for (Map.Entry<String, Object> entry : jsonObject.entrySet()) {
			params.put(entry.getKey(), entry.getValue().toString());
		}
		return params;
	}

	private void sendSuccessResponse(Connection conn, String path, com.incubator.core.net.http.HttpResponse response) {
		try {
			String responseBody = response.toJson();
			FullHttpResponse httpResponse = new DefaultFullHttpResponse(
					HttpVersion.HTTP_1_1,
					HttpResponseStatus.OK,
					Unpooled.wrappedBuffer(responseBody.getBytes(StandardCharsets.UTF_8))
			);
			httpResponse.headers().set(HttpHeaderNames.CONTENT_TYPE, "application/json; charset=UTF-8");
			conn.writeAndFlush(httpResponse);
			conn.close();
			Log.info("返回 HTTP 消息: path={}, response={}", path, responseBody);
		} catch (Exception e) {
			Log.error("发送成功响应时出错: path={}, error={}", path, e);
		}
	}

	private void sendErrorResponse(Connection conn, HttpResponseStatus status, String errorMessage) {
		try {
			FullHttpResponse errorResponse = new DefaultFullHttpResponse(
					HttpVersion.HTTP_1_1,
					status,
					Unpooled.copiedBuffer(errorMessage, StandardCharsets.UTF_8)
			);
			errorResponse.headers().set(HttpHeaderNames.CONTENT_TYPE, "text/plain; charset=UTF-8");
			conn.writeAndFlush(errorResponse);
			conn.close();
			Log.warn("发送错误响应: status={}, message={}", status, errorMessage);
		} catch (Exception e) {
			Log.error("发送错误响应时出错: status={}, error={}", status, e);
		}
	}
}
