package com.incubator.game.listener;

import com.incubator.common.log4j.Log;
import com.incubator.core.net.handler.listener.GameServerConnectionListener;
import com.incubator.core.net.ws.*;
import com.incubator.core.pool.WSRequestPool;
import com.incubator.core.pool.WSResponsePool;
import com.incubator.game.GGame;
import com.incubator.common.net.Connection;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.constant.WkServerStatus;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.player.GPlayer;
import com.incubator.game.player.Player;
import com.incubator.message.proto.CommonProto;
import io.netty.buffer.ByteBuf;

import java.util.HashMap;
import java.util.Map;

/**
 * 公共接口监听器
 */
public class PublicListener extends GameServerConnectionListener {

	@Override
	public void connectionOpened(Connection connection) {
		connection.setAttribute(HEART_CONFIG, new HeartConfig());
		connection.setAttribute(LAST_READ_TIME_MILLIS, System.currentTimeMillis());
		setHeartEnable(GGame.heartEnable);
		Log.info("连接已打开: {}", connection);
	}

	@Override
	public void connectionClosed(Connection connection) {
		try {
			if (connection != null && connection.getPlayerId() != null) {
				GPlayer player = GGame.onlinePlayers.get(connection.getPlayerId());
				if (player != null) {
					// 执行玩家登出操作
					((Player) player).destroy();
				}
			} else {
				Log.warn("无法处理玩家退出，PlayerId为空，connection={}", connection);
			}
		} catch (Exception e) {
			Log.error("玩家退出时发生异常: ", e);
		} finally {
			if (connection != null) {
				connection.close();
			}
			Log.info("连接已关闭: {}", connection);
		}
	}

	@Override
	public void messageArrived(Connection conn, Object msg) {
		if (!(msg instanceof ByteBuf)) {
			Log.warn("收到不支持的消息类型: {}", msg.getClass());
			return;
		}

		ByteBuf byteBuf = (ByteBuf) msg;
		if (byteBuf.readableBytes() <= 0) {
			Log.warn("收到空消息，关闭连接: {}", conn);
			conn.close();
			return;
		}

		WSRequest request = null;
		WSResponse response = null;

		try {
			// 从 ByteBuf 中读取字节数据
			byte[] bytes = new byte[byteBuf.readableBytes()];
			byteBuf.getBytes(byteBuf.readerIndex(), bytes);

			// 从对象池获取 WSRequest 实例
			request = WSRequestPool.getRequest(bytes);
//			if (!RequestFilter.isAllowed(conn, request.getCmd())) {
//				Log.warn("请求过于频繁，已被过滤: cmd={}, player={}", request.getCmd(), conn.getPlayerId());
//				return;
//			}

			// 获取处理器
			NetHandler handler = GGame.handlers.get(request.getCmd());
			if (handler == null) {
				Log.error("未知的消息号: cmd={}", request.getCmd());
				conn.close();
				return;
			}

			// 从对象池获取 WSResponse 实例
			response = WSResponsePool.getResponse();
			Log.debug("收到消息: cmd={}, 内容={}", request.getCmd(), request.toJson());
			handler.onDate(conn, request, response);

			conn.writeAndFlush(response.toBytes());
			Log.debug("返回消息: cmd={}, 内容={}", request.getCmd(), response.toJson());
		} catch (Exception e) {
			Log.error("处理消息时发生错误: ", e);
		} finally {
			// 回收对象，避免内存泄漏
			if (request != null) WSRequestPool.recycle(request);
			if (response != null) WSResponsePool.recycle(response);
		}
	}

	@Override
	public boolean doIsHeartReq(Object msg) {
		if (!(msg instanceof ByteBuf)) {
			return false;
		}

		try {
			ByteBuf byteBuf = (ByteBuf) msg;
			byte[] bytes = new byte[byteBuf.readableBytes()];
			byteBuf.getBytes(byteBuf.readerIndex(), bytes);

			WSRequest request = new WSRequest(bytes);
			return request.getCmd() == CommonProto.Cmd.HeartBeatReq_VALUE;
		} catch (Exception e) {
			Log.error("心跳请求解析失败: ", e);
			return false;
		}
	}

	@Override
	public void doSendHeartReq(Connection conn) {
		try {
			long now = System.currentTimeMillis();
			long lastTime = (long) conn.getAttribute(LAST_READ_TIME_MILLIS);
			if (now - lastTime > 10000L) {
//				Log.warn("心跳超时，关闭连接: {}", conn.getUid());
//				conn.close();
			} else {
//                HeartbeatPush push = new HeartbeatPush();
//                push.setTime(System.currentTimeMillis());
//                Message msg = new Message(ProtoCommon.MsgId.S2C_Heartbeat, push.toJson());
//                msg.setCode(ProtoCommon.Code.OK_VALUE);
//                conn.writeAndFlush(super.packMessage(msg));
			}
		} catch (Exception e) {
			Log.error("发送心跳请求出错: {}", e.getMessage(), e);
		}
	}

	@Override
	public void doSendHeartRsp(Connection conn) {
		// 发送心跳回复
		try {
			Map<String, Object> data = new HashMap<>();
			data.put("currentTimeMillis", System.currentTimeMillis());
			conn.writeAndFlush(new WSResponse(CommonProto.Cmd.HeartBeatRes, data).toBytes());
		} catch (Exception e) {
			Log.error("发送心跳回复出错: {}", e.getMessage(), e);
		}
	}
}
