package com.incubator.game.player;

import com.incubator.common.log4j.Log;
import com.incubator.game.GGame;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.constant.WkServerStatus;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.po.PlayerInfoPO;
import com.incubator.game.player.bag.BagManager;
import com.incubator.game.room.Room;
import com.incubator.game.room.RoomService;
import com.incubator.game.util.Const.PlayerEventType;
import com.incubator.game.util.Const.ManagerType;
import com.incubator.game.util.GDUtils;
import com.incubator.message.proto.CommonProto;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 玩家实体
 */
public class Player extends GPlayer {

    /** 玩家绑定的所有功能管理类 **/
    public Map<ManagerType, ModuleManager> allManagers = new ConcurrentHashMap<>();

    /** 玩家数据 **/
    public PlayerInfoPO data;

    public Player(PlayerInfoPO playerData) {
        try {
            this.init(playerData);
        } catch (Exception e) {
            Log.error("玩家初始化数据出错...{}", e.getMessage());
        }
    }

    /**
     * 注册玩家管理类
     *
     * @param manager
     */
    private void registerManager(ModuleManager manager) {
        if (manager.getManagerType() == null) {
            Log.error("{}, manager.getManagerType 为空", manager.getClass());
        }
        this.allManagers.put(manager.getManagerType(), manager);
    }

    /**
     * 初始化玩家数据，注册绑定玩家各功能管理类
     *
     * @param data
     */
    private void init(PlayerInfoPO data) {
        this.data = data;

        // 注册玩家背包模块
        registerManager(new BagManager(this));
    }

    /**
     * 获取玩家各功能管理类
     *
     * @param type
     * @param clazz
     * @return
     * @param <T>
     */
    @SuppressWarnings("unchecked")
    public <T> T getManager(ManagerType type, Class<T> clazz) {
        Object manager = this.allManagers.get(type);
        if (manager == null) {
            Log.error("Manager found for type {}", type);
            return null;
        }
        if (!clazz.isInstance(manager)) {
            Log.error("Manager found for type {} is not of the expected type: {}", type, clazz.getName());
            return null;
        }
        return (T) manager;
    }

    /**
     * 角色登录成功, 服务器内部处理逻辑在此
     */
    public void onLogin() {
        Log.info("玩家登录成功...");

        this.refreshNewDay();

        for (ModuleManager manager : this.allManagers.values()) {
            try {
                manager.onPlayerEvent(PlayerEventType.AFTER_LOGIN);
            } catch (Exception e) {
                Log.error(manager.getClass().getName(), " AFTER_LOGIN 登录出错: ", this.getId(), ", ", e);
            }
        }

        // 玩家是否在房间
        Room room = RoomService.getInstance().getRoomByPlayerId(this.getId());
        if (room != null) {
            this.data.roomId = room.data.roomId;
        }
    }

    /**
     * 销毁玩家，释放资源
     */
    public void destroy() {
        this.doLogout();
        // 重置玩家数据
        this.data = new PlayerInfoPO();
        // 断开链接
        this.playerRemoveSessionBind();
        // 归还玩家对象
        GGame.playerPool.releasePlayer(this);
        Log.debug("Player {} has been destroyed.", this.getId());
    }

    /**
     * 每日刷新
     */
    public void refreshNewDay() {
        //0点重置数据
//        Date now = new Date();
//        if (DateUtils.isSameDay(now, this.memePunksData.refreshTime)) {
//            return;
//        }

        for (ModuleManager manager : this.allManagers.values()) {
            manager.onPlayerEvent(PlayerEventType.REFRESH_NEWDAY);
        }

        Log.info("refreshNewDay playerId : {}, name : {}", this.getId(), this.data.name);
    }

    @Override
    public void onLogoutUpdate() {
        Log.info("处理玩家下线逻辑 : {}, {}", this.getId(), this.data.name);
        // 下线相关模块处理
        for (ModuleManager manager : this.allManagers.values()) {
            manager.onPlayerEvent(PlayerEventType.OFFLINE);
        }
        // 有房间就换成托管状态
        Room room = RoomService.getInstance().getRoomByPlayerId(this.getId());
        if (room != null) {
            if (room.state == 2) {
                this.data.isAuto = true;
            } else {
                room.leaveRoom(this);
            }
        }

        WkServerStatus wkToRedis = RedisUtil.getWkToRedis(RedisKeyConstant.WK_SERVE, RedisKeyConstant.WK_SERVE_KEY);
        //在线人数-1
        assert wkToRedis != null;
        if ( wkToRedis.getNum() > 0){
            int num = wkToRedis.getNum() - 1;
            wkToRedis.setNum(num);
            RedisUtil.saveWkToRedis(RedisKeyConstant.WK_SERVE, wkToRedis);
        }else {
            wkToRedis.setNum(0);
            RedisUtil.saveWkToRedis(RedisKeyConstant.WK_SERVE, wkToRedis);
        }
    }

    @Override
    public void sync() {
        Log.info("数据安全持久化...");
    }

    @Override
    public void free() {
        Log.info("释放玩家的相关引用...");
    }

    @Override
    public void playerRemoveSessionBind() {
        this.session.close();
        Log.info("玩家移除绑定了...");
    }

    @Override
    public void forbidPlayer(int forbidSec, String desc) {

    }

    @Override
    public int setLogoutTime(Date date) {
        return 0;
    }

    @Override
    public long getLogoutTime() {
        return 0;
    }

    @Override
    public void onPlayerUpdate() {

    }

    @Override
    public String getId() {
        return this.data.playerId;
    }

    @Override
    public int getLogicServerId() {
        return GGame.SERVER_ID;
    }

    @Override
    public String getUid() {
        return this.session.getUid();
    }

    @Override
    public void doUpdate() {

    }

    /**
     * 重置房间数据
     */
    public void resetRoom() {
        this.data.roomId = 0;
        this.data.tableId = 0;
        this.data.teamId = 0;
        this.data.pos = 0;
        this.data.state = 0;
        this.data.isAuto = false;
        this.data.remainCards = new int[]{};
        this.data.discardList = new int[]{};
        this.data.disCardType = GDUtils. CardType.INVALID;
        this.data.levelIndex = 0;
        this.data.levelPointView = new int[2];
        this.data.teammateId = "";
        this.data.tributeCard = 0;
        this.data.tributeGetCard = 0;
        this.data.isChangePos = false;
        this.data.bestScore = new HashMap<>();
    }

}
