package com.incubator.game.room;

import com.alibaba.fastjson2.JSONObject;
import com.incubator.common.log4j.Log;
import com.incubator.common.util.RandomUtil;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.constant.RoomStatus;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.po.RoomPO;
import com.incubator.game.player.Player;
import com.incubator.game.util.CardUtils;
import com.incubator.game.util.FSGDUtil;
import com.incubator.game.util.GDUtils;
import com.incubator.game.util.Struct;
import com.incubator.message.proto.CommonProto;
import org.apache.commons.lang3.StringUtils;

import java.util.*;

/**
 * 复式掼蛋房间
 */
public class FSGDRoom extends Room {

    /** 复式掼蛋桌子 **/
    public Map<Integer, FSGDTable> tableMap;
    /** 复式掼蛋 座位上的牌 **/
    public Map<Integer, int[]> cardMap;

    public FSGDRoom() {}

    @Override
    public void init(Player player, JSONObject jsonData) {
        this.jsonData = jsonData;
        // todo 默认配置
        this.data = new RoomPO();
        this.data.roomId = jsonData.getLong("roomId");
        this.data.type = jsonData.getInteger("type");
        this.data.gameType = jsonData.getInteger("gameType");
        this.data.ownerId = player != null? player.getId() : "";
        this.data.maxNum = 8;
        this.data.maxRound = jsonData.getInteger("maxRound");
        this.data.playerMap = new Player[8];
        // 是否有进还贡规则
        this.data.isTribute = false;
        // 初始化级牌点数
        this.data.curLevelIndex = RandomUtil.getIndex(GDUtils.levelPoint.length);
        this.data.curLevelPoint = GDUtils.levelPoint[this.data.curLevelIndex];
        // 初始化桌子
        this.tableMap = new HashMap<>();
        FSGDTable table;
        for (int i = 0; i < 2; i++) {
            table = new FSGDTable(this, i);
            table.init(jsonData);
            this.tableMap.put(i, table);
        }
        // 初始化牌库
        this.initCardList();
        // 初始化座位上的牌
        this.cardMap = new HashMap<>();
        for (int i = 0; i < 4; i++) {
            this.cardMap.put(i, new int[]{});
        }
    }

    /**
     * 玩家加入房间
     */
    @Override
    public synchronized void joinRoom(Player player) {
        if (player == null) {
            return;
        }
        lock.lock();
        try {
            if (this.actors.size() >= this.data.maxNum) {
                // 房间已满
                return;
            }
            // 玩家绑定房间
            player.data.roomId = this.data.roomId;

            // 加入房间角色缓存
            this.actors.put(player.getId(), player);

            // 加入桌子
            int tableIndex = (this.actors.size() - 1) / 4;
            FSGDTable table = this.tableMap.getOrDefault(tableIndex, null);
            if (table != null) {
                // 加入桌子
                table.joinTable(player);

                // 设置队伍
                player.data.teamId = FSGDUtil.assignTeam(this.actors.size() - 1);

                // 推送房间其他7个玩家
                for (Player tmPlayer : this.actors.values()) {
                    if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                        Map<String, Object> data = this.roomToMessage(tmPlayer, null);
                        tmPlayer.receive(CommonProto.Cmd.RoomPlayerJoin_VALUE, data);
                    }
                }
            }

            // 有人就开启定时器
            if (!this.actors.isEmpty())  {
                this.startRoomTask();
            }
        } finally {
            lock.unlock();
        }
    }

    /** 玩家离开房间 **/
    @Override
    public synchronized void leaveRoom(Player player) {
        if (player == null) {
            return;
        }
        lock.lock();
        try {
            // 清理房间数据
            this.actors.remove(player.getId());
            RoomStatus roomToRedis = RedisUtil.getRoomToRedis(RedisKeyConstant.WK_ROOM_MAP, String.valueOf(this.data.roomId));
            if (roomToRedis != null){
                List<String> players = roomToRedis.getPlayers();
                players.remove(player.getId());
                roomToRedis.setPlayers(players);
            }
            this.data.playerMap[player.data.pos] = null;
            // todo 房主退出

            // 离开桌子
            FSGDTable table = this.tableMap.getOrDefault(player.data.tableId, null);
            if (table != null) {
                table.leaveTable(player);

                // 推送房间其他玩家
                for (Player tmPlayer : this.actors.values()) {
                    if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                        Map<String, Object> data = this.roomToMessage(tmPlayer, null);
                        tmPlayer.receive(CommonProto.Cmd.RoomPlayerLeave_VALUE, data);
                    }
                }
            }
        } finally {
            lock.unlock();
        }
    }

    /**
     * 初始化牌库
     */
    @Override
    public void initCardList() {
        for (int i = 0; i < 2; i++) {
            this.data.cardList.addAll(Arrays.asList(GDUtils.values));
            Collections.shuffle(this.data.cardList);
        }
    }

    /**
     * 给玩家发牌，每人27张
     */
    @Override
    public synchronized void doSendCard() {
        // 发牌
        for (int i = 0; i < 4; i++) {
            int[] remainCards = this.cardMap.getOrDefault(i, new int[]{});
            for (int k = 0; k < 27; k++) {
                int card = this.data.cardList.get(0);
                // 设置当前赖子
                if (GDUtils.isWildCard(card, this.data.curLevelPoint)) {
                    this.data.wildCard = card;
                }
                remainCards = CardUtils.add(remainCards, card);
                this.data.cardList.remove(0);
            }
            this.cardMap.put(i, remainCards);
        }
    }

    /**
     * 主线程逻辑
     */
    @Override
    public void doUpdate() {
        synchronized (this) {
//            Log.debug("检测: 经典掼蛋房间主线程执行...");
            switch (this.state) {
                case 0:
                    // 等待状态
                    if (this.checkReadyStart()) {
                        this.state = 1;
                        this.flag = false;
                        this.time = 0;
                    }

                    // 120秒未开始直接解散
//                    if (this.time >= 120 && this.type != 1) {
//                        // 广播
//                        for (Player tmPlayer : this.actors.values()) {
//                            if (tmPlayer != null) {
//                                tmPlayer.receive(CommonProto.Cmd.DisbandRoom_VALUE, new HashMap<>());
//                            }
//                        }
//                        this.destroy();
//                    }

                    // 房间没人直接解散
                    if (this.actors.isEmpty()) {
                        this.destroy();
                        return;
                    }
                    break;
                case 1:
                    //锁
                    RedisUtil.set("roomGambit:"+this.data.roomId, "0");
                    // 开局
                    if (!this.flag) {
                        //开始
                        if (time >= 3) {
                            this.doStart();
                            this.state = 2;
                            this.flag = false;
                            this.time = 0;
                            Log.debug("检测: 房间号:{}, 房间人数:{}, 当前局数:{}, 状态:开局...VS动画...", this.data.roomId, this.actors.size(), this.data.curRound);
                        }
                    }
                    break;
                case 2:
                    // 进行中
                    if (!this.flag) {
                        if (this.checkAllOver()) {
                            this.settlement();
                            this.gameOver();
                        }
                    }
                    break;
                case 3:
                    // 结束
                    if (!this.flag) {
                        this.flag = true;
                        this.time = 0;
                        Log.debug("游戏结束: 房间号 : {}, 房间人数 : {}, 当前局数 : {}", this.data.roomId, this.actors.size(), this.data.curRound);
                    } else {
                        // 小于最大局数,切等待状态
                        if (this.data.curRound < this.data.maxRound) {
                            this.resetNextRoom();
                            this.state = 0;
                            this.flag = false;
                            this.time = 0;
                        } else {
                            //判断锁是否存在 //等待消失
                            while (RedisUtil.exists("roomGambit:"+this.data.roomId)){
                                try {
                                    Thread.sleep(1000);
                                } catch (InterruptedException e) {
                                    e.printStackTrace();
                                }
                            }
                            // 120秒未开始直接解散
//                            if (this.time >= 120 && this.type != 1) {
//                                // 广播
//                                for (Player tmPlayer : this.actors.values()) {
//                                    if (tmPlayer != null) {
//                                        tmPlayer.receive(CommonProto.Cmd.DisbandRoom_VALUE, new HashMap<>());
//                                    }
//                                }
//                                this.destroy();
//                            }
                        }
                    }
                    break;
                   default:
                    break;

            }

            this.time += 1;
        }
    }

    /**
     * 检测是否可以开始 3秒倒计时开始
     *
     * @return
     */
    public boolean checkReadyStart() {
        if (!this.isStart() || this.state != 0) {
            return false;
        }

        for (FSGDTable table : this.tableMap.values()) {
            if (table != null && !table.actors.isEmpty()) {
                // 广播3秒倒计时
                for (Player tmPlayer : table.actors.values()) {
                    if (tmPlayer != null) {
                        Map<String, Object> data = table.roomToMessage(tmPlayer, null);
                        data.put("state", 1);
                        data.put("time", 3);
                        tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
                    }
                }
            }
        }

        return true;
    }

    /**
     * 是否玩家都已准备,可开始
     *
     * @return
     */
    public boolean isStart() {
        int cnt = 0;
        for (Player player : this.actors.values()) {
            if (player != null && player.data.state == 1) {
                cnt += 1;
            }
        }
        return cnt >= this.data.maxNum;
    }

    /**
     * 游戏开局
     */
    @Override
    public synchronized void doStart() {
        lock.lock();
        try {
            this.data.curRound += 1;
            // 发牌
            this.doSendCard();

            for (FSGDTable table : this.tableMap.values()) {
                if (table != null && !table.actors.isEmpty()) {
                    // 开启定时任务
                    table.startTableTask();
                    table.state = 1;
                }
            }
        } catch (Exception e) {
            Log.error("===游戏开局出错...", e);
        } finally {
            lock.unlock();
        }
    }

    /**
     * 检测所有桌子是否已结束
     *
     * @return
     */
    public synchronized boolean checkAllOver() {
        int cnt = 0;
        for (FSGDTable table : this.tableMap.values()) {
            if (table != null && table.state == 3) {
                cnt += 1;
            }
        }
        return cnt >= this.tableMap.size();
    }

    /**
     * 玩家出牌
     *
     * @param player    出牌玩家
     * @param disCardList   打出的牌
     * @param disCardType   打出的牌型
     */
    @Override
    public synchronized void playerDisCard(Player player, int[] disCardList, GDUtils.CardType disCardType) {
        FSGDTable table = this.tableMap.getOrDefault(player.data.tableId, null);
        if (table != null) {
            table.playerDisCard(player, disCardList, disCardType);
        }
    }

    /**
     * 玩家过
     *
     * @param player    点击过玩家
     * @param isAuto    是否自动操作过
     */
    @Override
    public synchronized void playerPass(Player player, boolean isAuto) {
        FSGDTable table = this.tableMap.getOrDefault(player.data.tableId, null);
        if (table != null) {
            table.playerPass(player, isAuto);
        }
    }

    /**
     * 获取队友
     *
     * @param playerId
     * @return
     */
    private Struct.TeammateData getTeammate(String playerId) {
        Struct.TeammateData data = new Struct.TeammateData();
        for (Map.Entry<Integer, Struct. SettlementData> entry : this.data.result.entrySet()) {
            if (entry.getValue() != null && !StringUtils.isEmpty(entry.getValue().playerId)) {
                Player tmPlayer = this.actors.getOrDefault(entry.getValue().playerId, null);
                if (tmPlayer != null && tmPlayer.getId().equals(playerId)) {
                    data.player = tmPlayer;
                    data.rank = entry.getKey();
                }
            }
        }
        return data;
    }

    /**
     * 升级
     */
    private synchronized void upgrade() {
        StringBuilder builder = new StringBuilder();

        // 头游
        Struct.SettlementData winPlayerData = this.data.result.getOrDefault(1, null);
        if (winPlayerData == null || StringUtils.isEmpty(winPlayerData.playerId)) {
            return;
        }
        Player winPlayer = this.actors.getOrDefault(winPlayerData.playerId, null);
        if (winPlayer == null) {
            return;
        }

        // 末游
        Player p4 = null;

        // 原级牌点数
        int oldLevelPoint = this.data.curLevelPoint;

        // 2人房
        if (this.data.maxNum == 2) {
            Struct.SettlementData loserPlayerData = this.data.result.getOrDefault(2, null);
            if (loserPlayerData == null || StringUtils.isEmpty(loserPlayerData.playerId)) {
                return;
            }
            p4 = this.actors.getOrDefault(loserPlayerData.playerId, null);
            if (p4 == null) {
                return;
            }
            builder.append("1游玩家: ").append(winPlayer.data.name)
                    .append(" 升3级, ");
            // 设置我方玩家级牌点数展示
            int myIndex = Math.min(winPlayer.data.levelIndex + 3, GDUtils.levelPoint.length - 1);
            winPlayer.data.levelIndex = myIndex;
            winPlayer.data.levelPointView[0] = GDUtils.levelPoint[myIndex];
            // 设置对方级牌点数展示
            p4.data.levelPointView[1] = winPlayer.data.levelPointView[0];
        }

        // 4人房
        if (this.data.maxNum == 4) {
            // 级牌下标
            int index = 0;
            // 头游搭档数据
            Struct.TeammateData teammateData = this.getTeammate(winPlayer.data.teammateId);
            switch (teammateData.getRank()) {
                case 2:
                    // 搭档2游, 升3级
                    index = Math.min(winPlayer.data.levelIndex + 3, GDUtils.levelPoint.length - 1);
                    builder.append("1游玩家: ").append(winPlayer.data.name)
                            .append(", 搭档玩家: ").append(teammateData.player.data.name)
                            .append(" 2游, 升3级");
                    break;
                case 3:
                    // 搭档3游, 升2级
                    index = Math.min(winPlayer.data.levelIndex + 2, GDUtils.levelPoint.length - 1);
                    builder.append("1游玩家: ").append(winPlayer.data.name)
                            .append(", 搭档玩家: ").append(teammateData.player.data.name)
                            .append(" 3游, 升2级");
                    break;
                case 4:
                    // 搭档4游, 升1级
                    index = Math.min(winPlayer.data.levelIndex + 1, GDUtils.levelPoint.length - 1);
                    builder.append("1游玩家: ").append(winPlayer.data.name)
                            .append(", 搭档玩家: ").append(teammateData.player.data.name)
                            .append(" 4游, 升1级");
                    break;
            }

            // 设置头游玩家级牌点数展示
            winPlayer.data.levelIndex = index;
            winPlayer.data.levelPointView[0] = GDUtils.levelPoint[index];
            // 设置我方玩家级牌点数展示
            teammateData.player.data.levelPointView[0] = GDUtils.levelPoint[index];
            // 设置对方级牌点数展示
            for (Map.Entry<Integer, Struct.SettlementData> entry1 : this.data.result.entrySet()) {
                if (!entry1.getValue().playerId.equals(winPlayer.getId())
                        && !entry1.getValue().playerId.equals(teammateData.player.getId())) {
                    Player tmPlayer = this.actors.getOrDefault(entry1.getValue().playerId, null);
                    if (tmPlayer != null) {
                        tmPlayer.data.levelPointView[1] = GDUtils.levelPoint[index];
                    }
                }
            }
        }
        Log.info("升级...{}, 原级牌点数: {}, 当前级牌点数: {}", builder.toString(), oldLevelPoint, this.data.curLevelPoint);
    }

    /**
     * 牌局结算
     */
    private synchronized void settlement() {
        Log.debug("牌局结算...roomId : {}", this.data.roomId);
        // 广播结算
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null) {
                // 结算信息
                Map<String, Object> data = this.roomToMessage(tmPlayer, this.data.currentPlayer);
                data.put("state", 3);
                tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
            }
        }
        RedisUtil.del("roomGambit:"+this.data.roomId);
    }

    /**
     * 游戏结束
     */
    private synchronized void gameOver() {
        this.state = 3;
        this.time = 0;
        this.flag = false;
        Log.debug("牌局结束...roomId : {}", this.data.roomId);
    }

    /**
     * 下一局重置房间数据
     */
    public void resetNextRoom() {
        Log.debug("下一局重置玩家数据...");
        for (Player player : this.data.playerMap) {
            if (player != null) {
                player.data.state = 0;
                player.data.remainCards = new int[0];
                player.data.discardList = new int[0];
                player.data.disCardType = null;
                player.data.tributeCard = 0;
                player.data.tributeGetCard = 0;
                player.data.isChangePos = false;
            }
        }

        Log.debug("下一局重置桌子数据...");
        this.data.currentPlayer = null;
        this.data.otherOpPlayerList.clear();
        this.data.curDisCardList = new int[]{};
        this.data.curDisCardType = null;
        this.data.disCardPlayer = null;
        this.data.passList.clear();
        this.data.tributeState = 0;
        this.data.tributeCardMap.clear();
        this.data.cardList.clear();
        // 根据配置清理参数
        if (!this.data.isTribute) {
            this.data.result.clear();
            this.data.tributeList.clear();
            this.data.towKingList.clear();
        }

        // 初始化牌桌
        this.initCardList();
    }

    /**
     * 清理桌子信息
     */
    @Override
    public void resetRoom() {
        Log.debug("清理桌子数据...");
        this.data.init();
    }

    /**
     * 房间信息转消息
     *
     * @param player 本人
     * @param opPlayer 当前操作玩家
     * @return
     */
    @Override
    public Map<String, Object> roomToMessage(Player player, Player opPlayer) {
        Map<String, Object> data = new HashMap<>();
        data.put("roomId", this.data.roomId);
        data.put("type", this.data.type);
        data.put("gameType", this.data.gameType);
        data.put("ownerId", this.data.ownerId);
        data.put("state", this.state);
        data.put("time", this.time);

        data.put("maxRound", this.data.maxRound);
        data.put("curRound", this.data.curRound);
        data.put("zhuangPos", this.data.zhuangPos);
        // 当前级牌点数
        data.put("curLevelPoint", this.data.curLevelPoint);
        // 是否有进还贡
        data.put("isTribute", this.data.isTribute);
        data.put("opId", opPlayer != null ? opPlayer.getId() : "");
        data.put("opPos", opPlayer != null ? opPlayer.data.pos : -1);
        data.put("curDiscardList", this.data.curDisCardList);
        data.put("curDisCardType", this.data.curDisCardType);
        data.put("curDiscardPlayer", this.data.disCardPlayer != null ? this.data.disCardPlayer.getId() : "");
        data.put("curDiscardSex", this.data.disCardPlayer != null ? this.data.disCardPlayer.data.sex : 0);
        data.put("curPassList", this.data.passList);
        data.put("roomIp",this.data.roomIp);
        data.put("myInfo", this.playerRoomToMessage(player, true));
        List<Map<String, Object>> otherInfo = new ArrayList<>();
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                otherInfo.add(this.playerRoomToMessage(tmPlayer, false));
            }
        }
        data.put("otherInfo", otherInfo);
        // 增加结算玩家信息
        if (!this.data.result.isEmpty()) {
            Map<Integer, Object> settlementInfo = new HashMap<>();
            for (Map.Entry<Integer, Struct.SettlementData> entry : this.data.result.entrySet()) {
                Map<String, Object> settlementData = new HashMap<>();
                settlementData.put("playerId", entry.getValue().playerId);
                settlementInfo.put(entry.getKey(), settlementData);
            }
            data.put("settlementInfo", settlementInfo);
        }
        return data;
    }

    /**
     * 玩家信息转消息
     *
     * @param flag 是否显示手牌
     * @return
     */
    public Map<String, Object> playerRoomToMessage(Player player, boolean flag) {
        Map<String, Object> data = new HashMap<>();
        data.put("id", player.getId());
        data.put("name", player.data.name);
        data.put("avatar", "https://img95.699pic.com/element/40109/0194.png_300.png");
        data.put("sex", player.data.sex);
        data.put("pos", player.data.pos);
        data.put("state", player.data.state);
        data.put("isAuto", player.data.isAuto);
        // 转蛋积分-累计
        data.put("accumulatedPoints", player.data.accumulatedPoints);
        // 队友Id
        data.put("teammateId", player.data.teammateId);

        // 玩家牌数据
        Map<String, Object> cardInfo = new HashMap<>();
        cardInfo.put("remainCards", flag? player.data.remainCards : "");
        cardInfo.put("remainCardsNum", player.data.remainCards.length);
        cardInfo.put("discardList", player.data.discardList);
        cardInfo.put("disCardType", player.data.disCardType);
        cardInfo.put("tributeGetCard", player.data.tributeGetCard);
        data.put("cardInfo", cardInfo);
        return data;
    }
}
