package com.incubator.game.room;

import com.alibaba.fastjson2.JSONObject;
import com.incubator.common.log4j.Log;
import com.incubator.common.util.RandomUtil;
import com.incubator.game.data.po.FSGDTablePO;
import com.incubator.game.player.Player;
import com.incubator.game.util.CardUtils;
import com.incubator.game.util.GDUtils;
import com.incubator.game.util.Struct;
import com.incubator.message.proto.CommonProto;
import org.apache.commons.lang3.StringUtils;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 * 复式掼蛋桌子实体
 *
 * @author johnc
 */
public class FSGDTable {

    public int tableId;

    public FSGDTablePO data;

    public FSGDRoom room;

    /** 桌子角色 [key:玩家id, value:玩家对象]**/
    public Map<String, Player> actors = new ConcurrentHashMap<>();

    /** 激活标记 **/
    private boolean isActive;
    /** 检测标记 **/
    public boolean flag;
    /** 时间秒数 **/
    public int time;
    /** 房间状态(0等待 1开局 2进行中 3已结束) **/
    public int state;
    /** 并发锁 **/
    public final Lock lock = new ReentrantLock();
    /** 复式掼蛋2桌任务主线程 **/
    private ScheduledExecutorService scheduler;

    public FSGDTable(FSGDRoom room, int tableId) {
        this.tableId = tableId;
        this.room = room;
        this.scheduler = Executors.newSingleThreadScheduledExecutor();
    }

    public void init(JSONObject jsonData) {
        // todo 默认配置
        this.data = new FSGDTablePO();
        this.data.maxNum = 4;
        this.data.maxRound = this.room.data.maxRound;
        this.data.playerMap = new Player[this.data.maxNum];
        // 是否有进还贡规则
        this.data.isTribute = this.room.data.isTribute;
        // 随机级牌点数
        this.data.curLevelPoint = GDUtils.levelPoint[this.room.data.curLevelIndex];
        // 初始化牌库
        this.initCardList();
    }

    /**
     * 开启房间主线程任务
     */
    public void startTableTask() {
        this.isActive = true;
        this.scheduler.scheduleAtFixedRate(() -> {
            if (this.isActive) {
                this.doUpdate();
            }
        }, 0, 1, TimeUnit.SECONDS);
    }

    /**
     * 玩家加入桌子
     */
    public synchronized void joinTable(Player player) {
        if (player == null) {
            return;
        }
        if (this.actors.size() >= this.data.maxNum) {
            return; // 房间已满
        }

        // 玩家绑定房间
        player.data.roomId = this.room.data.roomId;
        player.data.tableId = this.tableId;

        // 加入房间角色缓存
        this.actors.put(player.getId(), player);

        // 设置座位号
        for (int i = 0; i < this.data.playerMap.length; i++) {
            Player tmPlayer = this.data.playerMap[i];
            if (tmPlayer == null) {
                player.data.pos = i;
                this.data.playerMap[i] = player;
                break;
            }
        }
    }

    /** 玩家离开桌子 **/
    public synchronized void leaveTable(Player player) {
        if (player == null) {
            return;
        }
        // 清理房间数据
        this.actors.remove(player.getId());
        this.data.playerMap[player.data.pos] = null;
        // 清理玩家数据
        player.data.pos = 0;
        player.data.roomId = 0;
        player.data.tableId = 0;
        player.data.teamId = 0;
    }

    /**
     * 初始化牌库
     */
    public void initCardList() {
        for (int i = 0; i < 2; i++) {
            this.data.cardList.addAll(Arrays.asList(GDUtils.values));
            Collections.shuffle(this.data.cardList);
        }
    }

    /**
     * 检测是否可以开始 3秒倒计时开始
     *
     * @return
     */
    public boolean checkReadyStart() {
        if (!this.isStart() || this.state != 0) {
            return false;
        }

        // 广播3秒倒计时
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null) {
                Map<String, Object> data = this.roomToMessage(tmPlayer, null);
                data.put("state", 1);
                data.put("time", 0);
                tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
            }
        }

        return true;
    }

    /**
     * 是否玩家都已准备,可开始
     *
     * @return
     */
    public boolean isStart() {
        int cnt = 0;
        for (Player player : this.actors.values()) {
            if (player != null && player.data.state == 1) {
                cnt += 1;
            }
        }
        return cnt >= this.data.maxNum;
    }

    /**
     * 给玩家发牌，每人27张
     */
    public synchronized void doSendCard() {
        for (Player player : this.actors.values()) {
            if (player != null) {
                // 设置用户状态
                player.data.state = 2;
                // 设置级牌点数
                if (this.data.curRound == 1) {
                    // 我方级数
                    player.data.levelPointView[0] = GDUtils.levelPoint[0];
                    // 对方级数
                    player.data.levelPointView[1] = GDUtils.levelPoint[0];
                }
                // 设置队友位置
                if (this.data.maxNum >= 4) {
                    Integer[] nextPos = this.getNextPos(player.data.pos);
                    Player tmpPlayer = this.data.playerMap[nextPos[1]];
                    if (tmpPlayer != null) {
                        player.data.teammateId = tmpPlayer.getId();
                    }
                }
                // 发牌
                int[] remainCards = this.room.cardMap.getOrDefault(player.data.pos, new int[]{});
                if (remainCards != null && remainCards.length > 0) {
                    player.data.remainCards = remainCards;
                }
            }
        }
    }

    /**
     * 主线程逻辑
     */
    public void doUpdate() {
        synchronized (this) {
//            Log.debug("检测: 经典掼蛋房间主线程执行...");
            switch (this.state) {
                case 0:
                    // 等待状态
                    if (this.checkReadyStart()) {
                        this.state = 1;
                        this.flag = false;
                        this.time = 0;
                    }

                    // 120秒未开始直接解散
//                    if (this.time >= 120 && this.type != 1) {
//                        // 广播
//                        for (Player tmPlayer : this.actors.values()) {
//                            if (tmPlayer != null) {
//                                tmPlayer.receive(CommonProto.Cmd.DisbandRoom_VALUE, new HashMap<>());
//                            }
//                        }
//                        this.destroy();
//                    }

                    // 房间没人直接解散
                    if (this.actors.isEmpty()) {
                        this.destroy();
                        return;
                    }
                    break;
                case 1:
                    // 开局
                    if (!this.flag) {
                        //开始
                        this.doStart();
                        Log.debug("检测: 房间号:{}, 房间人数:{}, 当前局数:{}, 状态:开局...VS动画...", this.room.data.roomId, this.actors.size(), this.data.curRound);
                    }
                    break;
                case 2:
                    // 进行中
                    if (!this.flag) {
                        // 检查进贡是否完成
                        if (this.data.tributeState == 1 && this.checkTributeStart()) {
                            // 切换还贡状态
                            this.doChangeBackTribute();
                            this.time = 0;
                        }

                        // 检查还贡是否完成
                        if (this.data.tributeState == 2 && this.checkBackTributeStart()) {
                            // 进还贡换牌
                            this.doChangeTributeCard();
                            this.flag = true;
                            this.time = 0;
                        }

                        // 80秒强制操作进还贡
//                        if (this.time >= 80) {
//                            this.doTribute();
//                            this.flag = true;
//                            this.time = 0;
//                        }
                    } else {
                        // 20秒未操作,强制过
//                        if (this.time >= 20) {
//                            Log.debug("强制操作过...");
//                            this.playerPass(this.data.currentPlayer, true);
//                            this.flag = true;
//                            this.time = 0;
//                        }
                    }
                    break;
                case 3:
                    // 结束
                    if (!this.flag) {
                        this.flag = true;
                        this.time = 0;
                        Log.debug("游戏结束: 房间号 : {}, 房间人数 : {}, 当前局数 : {}", this.room.data.roomId, this.actors.size(), this.data.curRound);
                    } else {
                        // 等待另一桌结束

//                        // 小于最大局数,切等待状态
//                        if (this.data.curRound < this.data.maxRound) {
//                            this.resetNextRoom();
//                            this.state = 0;
//                            this.flag = false;
//                            this.time = 0;
//                        } else {
//                            // 120秒未开始直接解散
////                            if (this.time >= 120 && this.type != 1) {
////                                // 广播
////                                for (Player tmPlayer : this.actors.values()) {
////                                    if (tmPlayer != null) {
////                                        tmPlayer.receive(CommonProto.Cmd.DisbandRoom_VALUE, new HashMap<>());
////                                    }
////                                }
////                                this.destroy();
////                            }
//                        }
                    }
                    break;
            }

            this.time += 1;
        }
    }

    /**
     * 游戏开局
     */
    public synchronized void doStart() {
        try {
            this.data.curRound += 1;
            // 随机定庄
            this.data.zhuangPos = RandomUtil.getIndex(this.data.maxNum);
            // 发牌
            this.doSendCard();
            // 进还贡数据
            Map<String, Object> tributeInfo = null;
            if (this.data.isTribute) {
                tributeInfo = new HashMap<>();
                this.checkTributeInfo();
                // 定庄
                if (this.isTwoKing()) {
                    // 定庄 头游先出
                    Player player = this.data.result.getOrDefault(1, null);
                    if (player != null) {
                        this.data.zhuangPos = player.data.pos;
                    }
                    this.data.result.clear();
                }
                if (this.data.towKingList.size() >= 2) {
                    this.data.tributeState = 3;
                } else {
                    if (!this.data.tributeList.isEmpty() && !this.isTwoKing() && this.data.tributeState == 0) {
                        this.data.tributeState = 1;
                    }
                }
                tributeInfo.put("tributeState", this.data.tributeState);
                tributeInfo.put("tributeList", this.data.tributeList);
                tributeInfo.put("twoKingInfo", this.data.towKingList);
                tributeInfo.put("tributeCardMap", this.data.tributeCardMap);
            }
            // 设置当前操作玩家
            this.data.currentPlayer = this.data.playerMap[this.data.zhuangPos];
            // 广播
            for (Player tmPlayer : this.actors.values()) {
                if (tmPlayer != null) {
                    Map<String, Object> data = this.roomToMessage(tmPlayer, this.data.currentPlayer);
                    data.put("zhuangPos", this.data.zhuangPos);
                    if (this.data.isTribute) {
                        data.put("zhuangPos", !this.data.tributeList.isEmpty() && !this.isTwoKing()? -1 : this.data.zhuangPos);
                    }
                    data.put("tributeInfo", tributeInfo);
                    data.put("state", 2);
                    data.put("time", !this.data.tributeList.isEmpty()? 40 : 20);
                    Log.debug("玩家 : {}, 手牌 : {}", tmPlayer.data.name, tmPlayer.data.remainCards);
                    tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
                }
            }

            // 是否有进还贡数据 或 抗贡
            if (this.data.isTribute && !this.data.tributeList.isEmpty() && !this.isTwoKing()) {
                this.state = 2;
                this.flag = false;
                this.time = 0;
            } else {
                this.state = 2;
                this.flag = true;
                this.time = 0;
            }
        } catch (Exception e) {
            Log.error("===游戏开局出错...", e);
        }
    }

    /**
     * 开局检测是否进贡、还贡
     *
     * @return
     */
    public synchronized void checkTributeInfo() {
        // 头游
        Player player = this.data.result.get(1);
        if (player == null) {
            return;
        }
        // 末游
        Player p4 = null;

        // 2人房
        if (this.data.maxNum == 2) {
            p4 = this.data.result.getOrDefault(2, null);
            if (p4 == null) {
                return;
            }
            // 单贡
            this.data.tributeList.add(new String[]{p4.getId() , player.getId()});
        }

        // 4人房
        if (this.data.maxNum == 4) {
            // 是否双贡
            boolean isDoubleTribute = false;
            // 头游搭档数据
            Struct.TeammateData teammateData = this.getTeammate(player.data.teammateId);
            if (teammateData.getRank() == 2) {
                // 进贡, 双下
                isDoubleTribute = true;
            }
            // 进还贡
            p4 = this.data.result.getOrDefault(4, null);
            if (p4 != null) {
                this.data.tributeList.add(new String[]{p4.getId() , player.getId()});
            }
            // 双贡
            if (isDoubleTribute) {
                Player p3 = this.data.result.getOrDefault(3, null);
                if (p3 != null) {
                    this.data.tributeList.add(new String[]{p3.getId(), teammateData.player.getId()});
                }
            }
        }

        // 双贡情况
        if (this.data.tributeList.size() >= 2) {
            // 1游[0:进贡者, 1:还贡者]
            String[] tribute1 = this.data.tributeList.get(0);
            // 2游[0:进贡者, 1:还贡者]
            String[] tribute2 = this.data.tributeList.get(1);

            // 1游
            Player player1 = this.actors.getOrDefault(tribute1[1], null);
            // 1游进贡者
            Player tributePlayer1 = this.actors.getOrDefault(tribute1[0], null);
            // 2游
            Player player2 = this.actors.getOrDefault(tribute2[1], null);
            // 2游进贡者
            Player tributePlayer2 = this.actors.getOrDefault(tribute2[0], null);

            if (player1 == null || tributePlayer1 == null || player2 == null || tributePlayer2 == null) {
                return;
            }
            // 1游进贡者最大的手牌
            int player1MaxCard = GDUtils.getTributeMaxCard(tributePlayer1.data.remainCards, this.data.curLevelPoint);
            // 2游进贡者最大手牌
            int player2MaxCard = GDUtils.getTributeMaxCard(tributePlayer2.data.remainCards, this.data.curLevelPoint);

            // 1游取牌点大的进贡者
            if (player2MaxCard > player1MaxCard) {
                tribute1[0] = tributePlayer2.getId();
                tribute2[0] = tributePlayer1.getId();
                // 设置庄家
                this.data.zhuangPos = tributePlayer2.data.pos;
            } else if (player2MaxCard == player1MaxCard) {
                // 牌点相同按顺时针进贡
                // 设置1游进贡者为1游下家
                Player nextPlayer = this.getNextPlayer(player1);
                if (nextPlayer != null) {
                    tribute1[0] = nextPlayer.getId();
                }
                //设置2游进贡者为2游下家
                nextPlayer = this.getNextPlayer(player2);
                if (nextPlayer != null) {
                    tribute2[0] = nextPlayer.getId();
                }
                // 设置庄家
                this.data.zhuangPos = tributePlayer2.data.pos;
            } else {
                // 设置庄家
                this.data.zhuangPos = tributePlayer1.data.pos;
            }
        }

        // 单贡情况
        if (this.data.tributeList.size() == 1) {
            // 1游[0:进贡者, 1:还贡者]
            String[] tribute1 = this.data.tributeList.get(0);
            // 1游
            Player player1 = this.actors.getOrDefault(tribute1[1], null);
            // 1游进贡者
            Player tributePlayer1 = this.actors.getOrDefault(tribute1[0], null);
            if (player1 == null || tributePlayer1 == null) {
                return;
            }
            // 设置庄家
            this.data.zhuangPos = tributePlayer1.data.pos;
        }

        // 抗贡
        for (String[] tributeId : this.data.tributeList) {
            // 进贡者
            Player tributePlayer = this.actors.getOrDefault(tributeId[0], null);
            if (tributePlayer != null) {
                // 统计手牌数量
                Map<Integer, Integer> cardMap = CardUtils.getCardMap(tributePlayer.data.remainCards);
                if (!cardMap.isEmpty()) {
                    int num = cardMap.getOrDefault(GDUtils.BIGKING, 0);
                    if (num > 0 && !this.data.towKingList.contains(tributeId[0])) {
                        // 记录信息
                        this.data.towKingList.add(tributeId[0]);
                    }
                }
            }
        }

        if (this.data.towKingList.size() < 2) {
            this.data.towKingList.clear();
        }
    }

    /**
     * 检测双王抗贡
     *
     * @return
     */
    public boolean isTwoKing() {
        return this.data.towKingList.size() >= 2;
    }

    /**
     * 检测进贡是否完成
     *
     * @return
     */
    public synchronized boolean checkTributeStart() {
//        Log.debug("检测进贡是否完成...");
        int cnt = 0;
        for (String[] playerIds : this.data.tributeList) {
            if (!StringUtils.isEmpty(playerIds[0]) && !StringUtils.isEmpty(playerIds[1])) {
                // 进贡者
                Player tributePlayer = this.actors.getOrDefault(playerIds[0], null);
                if (tributePlayer != null && tributePlayer.data.tributeCard > 0) {
                    cnt += 1;
                }
            }
        }
        return cnt == this.data.tributeList.size();
    }

    /**
     * 检测还贡是否完成
     *
     * @return
     */
    public synchronized boolean checkBackTributeStart() {
//        Log.debug("检测还贡是否完成...");
        int cnt = 0;
        for (String[] playerIds : this.data.tributeList) {
            if (!StringUtils.isEmpty(playerIds[0]) && !StringUtils.isEmpty(playerIds[1])) {
                // 还贡者
                Player backPlayer = this.actors.getOrDefault(playerIds[1], null);
                if (backPlayer != null && backPlayer.data.tributeCard > 0) {
                    cnt += 1;
                }
            }
        }
        return cnt == this.data.tributeList.size();
    }

    /**
     * 切换还贡状态
     */
    public synchronized void doChangeBackTribute() {
        this.data.tributeState = 2;
        // 进还贡数据
        Map<String, Object> tributeInfo = new HashMap<>();
        tributeInfo.put("tributeState", this.data.tributeState);
        tributeInfo.put("tributeList", this.data.tributeList);
        tributeInfo.put("twoKingInfo", this.data.towKingList);
        tributeInfo.put("tributeCardMap", this.data.tributeCardMap);

        // 广播
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null) {
                Map<String, Object> data = this.roomToMessage(tmPlayer, this.data.currentPlayer);
                data.put("zhuangPos", !this.data.tributeList.isEmpty() && !this.isTwoKing()? -1 : this.data.zhuangPos);
                data.put("tributeInfo", tributeInfo);
                data.put("state", 2);
                data.put("time", !this.data.tributeList.isEmpty()? 40 : 20);
                Log.debug("玩家 : {}, 手牌 : {}", tmPlayer.data.name, tmPlayer.data.remainCards);
                tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
            }
        }
    }

    /**
     * 强制进贡、还贡
     */
    public synchronized void doTribute() {
        Log.debug("强制进贡、还贡...");
        // 强制选牌
        for (String[] playerIds : this.data.tributeList) {
            if (!StringUtils.isEmpty(playerIds[0]) && !StringUtils.isEmpty(playerIds[1])) {
                // 进贡者
                Player tributePlayer = this.actors.getOrDefault(playerIds[0], null);
                // 还贡者
                Player backPlayer = this.actors.getOrDefault(playerIds[1], null);
                if (tributePlayer == null || backPlayer == null) {
                    continue;
                }
                // 进贡者选牌
                int maxCard = GDUtils.getTributeMaxCard(tributePlayer.data.remainCards, this.data.curLevelPoint);
                this.data.tributeCardMap.put(tributePlayer.getId(), maxCard);
                // 还贡者选牌
                int tributeCard = 0;
                for (Integer card : backPlayer.data.remainCards) {
                    byte low = (byte) (card & 0xf);
                    // 过滤级牌
                    if (low == this.data.curLevelPoint) {
                        continue;
                    }
                    if (low <= 10) {
                        tributeCard = card;
                        break;
                    }
                }
                this.data.tributeCardMap.put(backPlayer.getId(), tributeCard);
                Log.debug("强制进还贡...进贡者 : {}, 选牌 : {}, 还贡者 : {}, 选牌 : {}",
                        tributePlayer.data.name, maxCard, backPlayer.data.name, tributeCard);
            }
        }
        // 换牌
        this.doChangeTributeCard();
    }

    /**
     * 进还贡换牌
     */
    public synchronized void doChangeTributeCard() {
        if (this.data.tributeList.isEmpty()) {
            return;
        }
        Log.debug("进还贡换牌...");
        int tributeCard = 0;
        int backCard = 0;
        for (String[] playerIds : this.data.tributeList) {
            if (!StringUtils.isEmpty(playerIds[0]) && !StringUtils.isEmpty(playerIds[1])) {
                // 进贡者
                Player tributePlayer = this.actors.getOrDefault(playerIds[0], null);
                // 还贡者
                Player backPlayer = this.actors.getOrDefault(playerIds[1], null);
                if (tributePlayer == null || backPlayer == null) {
                    continue;
                }
                // 移除进贡者手牌
                tributeCard = this.data.tributeCardMap.getOrDefault(tributePlayer.getId(), 0);
                if (tributeCard > 0) {
                    // 还贡者添加牌
                    backPlayer.data.remainCards = CardUtils.add(backPlayer.data.remainCards, tributeCard);
                    backPlayer.data.tributeGetCard = tributeCard;
                }

                // 移除还贡者手牌
                backCard = this.data.tributeCardMap.getOrDefault(backPlayer.getId(), 0);
                if (backCard > 0) {
                    // 进贡者添加牌
                    tributePlayer.data.remainCards = CardUtils.add(tributePlayer.data.remainCards, backCard);
                    tributePlayer.data.tributeGetCard = backCard;
                }
                // 更新数据
                this.actors.put(tributePlayer.getId(), tributePlayer);
                this.actors.put(backPlayer.getId(), backPlayer);
            }
        }
        // 清理数据
        this.data.tributeState = 0;
        this.data.result.clear();
        this.data.tributeList.clear();
        // 进还贡数据
        Map<String, Object> tributeInfo = new HashMap<>();
        tributeInfo.put("tributeList", this.data.tributeList);
        tributeInfo.put("twoKingInfo", this.data.towKingList);
        tributeInfo.put("tributeState", this.data.tributeState);
        // 进还贡的选牌map
        tributeInfo.put("tributeCardMap", this.data.tributeCardMap);
        // 广播
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null) {
                Map<String, Object> data = this.roomToMessage(tmPlayer, this.data.currentPlayer);
                data.put("tributeInfo", tributeInfo);
                data.put("state", 2);
                data.put("time", 20);
                Log.debug("进还贡换牌...玩家 : {}, 手牌 : {}", tmPlayer.data.name, tmPlayer.data.remainCards);
                tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
            }
        }
    }

    /**
     * 根据当前玩家位置, 获取下一个玩家位置
     *
     * @param pos 当前操作玩家的位置
     * @return
     */
    public Integer[] getNextPos(int pos) {
        Integer[] nextpos = null;

        // 2人房
        if (this.data.maxNum == 2) {
            switch (pos) {
                case 1:
                    nextpos = new Integer[] { 0 };
                    break;
                case 0:
                    nextpos = new Integer[] { 1 };
                    break;
            }
        }

        // 4人房
        if (this.data.maxNum == 4) {
            switch (pos) {
                case 3:
                    nextpos = new Integer[] { 0, 1, 2 };
                    break;
                case 2:
                    nextpos = new Integer[] { 3, 0, 1 };
                    break;
                case 1:
                    nextpos = new Integer[] { 2, 3, 0 };
                    break;
                case 0:
                    nextpos = new Integer[] { 1, 2, 3 };
                    break;
            }
        }
        return nextpos;
    }

    /**
     * 根据当前玩家位置, 获取下一个可操作玩家
     *
     * @param player 当前操作玩家
     * @return
     */
    public Player getNextPlayer(Player player) {
        Player nextPlayer = null;
        Integer[] nextPos = this.getNextPos(player.data.pos);
        for (Integer index : nextPos) {
            Player temPlayer = this.data.playerMap[index];
            if (temPlayer.data.state == 2 && !temPlayer.getId().equals(player.getId())) {
                nextPlayer = temPlayer;
                break;
            }
        }
        return nextPlayer;
    }

    /**
     * 玩家出牌
     *
     * @param player    出牌玩家
     * @param disCardList   打出的牌
     * @param disCardType   打出的牌型
     */
    public synchronized void playerDisCard(Player player, int[] disCardList, GDUtils.CardType disCardType) {
        // 是否自己回合
        if (!this.data.currentPlayer.getId().equals(player.getId())) {
            return;
        }
        // 出的牌是否都在手上
        if (!CardUtils.contains(player.data.remainCards, disCardList)) {
            return;
        }
        Log.info("出牌...玩家ID:{}, 位置:{}, 出牌牌型:{}, 出牌:{}, 手牌大小:{}, 手牌信息: {}",
                player.getId(), player.data.pos, disCardType, Arrays.toString(disCardList),
                player.data.remainCards.length, Arrays.toString(player.data.remainCards));
        // 从手牌中移除
        for (Integer card : disCardList) {
            player.data.remainCards = CardUtils.remove(player.data.remainCards, card);
        }
        // 设置出牌堆
        this.data.curDisCardList = disCardList;
        this.data.curDisCardType = disCardType;
        // 设置当前出牌的玩家
        this.data.disCardPlayer = player;

        // 清空过玩家集合
        this.data.passList.removeIf(id -> player.getId().equals(id));

        // 设置接风玩家
        this.data.jieFengPlayer = null;
        // 设置下一个操作玩家
        this.data.currentPlayer = this.getNextPlayer(player);
        // 出完牌设置结算数据
        if (player.data.remainCards.length == 0) {
            player.data.state = 3;
            this.data.result.put(this.data.result.size() + 1, player);
        }
        // 设置其他可操作玩家集合
        this.data.otherOpPlayerList.clear();
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null && !tmPlayer.getId().equals(player.getId())) {
                if (tmPlayer.data.state == 2) {
                    if (!this.data.otherOpPlayerList.contains(tmPlayer.getId())) {
                        this.data.otherOpPlayerList.add(tmPlayer.getId());
                    }
                }
            }
        }

        // 广播其他3人
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                Map<String, Object> data = this.roomToMessage(tmPlayer, this.data.currentPlayer);
                data.put("time", 20);
                tmPlayer.receive(CommonProto.Cmd.OtherPlayCards_VALUE, data);
            }
        }

        boolean isGameOver = false;

        // 4人房2人一队手牌出完,直接结算
        if (this.data.result.size() == 2) {
            // 1游
            Player player1 = this.data.result.get(1);
            if (player1 == null) {
                return;
            }
            // 2游
            Player temPlayer = this.data.result.getOrDefault(2, null);
            if (temPlayer == null) {
                return;
            }
            // 1游2游是否一队
            if (player1.data.teammateId.equals(temPlayer.getId())) {
                // 已结算的用户id
                List<String> list = new ArrayList<>();
                for (Player tmPlayer : this.data.result.values()) {
                    if (tmPlayer != null) {
                        list.add(tmPlayer.getId());
                    }
                }
                // 补充3游4游玩家
                for (Player tmPlayer : this.data.playerMap) {
                    if (tmPlayer != null) {
                        tmPlayer.data.state = 3;
                        // 补充最后一个玩家
                        if (!list.contains(tmPlayer.getId())) {
                            this.data.result.put(this.data.result.size() + 1, tmPlayer);
                        }
                    }
                }
                isGameOver = true;
            }
        }

        // 4人房间3人手牌出完，直接结算
        if (this.data.result.size() >= this.data.maxNum - 1) {
            // 已结算的用户id
            List<String> list = new ArrayList<>();
            for (Player tmPlayer : this.data.result.values()) {
                if (tmPlayer != null) {
                    list.add(tmPlayer.getId());
                }
            }
            for (Player tmPlayer : this.data.playerMap) {
                if (tmPlayer != null) {
                    tmPlayer.data.state = 3;
                    // 补充最后一个玩家
                    if (!list.contains(tmPlayer.getId())) {
                        this.data.result.put(this.data.result.size() + 1, tmPlayer);
                    }
                }
            }

            isGameOver = true;
        }

        if (isGameOver) {
            // 升级
            this.upgrade();
            // 结算
            this.settlement();
            // 结束
            this.gameOver();
        }
    }

    /**
     * 玩家过
     *
     * @param player    点击过玩家
     * @param isAuto    是否自动操作过
     */
    public synchronized void playerPass(Player player, boolean isAuto) {
        // 当前可操作玩家是否自己
        if (!this.data.currentPlayer.getId().equals(player.getId())) {
            return;
        }
        Log.info("过...玩家ID:{}, 位置:{}, 手牌大小:{}, 手牌信息: {}", player.getId(), player.data.pos, player.data.remainCards.length, Arrays.toString(player.data.remainCards));

        boolean isClear = false;

        // 清掉自己出牌堆
        player.data.discardList = new int[0];

        // 添加过玩家列表
        if (!this.data.passList.contains(player.getId())) {
            this.data.passList.add(player.getId());
        }
        // 设置其他可操作玩家集合
        this.data.otherOpPlayerList.removeIf(id -> player.getId().equals(id));

        // 4人房3人不出，清空出牌堆
        if (this.data.otherOpPlayerList.isEmpty()) {
            // 清空出牌堆
            this.data.curDisCardList = new int[0];
            this.data.curDisCardType = null;

            for (Player p : this.actors.values()) {
                if (p != null && p.data.discardList.length > 0) {
                    p.data.discardList = new int[0];
                }
            }
            // 所有玩家都点过
            if (!this.data.passList.isEmpty()) {
                // 清空过玩家集合
                this.data.passList.clear();
                // 设置接风标识
                if (this.data.maxNum == 4 && this.data.disCardPlayer.data.remainCards.length == 0) {
                    this.data.jieFengPlayer = this.actors.getOrDefault(this.data.disCardPlayer.data.teammateId, null);
                }
            }
            isClear = true;
        }

        // 判断接风
        if (isClear && this.data.jieFengPlayer != null) {
            this.data.currentPlayer = this.actors.getOrDefault(this.data.jieFengPlayer.getId(), null);
            this.data.disCardPlayer = null;
        } else {
            // 设置下一个操作玩家
            this.data.currentPlayer = this.getNextPlayer(player);
        }

        // 当前过玩家的下家清掉出牌堆
        Integer[] nextPos = this.getNextPos(player.data.pos);
        Player nextPlayer = this.data.playerMap[nextPos[0]];
        if (nextPlayer != null && nextPlayer.data.discardList.length > 0) {
            nextPlayer.data.discardList = new int[0];
        }

        // 广播其他3人
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                Map<String, Object> data = this.roomToMessage(tmPlayer, this.data.currentPlayer);
                // 接风玩家
                data.put("jieFengPlayer", isClear && this.data.jieFengPlayer != null? this.data.jieFengPlayer.getId() : "");
                data.put("time", 20);
                tmPlayer.receive(CommonProto.Cmd.OtherPlayCards_VALUE, data);
            }
        }
    }

    /**
     * 获取队友
     *
     * @param playerId
     * @return
     */
    private Struct.TeammateData getTeammate(String playerId) {
        Struct.TeammateData data = new Struct.TeammateData();
        for (Map.Entry<Integer, Player> entry : this.data.result.entrySet()) {
            if (entry.getValue().getId().equals(playerId)) {
                data.player = entry.getValue();
                data.rank = entry.getKey();
            }
        }
        return data;
    }

    /**
     * 升级
     */
    private synchronized void upgrade() {
        StringBuilder builder = new StringBuilder();

        // 头游
        Player player = this.data.result.get(1);
        if (player == null) {
            return;
        }

        // 末游
        Player p4 = null;

        // 原级牌点数
        int oldLevelPoint = this.data.curLevelPoint;

        // 2人房
        if (this.data.maxNum == 2) {
            p4 = this.data.result.getOrDefault(2, null);
            if (p4 == null) {
                return;
            }
            builder.append("1游玩家: ").append(player.data.name)
                    .append(" 升3级, ");
            // 设置我方玩家级牌点数展示
            int myIndex = Math.min(player.data.levelIndex + 3, GDUtils.levelPoint.length - 1);
            player.data.levelIndex = myIndex;
            player.data.levelPointView[0] = GDUtils.levelPoint[myIndex];
            // 设置对方级牌点数展示
            p4.data.levelPointView[1] = player.data.levelPointView[0];
            // 设置全局当前级牌点数
            this.data.curLevelPoint = Math.max(player.data.levelPointView[0], player.data.levelPointView[1]);
        }

        // 4人房
        if (this.data.maxNum == 4) {
            // 级牌下标
            int index = 0;
            // 头游搭档数据
            Struct.TeammateData teammateData = this.getTeammate(player.data.teammateId);
            switch (teammateData.getRank()) {
                case 2:
                    // 搭档2游, 升3级
                    index = Math.min(player.data.levelIndex + 3, GDUtils.levelPoint.length - 1);
                    builder.append("1游玩家: ").append(player.data.name)
                            .append(", 搭档玩家: ").append(teammateData.player.data.name)
                            .append(" 2游, 升3级");
                    break;
                case 3:
                    // 搭档3游, 升2级
                    index = Math.min(player.data.levelIndex + 2, GDUtils.levelPoint.length - 1);
                    builder.append("1游玩家: ").append(player.data.name)
                            .append(", 搭档玩家: ").append(teammateData.player.data.name)
                            .append(" 3游, 升2级");
                    break;
                case 4:
                    // 搭档4游, 升1级
                    index = Math.min(player.data.levelIndex + 1, GDUtils.levelPoint.length - 1);
                    builder.append("1游玩家: ").append(player.data.name)
                            .append(", 搭档玩家: ").append(teammateData.player.data.name)
                            .append(" 4游, 升1级");
                    break;
            }

            // 设置头游玩家级牌点数展示
            player.data.levelIndex = index;
            player.data.levelPointView[0] = GDUtils.levelPoint[index];
            // 设置我方玩家级牌点数展示
            teammateData.player.data.levelPointView[0] = GDUtils.levelPoint[index];
            // 设置对方级牌点数展示
            for (Map.Entry<Integer, Player> entry1 : this.data.result.entrySet()) {
                if (!entry1.getValue().getId().equals(player.getId())
                        && !entry1.getValue().getId().equals(teammateData.player.getId())) {
                    entry1.getValue().data.levelPointView[1] = GDUtils.levelPoint[index];
                }
            }
            // 设置全局当前级牌点数
            this.data.curLevelPoint = Math.max(player.data.levelPointView[0], player.data.levelPointView[1]);
        }
        Log.info("升级...{}, 原级牌点数: {}, 当前级牌点数: {}", builder.toString(), oldLevelPoint, this.data.curLevelPoint);
    }

    /**
     * 牌局结算
     */
    private synchronized void settlement() {
        Log.debug("牌局结算...roomId : {}", this.room.data.roomId);
        // 广播结算
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null) {
                // 结算信息
                Map<String, Object> data = this.roomToMessage(tmPlayer, this.data.currentPlayer);
                data.put("state", 3);
                tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
            }
        }
        // 记录游戏回放
        Map<String, Object> data = this.roomToPlayBack(this.data.currentPlayer);
        data.put("state", 3);
    }

    /**
     * 游戏结束
     */
    private synchronized void gameOver() {
        this.state = 3;
        this.time = 60;
        this.flag = false;
        Log.debug("牌局结束...roomId : {}", this.room.data.roomId);
    }

    /**
     * 下一局重置房间数据
     */
    public void resetNextRoom() {
        Log.debug("下一局重置玩家数据...");
        for (Player player : this.data.playerMap) {
            if (player != null) {
                player.data.state = 0;
                player.data.remainCards = new int[0];
                player.data.discardList = new int[0];
                player.data.disCardType = null;
                player.data.tributeCard = 0;
                player.data.tributeGetCard = 0;
                player.data.isChangePos = false;
            }
        }

        Log.debug("下一局重置桌子数据...");
        this.data.currentPlayer = null;
        this.data.otherOpPlayerList.clear();
        this.data.curDisCardList = new int[]{};
        this.data.curDisCardType = null;
        this.data.disCardPlayer = null;
        this.data.passList.clear();
        this.data.tributeState = 0;
        this.data.tributeCardMap.clear();
        this.data.cardList.clear();
        // 根据配置清理参数
        if (!this.data.isTribute) {
            this.data.result.clear();
            this.data.tributeList.clear();
            this.data.towKingList.clear();
        }

        // 初始化牌桌
        this.initCardList();
    }

    /**
     * 清理桌子信息
     */
    public void resetTable() {
        Log.debug("清理桌子数据...");
        this.data.init();
    }

    /**
     * 销毁房间，释放资源
     */
    public void destroy() {
        this.isActive = false;
        this.scheduler.shutdownNow();
        this.scheduler = null;
        // 重置桌子数据
        this.resetTable();
        Log.debug("Room: {} Table has been destroyed.", this.room.data.roomId);
    }

    /**
     * 桌子信息转消息
     *
     * @param player 本人
     * @param opPlayer 当前操作玩家
     * @return
     */
    public Map<String, Object> roomToMessage(Player player, Player opPlayer) {
        Map<String, Object> data = new HashMap<>();
        data.put("roomId", this.room.data.roomId);
        data.put("type", this.room.data.type);
        data.put("gameType", this.room.data.gameType);
        data.put("ownerId", this.room.data.ownerId);
        data.put("state", this.state);
        data.put("time", this.time);

        data.put("maxRound", this.data.maxRound);
        data.put("curRound", this.data.curRound);
        data.put("zhuangPos", this.data.zhuangPos);
        // 当前级牌点数
        data.put("curLevelPoint", this.data.curLevelPoint);
        // 是否有进还贡
        data.put("isTribute", this.data.isTribute);
        data.put("opId", opPlayer != null ? opPlayer.getId() : "");
        data.put("opPos", opPlayer != null ? opPlayer.data.pos : -1);
        data.put("curDiscardList", this.data.curDisCardList);
        data.put("curDisCardType", this.data.curDisCardType);
        data.put("curDiscardPlayer", this.data.disCardPlayer != null ? this.data.disCardPlayer.getId() : "");
        data.put("curDiscardSex", this.data.disCardPlayer != null ? this.data.disCardPlayer.data.sex : 0);
        data.put("curPassList", this.data.passList);

        data.put("myInfo", this.playerRoomToMessage(player, true));
        List<Map<String, Object>> otherInfo = new ArrayList<>();
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                otherInfo.add(this.playerRoomToMessage(tmPlayer, false));
            }
        }
        // 增加结算玩家信息
        if (!this.data.result.isEmpty()) {
            Map<Integer, String> settlementData = new HashMap<>();
            for (Map.Entry<Integer, Player> entry : this.data.result.entrySet()) {
                settlementData.put(entry.getKey(), entry.getValue().getId());
            }
            data.put("settlementInfo", settlementData);
        }
        data.put("otherInfo", otherInfo);
        return data;
    }

    /**
     * 玩家信息转消息
     *
     * @param flag 是否显示手牌
     * @return
     */
    public Map<String, Object> playerRoomToMessage(Player player, boolean flag) {
        Map<String, Object> data = new HashMap<>();
        data.put("id", player.getId());
        data.put("name", player.data.name);
        data.put("avatar", "https://img95.699pic.com/element/40109/0194.png_300.png");
        data.put("sex", player.data.sex);
        data.put("pos", player.data.pos);
        data.put("state", player.data.state);
        data.put("isAuto", player.data.isAuto);
        // 转蛋积分-累计
        data.put("accumulatedPoints", player.data.accumulatedPoints);
        // 队友Id
        data.put("teammateId", player.data.teammateId);

        // 玩家牌数据
        Map<String, Object> cardInfo = new HashMap<>();
        cardInfo.put("remainCards", flag? player.data.remainCards : "");
        cardInfo.put("remainCardsNum", player.data.remainCards.length);
        cardInfo.put("discardList", player.data.discardList);
        cardInfo.put("disCardType", player.data.disCardType);
        cardInfo.put("tributeGetCard", player.data.tributeGetCard);
        data.put("cardInfo", cardInfo);
        return data;
    }

    /**
     * 房间信息转游戏回放
     *
     * @param opPlayer 当前操作玩家
     * @return
     */
    public Map<String, Object> roomToPlayBack(Player opPlayer) {
        Map<String, Object> data = new HashMap<>();
        data.put("roomId", this.room.data.roomId);
        data.put("type", this.room.data.type);
        data.put("gameType", this.room.data.gameType);
        data.put("ownerId", this.room.data.ownerId);
        data.put("state", this.state);
        data.put("time", this.time);

        data.put("maxRound", this.data.maxRound);
        data.put("curRound", this.data.curRound);
        data.put("zhuangPos", this.data.zhuangPos);
        // 当前级牌点数
        data.put("curLevelPoint", this.data.curLevelPoint);
        // 是否有进还贡
        data.put("isTribute", this.data.isTribute);
        data.put("opId", opPlayer != null ? opPlayer.getId() : "");
        data.put("opPos", opPlayer != null ? opPlayer.data.pos : -1);
        data.put("curDiscardList", this.data.curDisCardList);
        data.put("curDisCardType", this.data.curDisCardType);
        data.put("curDiscardPlayer", this.data.disCardPlayer != null ? this.data.disCardPlayer.getId() : "");
        data.put("curDiscardSex", this.data.disCardPlayer != null ? this.data.disCardPlayer.data.sex : 0);
        data.put("curPassList", this.data.passList);
        // 玩家信息
        if (!this.actors.isEmpty()) {
            List<Map<String, Object>> playerInfo = new ArrayList<>();
            for (Player tmPlayer : this.actors.values()) {
                if (tmPlayer != null) {
                    playerInfo.add(playerRoomToPlayBack(tmPlayer));
                }
            }
            data.put("playerInfo", playerInfo);
        }
        // 增加结算玩家信息
        if (!this.data.result.isEmpty()) {
            Map<String, String> settlementData = new HashMap<>();
            for (Map.Entry<Integer, Player> entry : this.data.result.entrySet()) {
                settlementData.put(entry.getKey().toString(), entry.getValue().getId());
            }
            data.put("settlementInfo", settlementData);
        }
        return data;
    }

    /**
     * 玩家信息转消息
     *
     * @param player 玩家
     * @return
     */
    public Map<String, Object> playerRoomToPlayBack(Player player) {
        Map<String, Object> data = new HashMap<>();
        data.put("id", player.getId());
        data.put("name", player.data.name);
        data.put("avatar", "https://img95.699pic.com/element/40109/0194.png_300.png");
        data.put("sex", player.data.sex);
        data.put("pos", player.data.pos);
        data.put("state", player.data.state);
        data.put("isAuto", player.data.isAuto);
        // 级数显示 [我方级数, 对方级数]
        data.put("levelPointView", player.data.levelPointView);
        // 队友Id
        data.put("teammateId", player.data.teammateId);

        // 玩家牌数据
        Map<String, Object> cardInfo = new HashMap<>();
        cardInfo.put("remainCards", player.data.remainCards);
        cardInfo.put("remainCardsNum", player.data.remainCards.length);
        cardInfo.put("discardList", player.data.discardList);
        cardInfo.put("disCardType", player.data.disCardType);
        cardInfo.put("tributeGetCard", player.data.tributeGetCard);
        data.put("cardInfo", cardInfo);
        return data;
    }
}
