package com.incubator.game.room;

import java.util.Map;
import java.util.Queue;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 房间对象池
 */
public final class RoomPool {

    /** 房间对象池，按玩法类型存储不同队列 **/
    private final Map<Integer, Queue<Room>> roomPools;
    /** 最大空闲房间数量 **/
    private final int maxIdleRooms;
    /** 房间清理线程 **/
    private final ScheduledExecutorService cleaner;

    /**
     * 初始化
     *
     * @param maxIdleRooms 最大空闲房间数量
     */
    public RoomPool(int maxIdleRooms) {
        this.roomPools = new ConcurrentHashMap<>();
        this.maxIdleRooms = maxIdleRooms;
        this.cleaner = Executors.newSingleThreadScheduledExecutor();
        // 定期清理空闲房间
        this.cleaner.scheduleAtFixedRate(this::cleanIdleRooms, 10, 30, TimeUnit.SECONDS);
    }

    /**
     * 获取房间对象
     *
     * @param gameType  玩法类型 1:掼蛋 2:转蛋
     * @return 房间对象
     */
    public Room acquireRoom(int gameType) {
        // 获取对应类型的房间队列
        Queue<Room> pool = this.roomPools.computeIfAbsent(gameType, k -> new ConcurrentLinkedQueue<>());
        // 从队列中取房间
        Room room = pool.poll();
        if (room == null) {
            // 创建指定类型的房间
            room = RoomFactory.createRoom(gameType);
        }
        return room;
    }

    /**
     * 归还房间对象
     *
     * @param room 归还的房间
     */
    public void releaseRoom(Room room) {
        if (room != null) {
            // 获取对应类型的房间队列
            Queue<Room> pool = this.roomPools.computeIfAbsent(room.data.gameType, k -> new ConcurrentLinkedQueue<>());
            pool.offer(room); // 放回队列中
        }
    }

    /**
     * 定期清理空闲房间
     */
    private void cleanIdleRooms() {
        for (Map.Entry<Integer, Queue<Room>> entry : this.roomPools.entrySet()) {
            Queue<Room> pool = entry.getValue();
            while (pool.size() > this.maxIdleRooms) {
                Room room = pool.poll();
                if (room != null) {
                    // 销毁房间，释放资源
                    room.destroy();
                }
            }
        }
    }

    /**
     * 关闭对象池
     */
    public void shutdown() {
        this.cleaner.shutdownNow();
        this.roomPools.values().forEach(pool -> {
            pool.forEach(Room::destroy);
            pool.clear();
        });
        this.roomPools.clear();
    }

    /**
     * 获取指定玩法类型的可用房间数量
     *
     * @param gameType 玩法类型
     * @return 可用房间数量
     */
    public int getAvailableRooms(int gameType) {
        Queue<Room> pool = this.roomPools.get(gameType);
        return pool == null ? 0 : pool.size();
    }

    /**
     * 获取所有玩法类型的房间总数量
     *
     * @return 总房间数量
     */
    public int getTotalAvailableRooms() {
        return this.roomPools.values().stream().mapToInt(Queue::size).sum();
    }
}
