package com.incubator.game.room;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson2.JSONObject;
import com.incubator.common.log4j.Log;
import com.incubator.common.util.RandomUtil;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.constant.WkServerStatus;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.player.Player;
import com.incubator.game.util.UtilCore;

import java.util.*;
import java.util.concurrent.*;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 * 房间 管理类
 */
public class RoomService {

    private static RoomService instance;
    /** 房间对象池 **/
    public RoomPool roomPool;
    /** 并发锁 **/
    public final Lock lock = new ReentrantLock();
    /** 房间缓存 [key:房间id, value:房间实例] **/
    public Map<Long, Room> roomMap = new ConcurrentHashMap<>();

    public synchronized static RoomService getInstance() {
        if (instance == null) {
            instance = new RoomService();
        }
        return instance;
    }

    /**
     * 初始化
     *
     * @param maxIdleRooms 最大空闲房间数量
     */
    public void init(int maxIdleRooms) {
        Log.info("初始化房间服务...");
        this.roomPool = new RoomPool(maxIdleRooms);
    }

    /**
     * 每日刷新
     */
    public void refreshNewDay() {

    }

    /**
     * 生成唯一6位数房间号
     *
     * @return
     */
    public long randomRoomId() {
        long roomNumber;
        // 循环生成，直到找到一个唯一的房间号
        do {
            roomNumber = RandomUtil.randomSixDigits();
        } while (this.roomMap.containsKey(roomNumber));
        return roomNumber;
    }

    /**
     * 生成唯一7位数赛事房间号
     *
     * @return
     */
    public long randomRoomIdByContest() {
        long roomNumber;
        // 循环生成，直到找到一个唯一的房间号
        do {
            roomNumber = RandomUtil.randomSevenDigits();
        } while (this.roomMap.containsKey(roomNumber));
        return roomNumber;
    }

    /**
     * 获取房间
     *
     * @param roomId
     * @return
     */
    public Room getRoom(long roomId) {
        return this.roomMap.getOrDefault(roomId, null);
    }

    /**
     * 获取玩家房间
     *
     * @param playerId
     * @return
     */
    public Room getRoomByPlayerId(String playerId) {
        for (Room room : this.roomMap.values()) {
            if (room != null && room.actors != null && room.actors.containsKey(playerId)) {
                return room;
            }
        }
        return null;
    }

    /**
     * 创建房间
     *
     * @param player    创建房间玩家
     * @param gameType  玩法类型 1:掼蛋 2:转蛋
     * @param jsonData  房间规则数据
     * @return
     */
    public synchronized Room creatRoom(Player player, int gameType, JSONObject jsonData) {
        // 从对象池获取房间
        Room room = this.roomPool.acquireRoom(gameType);
        // 初始化房间
        room.init(player, jsonData);
        // 房主加入房间
        room.joinRoom(player);
        //绑定服务器ip
//        bindServerIp(room);
        // 加入管理映射
        this.roomMap.put(room.data.roomId, room);
        //RedisUtil.saveRoomMapToRedis(RedisKeyConstant.WK_ROOM_MAP ,roomMap);
        return room;
    }

    /**
     * 绑定服务器ip
     * @param room
     */
    private void bindServerIp(Room room){
        //查询所有服务器在线人数
        HashMap<String, WkServerStatus> wkServes = RedisUtil.getWkToRedis(RedisKeyConstant.WK_SERVE);
        //判断房间类型
        int type = room.data.type;
        ArrayList<WkServerStatus> wkServerStatuses = new ArrayList<>();
        for (Map.Entry<String, WkServerStatus> entry : wkServes.entrySet()) {
            String type1 = entry.getValue().getType();
            if (type1.equals(String.valueOf(type))){
                wkServerStatuses.add(entry.getValue());
            }
        }
        room.data.roomIp = selectLeastPopulatedServer(wkServerStatuses).get().getName();
        System.out.println(" room.data.roomIp ======================================================>>>>>>>>>>>" +  room.data.roomIp);
    }

    /**
     * 选择在线人数最少的服务器。
     *
     * @return 最少在线人数的服务器，如果不存在任何在线服务器，则返回 Optional.empty()。
     */
    public static Optional<WkServerStatus> selectLeastPopulatedServer(List<WkServerStatus> fetchOnlineServers){
        // 使用流式 API 找到在线人数最少的服务器
        return fetchOnlineServers.stream()
                .min(Comparator.comparingInt(WkServerStatus::getNum));
    }
}
